-- The layer1 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/07/01

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer1 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  CO9 : in std_logic_vector(3 downto 0); 
  pix_r : in std_logic_vector(7 downto 0); 
  pix_g : in std_logic_vector(7 downto 0); 
  pix_b : in std_logic_vector(7 downto 0); 
  CnnStart : in std_logic; 
  in1ChSel : in std_logic_vector(1 downto 0); 
  ReluAll : out std_logic_vector(79 downto 0)); 
end conv_layer1;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer1 is

component ROM_W1
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(4 downto 0);
          DATA : Out std_logic_vector(143 downto 0) );
end component;

component dotpro
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(7 downto 0); 
  MatRomData : in std_logic_vector(8 downto 0);
  CO9is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(19 downto 0) );  
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal W1Data : std_logic_vector(143 downto 0);
signal W1Addr : std_logic_vector(4 downto 0);

signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(8 downto 0);
signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15: std_logic_vector(8 downto 0);

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09: std_logic_vector(19 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15: std_logic_vector(19 downto 0);

signal BIAS_00, BIAS_01, BIAS_02, BIAS_03, BIAS_04, BIAS_05, BIAS_06, BIAS_07, BIAS_08, BIAS_09: std_logic_vector(19 downto 0);
signal BIAS_10, BIAS_11, BIAS_12, BIAS_13, BIAS_14, BIAS_15: std_logic_vector(19 downto 0);

signal plusBias_00, plusBias_01, plusBias_02, plusBias_03, plusBias_04, plusBias_05, plusBias_06, plusBias_07, plusBias_08, plusBias_09: std_logic_vector(19 downto 0);
signal plusBias_10, plusBias_11, plusBias_12, plusBias_13, plusBias_14, plusBias_15: std_logic_vector(19 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09: std_logic_vector(8 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15: std_logic_vector(8 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09: std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15: std_logic_vector(4 downto 0);

signal CO9is0 : std_logic;
signal pix_rgb : std_logic_vector(7 downto 0);
signal in1ChSel_dly : std_logic_vector(1 downto 0);
signal CnnStart_dly1, CnnStart_dly2 : std_logic;
signal CO9_p0, CO9_p9, CO9_p18 : std_logic_vector(4 downto 0);

begin

coeff_00 <= W1Data(143 downto 135) ;
coeff_01 <= W1Data(134 downto 126) ;
coeff_02 <= W1Data(125 downto 117) ;
coeff_03 <= W1Data(116 downto 108) ;
coeff_04 <= W1Data(107 downto 99) ;
coeff_05 <= W1Data(98 downto 90) ;
coeff_06 <= W1Data(89 downto 81) ;
coeff_07 <= W1Data(80 downto 72) ;
coeff_08 <= W1Data(71 downto 63) ;
coeff_09 <= W1Data(62 downto 54) ;
coeff_10 <= W1Data(53 downto 45) ;
coeff_11 <= W1Data(44 downto 36) ;
coeff_12 <= W1Data(35 downto 27) ;
coeff_13 <= W1Data(26 downto 18) ;
coeff_14 <= W1Data(17 downto 9) ;
coeff_15 <= W1Data(8 downto 0) ;

-------------------------------------------------------------------------------
  CO9_p0 <= "0" & CO9;
  CO9_p9 <= ("0" & CO9) + 9;
  CO9_p18 <= ("0" & CO9) + 18;

  W1Addr <= CO9_p0 when in1ChSel = "10" else CO9_p9 when in1ChSel = "01" else CO9_p18;
  ----CO9is0 <= '1' when (CO9 = "1000" and in1ChSel_dly = "11") or CnnStart_dly2 = '1' else '0';
  CO9is0 <= '1' when (CO9 = "0001" and in1ChSel = "00") else '0';

--------------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                CnnStart_dly2 <= '0';
                CnnStart_dly1 <= '0';
        elsif( CLK'event and CLK = '1' ) then
                CnnStart_dly2 <= CnnStart_dly1;
                CnnStart_dly1 <= CnnStart;
        end if;
end process;

-------------------------------------------------------------------------------
ROM_W1_inst : ROM_W1 port map (
  CLK => CLK,  
  ADDR => W1Addr,
  DATA => W1Data );

-------------------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    in1ChSel_dly <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    in1ChSel_dly <= in1ChSel;
  end if;
end process;

  pix_rgb <= pix_r when in1ChSel_dly = "00" else 
             pix_g when in1ChSel_dly = "01" else
             pix_b when in1ChSel_dly = "10" else
             (others => '0');

-------------------------------------------------------------------------------
dotpro_00 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00 );

-------------------------------------------------------------------------------
dotpro_01 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01 );

-------------------------------------------------------------------------------
dotpro_02 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02 );

-------------------------------------------------------------------------------
dotpro_03 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03 );

-------------------------------------------------------------------------------
dotpro_04 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_04,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04 );

-------------------------------------------------------------------------------
dotpro_05 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_05,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05 );

-------------------------------------------------------------------------------
dotpro_06 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_06,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06 );

-------------------------------------------------------------------------------
dotpro_07 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_07,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07 );

-------------------------------------------------------------------------------
dotpro_08 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_08,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08 );

-------------------------------------------------------------------------------
dotpro_09 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_09,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09 );

-------------------------------------------------------------------------------
dotpro_10 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_10,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10 );

-------------------------------------------------------------------------------
dotpro_11 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_11,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11 );

-------------------------------------------------------------------------------
dotpro_12 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_12,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12 );

-------------------------------------------------------------------------------
dotpro_13 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_13,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13 );

-------------------------------------------------------------------------------
dotpro_14 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_14,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14 );

-------------------------------------------------------------------------------
dotpro_15 : dotpro port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => pix_rgb,
  MatRomData => coeff_15,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15 );

-------------------------------------------------------------------------------
  BIAS_00 <= "11111111010110001110";
  BIAS_01 <= "11111111111010100111";
  BIAS_02 <= "00000000000010100100";
  BIAS_03 <= "11111111110111011100";
  BIAS_04 <= "11111111001110011000";
  BIAS_05 <= "11111111110001011100";
  BIAS_06 <= "11111111001111011101";
  BIAS_07 <= "11111111011110001110";
  BIAS_08 <= "11111111110111010100";
  BIAS_09 <= "00000000010101011111";
  BIAS_10 <= "11111111110101101011";
  BIAS_11 <= "00000000100111111000";
  BIAS_12 <= "11111110101101111001";
  BIAS_13 <= "00000000110010001011";
  BIAS_14 <= "11111111101011100100";
  BIAS_15 <= "11111111101111101010";

  plusBias_00 <= mac_00 + BIAS_00;
  plusBias_01 <= mac_01 + BIAS_01;
  plusBias_02 <= mac_02 + BIAS_02;
  plusBias_03 <= mac_03 + BIAS_03;
  plusBias_04 <= mac_04 + BIAS_04;
  plusBias_05 <= mac_05 + BIAS_05;
  plusBias_06 <= mac_06 + BIAS_06;
  plusBias_07 <= mac_07 + BIAS_07;
  plusBias_08 <= mac_08 + BIAS_08;
  plusBias_09 <= mac_09 + BIAS_09;
  plusBias_10 <= mac_10 + BIAS_10;
  plusBias_11 <= mac_11 + BIAS_11;
  plusBias_12 <= mac_12 + BIAS_12;
  plusBias_13 <= mac_13 + BIAS_13;
  plusBias_14 <= mac_14 + BIAS_14;
  plusBias_15 <= mac_15 + BIAS_15;

  truncate_00 <= plusBias_00(19 downto 11);
  truncate_01 <= plusBias_01(19 downto 11);
  truncate_02 <= plusBias_02(19 downto 11);
  truncate_03 <= plusBias_03(19 downto 11);
  truncate_04 <= plusBias_04(19 downto 11);
  truncate_05 <= plusBias_05(19 downto 11);
  truncate_06 <= plusBias_06(19 downto 11);
  truncate_07 <= plusBias_07(19 downto 11);
  truncate_08 <= plusBias_08(19 downto 11);
  truncate_09 <= plusBias_09(19 downto 11);
  truncate_10 <= plusBias_10(19 downto 11);
  truncate_11 <= plusBias_11(19 downto 11);
  truncate_12 <= plusBias_12(19 downto 11);
  truncate_13 <= plusBias_13(19 downto 11);
  truncate_14 <= plusBias_14(19 downto 11);
  truncate_15 <= plusBias_15(19 downto 11);

  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

ReluAll <= Relu_00 & Relu_01
		& Relu_02 & Relu_03
		& Relu_04 & Relu_05
		& Relu_06 & Relu_07
		& Relu_08 & Relu_09
		& Relu_10 & Relu_11
		& Relu_12 & Relu_13
		& Relu_14 & Relu_15;

end RTL;

