-- The layer2 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/07/01

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer2 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  CO9 : in std_logic_vector(3 downto 0); 
  CO28 : in std_logic_vector(4 downto 0); 
  P_WR_N : in std_logic; 
  CnnStart : IN std_logic;
  in1ChSel : in std_logic_vector(1 downto 0);
  LAY2IN : in std_logic_vector(79 downto 0); 
  LAY2OUT : out std_logic_vector(159 downto 0)); 
end conv_layer2;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer2 is

component ROM_W2_00
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(5 downto 0);
          DATA : Out std_logic_vector(95 downto 0) );
end component;

component ROM_W2_01
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(5 downto 0);
          DATA : Out std_logic_vector(95 downto 0) );
end component;

component ROM_W2_02
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(5 downto 0);
          DATA : Out std_logic_vector(95 downto 0) );
end component;

component ROM_W2_03
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(5 downto 0);
          DATA : Out std_logic_vector(95 downto 0) );
end component;

component dotpro_w2
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(15 downto 0) );  
end component;

component pooling_lay2
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(159 downto 0);
  WR_N : in std_logic;
  HCO28 : in std_logic_vector(4 downto 0);
  Poolout : Out std_logic_vector(159 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal W2Data_00, W2Data_01, W2Data_02, W2Data_03 : std_logic_vector(95 downto 0);

signal W2Addr : std_logic_vector(5 downto 0);

signal coeff_00_00, coeff_00_01, coeff_00_02, coeff_00_03: std_logic_vector(2 downto 0);
signal coeff_01_00, coeff_01_01, coeff_01_02, coeff_01_03: std_logic_vector(2 downto 0);
signal coeff_02_00, coeff_02_01, coeff_02_02, coeff_02_03: std_logic_vector(2 downto 0);
signal coeff_03_00, coeff_03_01, coeff_03_02, coeff_03_03: std_logic_vector(2 downto 0);
signal coeff_04_00, coeff_04_01, coeff_04_02, coeff_04_03: std_logic_vector(2 downto 0);
signal coeff_05_00, coeff_05_01, coeff_05_02, coeff_05_03: std_logic_vector(2 downto 0);
signal coeff_06_00, coeff_06_01, coeff_06_02, coeff_06_03: std_logic_vector(2 downto 0);
signal coeff_07_00, coeff_07_01, coeff_07_02, coeff_07_03: std_logic_vector(2 downto 0);
signal coeff_08_00, coeff_08_01, coeff_08_02, coeff_08_03: std_logic_vector(2 downto 0);
signal coeff_09_00, coeff_09_01, coeff_09_02, coeff_09_03: std_logic_vector(2 downto 0);
signal coeff_10_00, coeff_10_01, coeff_10_02, coeff_10_03: std_logic_vector(2 downto 0);
signal coeff_11_00, coeff_11_01, coeff_11_02, coeff_11_03: std_logic_vector(2 downto 0);
signal coeff_12_00, coeff_12_01, coeff_12_02, coeff_12_03: std_logic_vector(2 downto 0);
signal coeff_13_00, coeff_13_01, coeff_13_02, coeff_13_03: std_logic_vector(2 downto 0);
signal coeff_14_00, coeff_14_01, coeff_14_02, coeff_14_03: std_logic_vector(2 downto 0);
signal coeff_15_00, coeff_15_01, coeff_15_02, coeff_15_03: std_logic_vector(2 downto 0);
signal coeff_16_00, coeff_16_01, coeff_16_02, coeff_16_03: std_logic_vector(2 downto 0);
signal coeff_17_00, coeff_17_01, coeff_17_02, coeff_17_03: std_logic_vector(2 downto 0);
signal coeff_18_00, coeff_18_01, coeff_18_02, coeff_18_03: std_logic_vector(2 downto 0);
signal coeff_19_00, coeff_19_01, coeff_19_02, coeff_19_03: std_logic_vector(2 downto 0);
signal coeff_20_00, coeff_20_01, coeff_20_02, coeff_20_03: std_logic_vector(2 downto 0);
signal coeff_21_00, coeff_21_01, coeff_21_02, coeff_21_03: std_logic_vector(2 downto 0);
signal coeff_22_00, coeff_22_01, coeff_22_02, coeff_22_03: std_logic_vector(2 downto 0);
signal coeff_23_00, coeff_23_01, coeff_23_02, coeff_23_03: std_logic_vector(2 downto 0);
signal coeff_24_00, coeff_24_01, coeff_24_02, coeff_24_03: std_logic_vector(2 downto 0);
signal coeff_25_00, coeff_25_01, coeff_25_02, coeff_25_03: std_logic_vector(2 downto 0);
signal coeff_26_00, coeff_26_01, coeff_26_02, coeff_26_03: std_logic_vector(2 downto 0);
signal coeff_27_00, coeff_27_01, coeff_27_02, coeff_27_03: std_logic_vector(2 downto 0);
signal coeff_28_00, coeff_28_01, coeff_28_02, coeff_28_03: std_logic_vector(2 downto 0);
signal coeff_29_00, coeff_29_01, coeff_29_02, coeff_29_03: std_logic_vector(2 downto 0);
signal coeff_30_00, coeff_30_01, coeff_30_02, coeff_30_03: std_logic_vector(2 downto 0);
signal coeff_31_00, coeff_31_01, coeff_31_02, coeff_31_03: std_logic_vector(2 downto 0);


signal mac_00_00, mac_00_01, mac_00_02, mac_00_03: std_logic_vector(15 downto 0);
signal mac_01_00, mac_01_01, mac_01_02, mac_01_03: std_logic_vector(15 downto 0);
signal mac_02_00, mac_02_01, mac_02_02, mac_02_03: std_logic_vector(15 downto 0);
signal mac_03_00, mac_03_01, mac_03_02, mac_03_03: std_logic_vector(15 downto 0);
signal mac_04_00, mac_04_01, mac_04_02, mac_04_03: std_logic_vector(15 downto 0);
signal mac_05_00, mac_05_01, mac_05_02, mac_05_03: std_logic_vector(15 downto 0);
signal mac_06_00, mac_06_01, mac_06_02, mac_06_03: std_logic_vector(15 downto 0);
signal mac_07_00, mac_07_01, mac_07_02, mac_07_03: std_logic_vector(15 downto 0);
signal mac_08_00, mac_08_01, mac_08_02, mac_08_03: std_logic_vector(15 downto 0);
signal mac_09_00, mac_09_01, mac_09_02, mac_09_03: std_logic_vector(15 downto 0);
signal mac_10_00, mac_10_01, mac_10_02, mac_10_03: std_logic_vector(15 downto 0);
signal mac_11_00, mac_11_01, mac_11_02, mac_11_03: std_logic_vector(15 downto 0);
signal mac_12_00, mac_12_01, mac_12_02, mac_12_03: std_logic_vector(15 downto 0);
signal mac_13_00, mac_13_01, mac_13_02, mac_13_03: std_logic_vector(15 downto 0);
signal mac_14_00, mac_14_01, mac_14_02, mac_14_03: std_logic_vector(15 downto 0);
signal mac_15_00, mac_15_01, mac_15_02, mac_15_03: std_logic_vector(15 downto 0);
signal mac_16_00, mac_16_01, mac_16_02, mac_16_03: std_logic_vector(15 downto 0);
signal mac_17_00, mac_17_01, mac_17_02, mac_17_03: std_logic_vector(15 downto 0);
signal mac_18_00, mac_18_01, mac_18_02, mac_18_03: std_logic_vector(15 downto 0);
signal mac_19_00, mac_19_01, mac_19_02, mac_19_03: std_logic_vector(15 downto 0);
signal mac_20_00, mac_20_01, mac_20_02, mac_20_03: std_logic_vector(15 downto 0);
signal mac_21_00, mac_21_01, mac_21_02, mac_21_03: std_logic_vector(15 downto 0);
signal mac_22_00, mac_22_01, mac_22_02, mac_22_03: std_logic_vector(15 downto 0);
signal mac_23_00, mac_23_01, mac_23_02, mac_23_03: std_logic_vector(15 downto 0);
signal mac_24_00, mac_24_01, mac_24_02, mac_24_03: std_logic_vector(15 downto 0);
signal mac_25_00, mac_25_01, mac_25_02, mac_25_03: std_logic_vector(15 downto 0);
signal mac_26_00, mac_26_01, mac_26_02, mac_26_03: std_logic_vector(15 downto 0);
signal mac_27_00, mac_27_01, mac_27_02, mac_27_03: std_logic_vector(15 downto 0);
signal mac_28_00, mac_28_01, mac_28_02, mac_28_03: std_logic_vector(15 downto 0);
signal mac_29_00, mac_29_01, mac_29_02, mac_29_03: std_logic_vector(15 downto 0);
signal mac_30_00, mac_30_01, mac_30_02, mac_30_03: std_logic_vector(15 downto 0);
signal mac_31_00, mac_31_01, mac_31_02, mac_31_03: std_logic_vector(15 downto 0);

signal MacAll_00, MacAll_01, MacAll_02, MacAll_03, MacAll_04, MacAll_05, MacAll_06, MacAll_07, MacAll_08, MacAll_09: std_logic_vector(15 downto 0);
signal MacAll_10, MacAll_11, MacAll_12, MacAll_13, MacAll_14, MacAll_15, MacAll_16, MacAll_17, MacAll_18, MacAll_19: std_logic_vector(15 downto 0);
signal MacAll_20, MacAll_21, MacAll_22, MacAll_23, MacAll_24, MacAll_25, MacAll_26, MacAll_27, MacAll_28, MacAll_29: std_logic_vector(15 downto 0);
signal MacAll_30, MacAll_31: std_logic_vector(15 downto 0);

signal MacShift_00, MacShift_01, MacShift_02, MacShift_03, MacShift_04, MacShift_05, MacShift_06, MacShift_07, MacShift_08, MacShift_09: std_logic_vector(17 downto 0);
signal MacShift_10, MacShift_11, MacShift_12, MacShift_13, MacShift_14, MacShift_15, MacShift_16, MacShift_17, MacShift_18, MacShift_19: std_logic_vector(17 downto 0);
signal MacShift_20, MacShift_21, MacShift_22, MacShift_23, MacShift_24, MacShift_25, MacShift_26, MacShift_27, MacShift_28, MacShift_29: std_logic_vector(17 downto 0);
signal MacShift_30, MacShift_31: std_logic_vector(17 downto 0);

signal BIAS_00, BIAS_01, BIAS_02, BIAS_03, BIAS_04, BIAS_05, BIAS_06, BIAS_07, BIAS_08, BIAS_09: std_logic_vector(17 downto 0);
signal BIAS_10, BIAS_11, BIAS_12, BIAS_13, BIAS_14, BIAS_15, BIAS_16, BIAS_17, BIAS_18, BIAS_19: std_logic_vector(17 downto 0);
signal BIAS_20, BIAS_21, BIAS_22, BIAS_23, BIAS_24, BIAS_25, BIAS_26, BIAS_27, BIAS_28, BIAS_29: std_logic_vector(17 downto 0);
signal BIAS_30, BIAS_31: std_logic_vector(17 downto 0);

signal plusBias_00, plusBias_01, plusBias_02, plusBias_03, plusBias_04, plusBias_05, plusBias_06, plusBias_07, plusBias_08, plusBias_09: std_logic_vector(17 downto 0);
signal plusBias_10, plusBias_11, plusBias_12, plusBias_13, plusBias_14, plusBias_15, plusBias_16, plusBias_17, plusBias_18, plusBias_19: std_logic_vector(17 downto 0);
signal plusBias_20, plusBias_21, plusBias_22, plusBias_23, plusBias_24, plusBias_25, plusBias_26, plusBias_27, plusBias_28, plusBias_29: std_logic_vector(17 downto 0);
signal plusBias_30, plusBias_31: std_logic_vector(17 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09: std_logic_vector(9 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15, truncate_16, truncate_17, truncate_18, truncate_19: std_logic_vector(9 downto 0);
signal truncate_20, truncate_21, truncate_22, truncate_23, truncate_24, truncate_25, truncate_26, truncate_27, truncate_28, truncate_29: std_logic_vector(9 downto 0);
signal truncate_30, truncate_31: std_logic_vector(9 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09: std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15, Relu_16, Relu_17, Relu_18, Relu_19: std_logic_vector(4 downto 0);
signal Relu_20, Relu_21, Relu_22, Relu_23, Relu_24, Relu_25, Relu_26, Relu_27, Relu_28, Relu_29: std_logic_vector(4 downto 0);
signal Relu_30, Relu_31: std_logic_vector(4 downto 0);

signal ReluAll: std_logic_vector(159 downto 0);
signal Poolout: std_logic_vector(159 downto 0);
signal pool_00, pool_01, pool_02, pool_03: std_logic_vector(4 downto 0);

signal CO9is0 : std_logic;

signal inch_00, inch_01, inch_02, inch_03, inch_04, inch_05, inch_06, inch_07, inch_08, inch_09: std_logic_vector(4 downto 0);
signal inch_10, inch_11, inch_12, inch_13, inch_14, inch_15: std_logic_vector(4 downto 0);
signal CO9_p0, CO9_p9, CO9_p18, CO9_p27 : std_logic_vector(5 downto 0);
signal in1ChSel_dly : std_logic_vector(1 downto 0);
signal CnnStart_dly : std_logic;

begin

  coeff_00_00 <= W2Data_00( 95 downto 93 );
  coeff_00_01 <= W2Data_01( 95 downto 93 );
  coeff_00_02 <= W2Data_02( 95 downto 93 );
  coeff_00_03 <= W2Data_03( 95 downto 93 ); 


  coeff_01_00 <= W2Data_00( 92 downto 90 );
  coeff_01_01 <= W2Data_01( 92 downto 90 );
  coeff_01_02 <= W2Data_02( 92 downto 90 );
  coeff_01_03 <= W2Data_03( 92 downto 90 ); 


  coeff_02_00 <= W2Data_00( 89 downto 87 );
  coeff_02_01 <= W2Data_01( 89 downto 87 );
  coeff_02_02 <= W2Data_02( 89 downto 87 );
  coeff_02_03 <= W2Data_03( 89 downto 87 ); 


  coeff_03_00 <= W2Data_00( 86 downto 84 );
  coeff_03_01 <= W2Data_01( 86 downto 84 );
  coeff_03_02 <= W2Data_02( 86 downto 84 );
  coeff_03_03 <= W2Data_03( 86 downto 84 ); 


  coeff_04_00 <= W2Data_00( 83 downto 81 );
  coeff_04_01 <= W2Data_01( 83 downto 81 );
  coeff_04_02 <= W2Data_02( 83 downto 81 );
  coeff_04_03 <= W2Data_03( 83 downto 81 ); 


  coeff_05_00 <= W2Data_00( 80 downto 78 );
  coeff_05_01 <= W2Data_01( 80 downto 78 );
  coeff_05_02 <= W2Data_02( 80 downto 78 );
  coeff_05_03 <= W2Data_03( 80 downto 78 ); 


  coeff_06_00 <= W2Data_00( 77 downto 75 );
  coeff_06_01 <= W2Data_01( 77 downto 75 );
  coeff_06_02 <= W2Data_02( 77 downto 75 );
  coeff_06_03 <= W2Data_03( 77 downto 75 ); 


  coeff_07_00 <= W2Data_00( 74 downto 72 );
  coeff_07_01 <= W2Data_01( 74 downto 72 );
  coeff_07_02 <= W2Data_02( 74 downto 72 );
  coeff_07_03 <= W2Data_03( 74 downto 72 ); 


  coeff_08_00 <= W2Data_00( 71 downto 69 );
  coeff_08_01 <= W2Data_01( 71 downto 69 );
  coeff_08_02 <= W2Data_02( 71 downto 69 );
  coeff_08_03 <= W2Data_03( 71 downto 69 ); 


  coeff_09_00 <= W2Data_00( 68 downto 66 );
  coeff_09_01 <= W2Data_01( 68 downto 66 );
  coeff_09_02 <= W2Data_02( 68 downto 66 );
  coeff_09_03 <= W2Data_03( 68 downto 66 ); 


  coeff_10_00 <= W2Data_00( 65 downto 63 );
  coeff_10_01 <= W2Data_01( 65 downto 63 );
  coeff_10_02 <= W2Data_02( 65 downto 63 );
  coeff_10_03 <= W2Data_03( 65 downto 63 ); 


  coeff_11_00 <= W2Data_00( 62 downto 60 );
  coeff_11_01 <= W2Data_01( 62 downto 60 );
  coeff_11_02 <= W2Data_02( 62 downto 60 );
  coeff_11_03 <= W2Data_03( 62 downto 60 ); 


  coeff_12_00 <= W2Data_00( 59 downto 57 );
  coeff_12_01 <= W2Data_01( 59 downto 57 );
  coeff_12_02 <= W2Data_02( 59 downto 57 );
  coeff_12_03 <= W2Data_03( 59 downto 57 ); 


  coeff_13_00 <= W2Data_00( 56 downto 54 );
  coeff_13_01 <= W2Data_01( 56 downto 54 );
  coeff_13_02 <= W2Data_02( 56 downto 54 );
  coeff_13_03 <= W2Data_03( 56 downto 54 ); 


  coeff_14_00 <= W2Data_00( 53 downto 51 );
  coeff_14_01 <= W2Data_01( 53 downto 51 );
  coeff_14_02 <= W2Data_02( 53 downto 51 );
  coeff_14_03 <= W2Data_03( 53 downto 51 ); 


  coeff_15_00 <= W2Data_00( 50 downto 48 );
  coeff_15_01 <= W2Data_01( 50 downto 48 );
  coeff_15_02 <= W2Data_02( 50 downto 48 );
  coeff_15_03 <= W2Data_03( 50 downto 48 ); 


  coeff_16_00 <= W2Data_00( 47 downto 45 );
  coeff_16_01 <= W2Data_01( 47 downto 45 );
  coeff_16_02 <= W2Data_02( 47 downto 45 );
  coeff_16_03 <= W2Data_03( 47 downto 45 ); 


  coeff_17_00 <= W2Data_00( 44 downto 42 );
  coeff_17_01 <= W2Data_01( 44 downto 42 );
  coeff_17_02 <= W2Data_02( 44 downto 42 );
  coeff_17_03 <= W2Data_03( 44 downto 42 ); 


  coeff_18_00 <= W2Data_00( 41 downto 39 );
  coeff_18_01 <= W2Data_01( 41 downto 39 );
  coeff_18_02 <= W2Data_02( 41 downto 39 );
  coeff_18_03 <= W2Data_03( 41 downto 39 ); 


  coeff_19_00 <= W2Data_00( 38 downto 36 );
  coeff_19_01 <= W2Data_01( 38 downto 36 );
  coeff_19_02 <= W2Data_02( 38 downto 36 );
  coeff_19_03 <= W2Data_03( 38 downto 36 ); 


  coeff_20_00 <= W2Data_00( 35 downto 33 );
  coeff_20_01 <= W2Data_01( 35 downto 33 );
  coeff_20_02 <= W2Data_02( 35 downto 33 );
  coeff_20_03 <= W2Data_03( 35 downto 33 ); 


  coeff_21_00 <= W2Data_00( 32 downto 30 );
  coeff_21_01 <= W2Data_01( 32 downto 30 );
  coeff_21_02 <= W2Data_02( 32 downto 30 );
  coeff_21_03 <= W2Data_03( 32 downto 30 ); 


  coeff_22_00 <= W2Data_00( 29 downto 27 );
  coeff_22_01 <= W2Data_01( 29 downto 27 );
  coeff_22_02 <= W2Data_02( 29 downto 27 );
  coeff_22_03 <= W2Data_03( 29 downto 27 ); 


  coeff_23_00 <= W2Data_00( 26 downto 24 );
  coeff_23_01 <= W2Data_01( 26 downto 24 );
  coeff_23_02 <= W2Data_02( 26 downto 24 );
  coeff_23_03 <= W2Data_03( 26 downto 24 ); 


  coeff_24_00 <= W2Data_00( 23 downto 21 );
  coeff_24_01 <= W2Data_01( 23 downto 21 );
  coeff_24_02 <= W2Data_02( 23 downto 21 );
  coeff_24_03 <= W2Data_03( 23 downto 21 ); 


  coeff_25_00 <= W2Data_00( 20 downto 18 );
  coeff_25_01 <= W2Data_01( 20 downto 18 );
  coeff_25_02 <= W2Data_02( 20 downto 18 );
  coeff_25_03 <= W2Data_03( 20 downto 18 ); 


  coeff_26_00 <= W2Data_00( 17 downto 15 );
  coeff_26_01 <= W2Data_01( 17 downto 15 );
  coeff_26_02 <= W2Data_02( 17 downto 15 );
  coeff_26_03 <= W2Data_03( 17 downto 15 ); 


  coeff_27_00 <= W2Data_00( 14 downto 12 );
  coeff_27_01 <= W2Data_01( 14 downto 12 );
  coeff_27_02 <= W2Data_02( 14 downto 12 );
  coeff_27_03 <= W2Data_03( 14 downto 12 ); 


  coeff_28_00 <= W2Data_00( 11 downto 09 );
  coeff_28_01 <= W2Data_01( 11 downto 09 );
  coeff_28_02 <= W2Data_02( 11 downto 09 );
  coeff_28_03 <= W2Data_03( 11 downto 09 ); 


  coeff_29_00 <= W2Data_00( 08 downto 06 );
  coeff_29_01 <= W2Data_01( 08 downto 06 );
  coeff_29_02 <= W2Data_02( 08 downto 06 );
  coeff_29_03 <= W2Data_03( 08 downto 06 ); 


  coeff_30_00 <= W2Data_00( 05 downto 03 );
  coeff_30_01 <= W2Data_01( 05 downto 03 );
  coeff_30_02 <= W2Data_02( 05 downto 03 );
  coeff_30_03 <= W2Data_03( 05 downto 03 ); 


  coeff_31_00 <= W2Data_00( 02 downto 00 );
  coeff_31_01 <= W2Data_01( 02 downto 00 );
  coeff_31_02 <= W2Data_02( 02 downto 00 );
  coeff_31_03 <= W2Data_03( 02 downto 00 ); 

---------------------------------------------------------------------------------
  CO9_p0 <= "00" & CO9;
  CO9_p9 <= ("00" & CO9) + 9;
  CO9_p18 <= ("00" & CO9) + 18;
  CO9_p27 <= ("00" & CO9) + 27;

  W2Addr <= CO9_p0 when in1ChSel = "00" else CO9_p9 when in1ChSel = "01" else CO9_p18 when in1ChSel = "10" else CO9_p27;

  CO9is0 <= '1' when (CO9 = "0001" and in1ChSel = "00") else '0';

--------------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                CnnStart_dly <= '0';
        elsif( CLK'event and CLK = '1' ) then
                CnnStart_dly <= CnnStart;
        end if;
end process;

---------------------------------------------------------------------------------
ROM_W2_00_inst : ROM_W2_00 port map (
  CLK => CLK,  
  ADDR => W2Addr,
  DATA => W2Data_00 );

ROM_W2_01_inst : ROM_W2_01 port map (
  CLK => CLK,  
  ADDR => W2Addr,
  DATA => W2Data_01 );

ROM_W2_02_inst : ROM_W2_02 port map (
  CLK => CLK,  
  ADDR => W2Addr,
  DATA => W2Data_02 );

ROM_W2_03_inst : ROM_W2_03 port map (
  CLK => CLK,  
  ADDR => W2Addr,
  DATA => W2Data_03 );

-------------------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    in1ChSel_dly <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    in1ChSel_dly <= in1ChSel;
  end if;
end process;

inch_00 <= LAY2IN(79 downto 75) when in1ChSel_dly = "00" else LAY2IN(74 downto 70) when in1ChSel_dly = "01" else LAY2IN(69 downto 65) when in1ChSel_dly = "10" else LAY2IN(64 downto 60);

inch_01 <= LAY2IN(59 downto 55) when in1ChSel_dly = "00" else LAY2IN(54 downto 50) when in1ChSel_dly = "01" else LAY2IN(49 downto 45) when in1ChSel_dly = "10" else LAY2IN(44 downto 40);

inch_02 <= LAY2IN(39 downto 35) when in1ChSel_dly = "00" else LAY2IN(34 downto 30) when in1ChSel_dly = "01" else LAY2IN(29 downto 25) when in1ChSel_dly = "10" else LAY2IN(24 downto 20);

inch_03 <= LAY2IN(19 downto 15) when in1ChSel_dly = "00" else LAY2IN(14 downto 10) when in1ChSel_dly = "01" else LAY2IN(9 downto 5) when in1ChSel_dly = "10" else LAY2IN(4 downto 0);

---------------------------------------------------------------------------------
dotpro_00_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_00_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00_00 );

dotpro_00_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_00_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00_01 );

dotpro_00_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_00_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00_02 );

dotpro_00_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_00_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00_03 );

dotpro_01_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_01_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01_00 );

dotpro_01_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_01_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01_01 );

dotpro_01_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_01_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01_02 );

dotpro_01_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_01_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01_03 );

dotpro_02_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_02_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02_00 );

dotpro_02_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_02_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02_01 );

dotpro_02_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_02_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02_02 );

dotpro_02_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_02_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02_03 );

dotpro_03_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_03_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03_00 );

dotpro_03_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_03_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03_01 );

dotpro_03_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_03_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03_02 );

dotpro_03_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_03_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03_03 );

dotpro_04_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_04_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04_00 );

dotpro_04_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_04_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04_01 );

dotpro_04_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_04_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04_02 );

dotpro_04_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_04_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04_03 );

dotpro_05_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_05_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05_00 );

dotpro_05_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_05_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05_01 );

dotpro_05_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_05_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05_02 );

dotpro_05_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_05_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05_03 );

dotpro_06_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_06_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06_00 );

dotpro_06_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_06_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06_01 );

dotpro_06_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_06_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06_02 );

dotpro_06_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_06_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06_03 );

dotpro_07_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_07_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07_00 );

dotpro_07_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_07_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07_01 );

dotpro_07_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_07_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07_02 );

dotpro_07_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_07_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07_03 );

dotpro_08_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_08_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08_00 );

dotpro_08_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_08_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08_01 );

dotpro_08_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_08_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08_02 );

dotpro_08_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_08_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08_03 );

dotpro_09_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_09_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09_00 );

dotpro_09_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_09_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09_01 );

dotpro_09_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_09_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09_02 );

dotpro_09_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_09_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09_03 );

dotpro_10_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_10_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10_00 );

dotpro_10_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_10_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10_01 );

dotpro_10_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_10_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10_02 );

dotpro_10_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_10_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10_03 );

dotpro_11_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_11_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11_00 );

dotpro_11_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_11_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11_01 );

dotpro_11_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_11_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11_02 );

dotpro_11_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_11_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11_03 );

dotpro_12_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_12_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12_00 );

dotpro_12_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_12_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12_01 );

dotpro_12_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_12_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12_02 );

dotpro_12_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_12_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12_03 );

dotpro_13_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_13_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13_00 );

dotpro_13_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_13_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13_01 );

dotpro_13_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_13_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13_02 );

dotpro_13_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_13_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13_03 );

dotpro_14_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_14_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14_00 );

dotpro_14_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_14_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14_01 );

dotpro_14_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_14_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14_02 );

dotpro_14_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_14_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14_03 );

dotpro_15_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_15_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15_00 );

dotpro_15_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_15_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15_01 );

dotpro_15_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_15_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15_02 );

dotpro_15_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_15_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15_03 );

dotpro_16_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_16_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_16_00 );

dotpro_16_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_16_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_16_01 );

dotpro_16_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_16_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_16_02 );

dotpro_16_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_16_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_16_03 );

dotpro_17_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_17_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_17_00 );

dotpro_17_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_17_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_17_01 );

dotpro_17_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_17_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_17_02 );

dotpro_17_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_17_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_17_03 );

dotpro_18_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_18_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_18_00 );

dotpro_18_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_18_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_18_01 );

dotpro_18_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_18_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_18_02 );

dotpro_18_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_18_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_18_03 );

dotpro_19_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_19_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_19_00 );

dotpro_19_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_19_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_19_01 );

dotpro_19_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_19_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_19_02 );

dotpro_19_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_19_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_19_03 );

dotpro_20_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_20_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_20_00 );

dotpro_20_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_20_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_20_01 );

dotpro_20_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_20_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_20_02 );

dotpro_20_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_20_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_20_03 );

dotpro_21_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_21_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_21_00 );

dotpro_21_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_21_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_21_01 );

dotpro_21_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_21_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_21_02 );

dotpro_21_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_21_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_21_03 );

dotpro_22_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_22_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_22_00 );

dotpro_22_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_22_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_22_01 );

dotpro_22_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_22_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_22_02 );

dotpro_22_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_22_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_22_03 );

dotpro_23_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_23_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_23_00 );

dotpro_23_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_23_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_23_01 );

dotpro_23_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_23_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_23_02 );

dotpro_23_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_23_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_23_03 );

dotpro_24_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_24_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_24_00 );

dotpro_24_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_24_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_24_01 );

dotpro_24_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_24_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_24_02 );

dotpro_24_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_24_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_24_03 );

dotpro_25_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_25_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_25_00 );

dotpro_25_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_25_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_25_01 );

dotpro_25_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_25_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_25_02 );

dotpro_25_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_25_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_25_03 );

dotpro_26_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_26_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_26_00 );

dotpro_26_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_26_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_26_01 );

dotpro_26_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_26_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_26_02 );

dotpro_26_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_26_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_26_03 );

dotpro_27_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_27_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_27_00 );

dotpro_27_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_27_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_27_01 );

dotpro_27_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_27_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_27_02 );

dotpro_27_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_27_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_27_03 );

dotpro_28_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_28_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_28_00 );

dotpro_28_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_28_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_28_01 );

dotpro_28_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_28_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_28_02 );

dotpro_28_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_28_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_28_03 );

dotpro_29_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_29_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_29_00 );

dotpro_29_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_29_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_29_01 );

dotpro_29_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_29_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_29_02 );

dotpro_29_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_29_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_29_03 );

dotpro_30_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_30_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_30_00 );

dotpro_30_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_30_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_30_01 );

dotpro_30_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_30_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_30_02 );

dotpro_30_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_30_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_30_03 );

dotpro_31_00 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_31_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_31_00 );

dotpro_31_01 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_31_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_31_01 );

dotpro_31_02 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_31_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_31_02 );

dotpro_31_03 : dotpro_w2 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_31_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_31_03 );


---------------------------------------------------------------------------------
  MacAll_00 <= mac_00_00 + mac_00_01 + mac_00_02 + mac_00_03;
  MacShift_00 <= MacAll_00 & "00";

  MacAll_01 <= mac_01_00 + mac_01_01 + mac_01_02 + mac_01_03;
  MacShift_01 <= MacAll_01 & "00";

  MacAll_02 <= mac_02_00 + mac_02_01 + mac_02_02 + mac_02_03;
  MacShift_02 <= MacAll_02 & "00";

  MacAll_03 <= mac_03_00 + mac_03_01 + mac_03_02 + mac_03_03;
  MacShift_03 <= MacAll_03 & "00";

  MacAll_04 <= mac_04_00 + mac_04_01 + mac_04_02 + mac_04_03;
  MacShift_04 <= MacAll_04 & "00";

  MacAll_05 <= mac_05_00 + mac_05_01 + mac_05_02 + mac_05_03;
  MacShift_05 <= MacAll_05 & "00";

  MacAll_06 <= mac_06_00 + mac_06_01 + mac_06_02 + mac_06_03;
  MacShift_06 <= MacAll_06 & "00";

  MacAll_07 <= mac_07_00 + mac_07_01 + mac_07_02 + mac_07_03;
  MacShift_07 <= MacAll_07 & "00";

  MacAll_08 <= mac_08_00 + mac_08_01 + mac_08_02 + mac_08_03;
  MacShift_08 <= MacAll_08 & "00";

  MacAll_09 <= mac_09_00 + mac_09_01 + mac_09_02 + mac_09_03;
  MacShift_09 <= MacAll_09 & "00";

  MacAll_10 <= mac_10_00 + mac_10_01 + mac_10_02 + mac_10_03;
  MacShift_10 <= MacAll_10 & "00";

  MacAll_11 <= mac_11_00 + mac_11_01 + mac_11_02 + mac_11_03;
  MacShift_11 <= MacAll_11 & "00";

  MacAll_12 <= mac_12_00 + mac_12_01 + mac_12_02 + mac_12_03;
  MacShift_12 <= MacAll_12 & "00";

  MacAll_13 <= mac_13_00 + mac_13_01 + mac_13_02 + mac_13_03;
  MacShift_13 <= MacAll_13 & "00";

  MacAll_14 <= mac_14_00 + mac_14_01 + mac_14_02 + mac_14_03;
  MacShift_14 <= MacAll_14 & "00";

  MacAll_15 <= mac_15_00 + mac_15_01 + mac_15_02 + mac_15_03;
  MacShift_15 <= MacAll_15 & "00";

  MacAll_16 <= mac_16_00 + mac_16_01 + mac_16_02 + mac_16_03;
  MacShift_16 <= MacAll_16 & "00";

  MacAll_17 <= mac_17_00 + mac_17_01 + mac_17_02 + mac_17_03;
  MacShift_17 <= MacAll_17 & "00";

  MacAll_18 <= mac_18_00 + mac_18_01 + mac_18_02 + mac_18_03;
  MacShift_18 <= MacAll_18 & "00";

  MacAll_19 <= mac_19_00 + mac_19_01 + mac_19_02 + mac_19_03;
  MacShift_19 <= MacAll_19 & "00";

  MacAll_20 <= mac_20_00 + mac_20_01 + mac_20_02 + mac_20_03;
  MacShift_20 <= MacAll_20 & "00";

  MacAll_21 <= mac_21_00 + mac_21_01 + mac_21_02 + mac_21_03;
  MacShift_21 <= MacAll_21 & "00";

  MacAll_22 <= mac_22_00 + mac_22_01 + mac_22_02 + mac_22_03;
  MacShift_22 <= MacAll_22 & "00";

  MacAll_23 <= mac_23_00 + mac_23_01 + mac_23_02 + mac_23_03;
  MacShift_23 <= MacAll_23 & "00";

  MacAll_24 <= mac_24_00 + mac_24_01 + mac_24_02 + mac_24_03;
  MacShift_24 <= MacAll_24 & "00";

  MacAll_25 <= mac_25_00 + mac_25_01 + mac_25_02 + mac_25_03;
  MacShift_25 <= MacAll_25 & "00";

  MacAll_26 <= mac_26_00 + mac_26_01 + mac_26_02 + mac_26_03;
  MacShift_26 <= MacAll_26 & "00";

  MacAll_27 <= mac_27_00 + mac_27_01 + mac_27_02 + mac_27_03;
  MacShift_27 <= MacAll_27 & "00";

  MacAll_28 <= mac_28_00 + mac_28_01 + mac_28_02 + mac_28_03;
  MacShift_28 <= MacAll_28 & "00";

  MacAll_29 <= mac_29_00 + mac_29_01 + mac_29_02 + mac_29_03;
  MacShift_29 <= MacAll_29 & "00";

  MacAll_30 <= mac_30_00 + mac_30_01 + mac_30_02 + mac_30_03;
  MacShift_30 <= MacAll_30 & "00";

  MacAll_31 <= mac_31_00 + mac_31_01 + mac_31_02 + mac_31_03;
  MacShift_31 <= MacAll_31 & "00"; 

  BIAS_00 <= "111111111100111011";
  BIAS_01 <= "111111111010000010";
  BIAS_02 <= "000000000101101000";
  BIAS_03 <= "000000000000001010";
  BIAS_04 <= "000000001001111001";
  BIAS_05 <= "111111111000101001";
  BIAS_06 <= "111111111111111100";
  BIAS_07 <= "000000000100101010";
  BIAS_08 <= "111111111111001101";
  BIAS_09 <= "111111110110001111";
  BIAS_10 <= "000000000001111010";
  BIAS_11 <= "111111111100101101";
  BIAS_12 <= "111111111110101011";
  BIAS_13 <= "111111111111000011";
  BIAS_14 <= "111111111011110101";
  BIAS_15 <= "111111111111011110";
  BIAS_16 <= "000000000110000010";
  BIAS_17 <= "000000000011110001";
  BIAS_18 <= "111111111101011100";
  BIAS_19 <= "000000010011111101";
  BIAS_20 <= "000000000101001111";
  BIAS_21 <= "111111111100111010";
  BIAS_22 <= "111111111101000110";
  BIAS_23 <= "000000000000110101";
  BIAS_24 <= "111111110110111000";
  BIAS_25 <= "111111111101101000";
  BIAS_26 <= "000000000011111101";
  BIAS_27 <= "000000000100001000";
  BIAS_28 <= "000000000001100100";
  BIAS_29 <= "000000000011010011";
  BIAS_30 <= "000000000011101111";
  BIAS_31 <= "000000000001000101";

  plusBias_00 <= MacShift_00 + BIAS_00;
  plusBias_01 <= MacShift_01 + BIAS_01;
  plusBias_02 <= MacShift_02 + BIAS_02;
  plusBias_03 <= MacShift_03 + BIAS_03;
  plusBias_04 <= MacShift_04 + BIAS_04;
  plusBias_05 <= MacShift_05 + BIAS_05;
  plusBias_06 <= MacShift_06 + BIAS_06;
  plusBias_07 <= MacShift_07 + BIAS_07;
  plusBias_08 <= MacShift_08 + BIAS_08;
  plusBias_09 <= MacShift_09 + BIAS_09;
  plusBias_10 <= MacShift_10 + BIAS_10;
  plusBias_11 <= MacShift_11 + BIAS_11;
  plusBias_12 <= MacShift_12 + BIAS_12;
  plusBias_13 <= MacShift_13 + BIAS_13;
  plusBias_14 <= MacShift_14 + BIAS_14;
  plusBias_15 <= MacShift_15 + BIAS_15;
  plusBias_16 <= MacShift_16 + BIAS_16;
  plusBias_17 <= MacShift_17 + BIAS_17;
  plusBias_18 <= MacShift_18 + BIAS_18;
  plusBias_19 <= MacShift_19 + BIAS_19;
  plusBias_20 <= MacShift_20 + BIAS_20;
  plusBias_21 <= MacShift_21 + BIAS_21;
  plusBias_22 <= MacShift_22 + BIAS_22;
  plusBias_23 <= MacShift_23 + BIAS_23;
  plusBias_24 <= MacShift_24 + BIAS_24;
  plusBias_25 <= MacShift_25 + BIAS_25;
  plusBias_26 <= MacShift_26 + BIAS_26;
  plusBias_27 <= MacShift_27 + BIAS_27;
  plusBias_28 <= MacShift_28 + BIAS_28;
  plusBias_29 <= MacShift_29 + BIAS_29;
  plusBias_30 <= MacShift_30 + BIAS_30;
  plusBias_31 <= MacShift_31 + BIAS_31;

  truncate_00 <= plusBias_00(17 downto 8);
  truncate_01 <= plusBias_01(17 downto 8);
  truncate_02 <= plusBias_02(17 downto 8);
  truncate_03 <= plusBias_03(17 downto 8);
  truncate_04 <= plusBias_04(17 downto 8);
  truncate_05 <= plusBias_05(17 downto 8);
  truncate_06 <= plusBias_06(17 downto 8);
  truncate_07 <= plusBias_07(17 downto 8);
  truncate_08 <= plusBias_08(17 downto 8);
  truncate_09 <= plusBias_09(17 downto 8);
  truncate_10 <= plusBias_10(17 downto 8);
  truncate_11 <= plusBias_11(17 downto 8);
  truncate_12 <= plusBias_12(17 downto 8);
  truncate_13 <= plusBias_13(17 downto 8);
  truncate_14 <= plusBias_14(17 downto 8);
  truncate_15 <= plusBias_15(17 downto 8);
  truncate_16 <= plusBias_16(17 downto 8);
  truncate_17 <= plusBias_17(17 downto 8);
  truncate_18 <= plusBias_18(17 downto 8);
  truncate_19 <= plusBias_19(17 downto 8);
  truncate_20 <= plusBias_20(17 downto 8);
  truncate_21 <= plusBias_21(17 downto 8);
  truncate_22 <= plusBias_22(17 downto 8);
  truncate_23 <= plusBias_23(17 downto 8);
  truncate_24 <= plusBias_24(17 downto 8);
  truncate_25 <= plusBias_25(17 downto 8);
  truncate_26 <= plusBias_26(17 downto 8);
  truncate_27 <= plusBias_27(17 downto 8);
  truncate_28 <= plusBias_28(17 downto 8);
  truncate_29 <= plusBias_29(17 downto 8);
  truncate_30 <= plusBias_30(17 downto 8);
  truncate_31 <= plusBias_31(17 downto 8);

  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

  Relu_16 <= (others => '0') when truncate_16 < 0 else
             (others => '1') when truncate_16 > 31 else
             truncate_16(4 downto 0);

  Relu_17 <= (others => '0') when truncate_17 < 0 else
             (others => '1') when truncate_17 > 31 else
             truncate_17(4 downto 0);

  Relu_18 <= (others => '0') when truncate_18 < 0 else
             (others => '1') when truncate_18 > 31 else
             truncate_18(4 downto 0);

  Relu_19 <= (others => '0') when truncate_19 < 0 else
             (others => '1') when truncate_19 > 31 else
             truncate_19(4 downto 0);

  Relu_20 <= (others => '0') when truncate_20 < 0 else
             (others => '1') when truncate_20 > 31 else
             truncate_20(4 downto 0);

  Relu_21 <= (others => '0') when truncate_21 < 0 else
             (others => '1') when truncate_21 > 31 else
             truncate_21(4 downto 0);

  Relu_22 <= (others => '0') when truncate_22 < 0 else
             (others => '1') when truncate_22 > 31 else
             truncate_22(4 downto 0);

  Relu_23 <= (others => '0') when truncate_23 < 0 else
             (others => '1') when truncate_23 > 31 else
             truncate_23(4 downto 0);

  Relu_24 <= (others => '0') when truncate_24 < 0 else
             (others => '1') when truncate_24 > 31 else
             truncate_24(4 downto 0);

  Relu_25 <= (others => '0') when truncate_25 < 0 else
             (others => '1') when truncate_25 > 31 else
             truncate_25(4 downto 0);

  Relu_26 <= (others => '0') when truncate_26 < 0 else
             (others => '1') when truncate_26 > 31 else
             truncate_26(4 downto 0);

  Relu_27 <= (others => '0') when truncate_27 < 0 else
             (others => '1') when truncate_27 > 31 else
             truncate_27(4 downto 0);

  Relu_28 <= (others => '0') when truncate_28 < 0 else
             (others => '1') when truncate_28 > 31 else
             truncate_28(4 downto 0);

  Relu_29 <= (others => '0') when truncate_29 < 0 else
             (others => '1') when truncate_29 > 31 else
             truncate_29(4 downto 0);

  Relu_30 <= (others => '0') when truncate_30 < 0 else
             (others => '1') when truncate_30 > 31 else
             truncate_30(4 downto 0);

  Relu_31 <= (others => '0') when truncate_31 < 0 else
             (others => '1') when truncate_31 > 31 else
             truncate_31(4 downto 0);


ReluAll <= Relu_00 & Relu_01
		& Relu_02 & Relu_03
		& Relu_04 & Relu_05
		& Relu_06 & Relu_07
		& Relu_08 & Relu_09
                & Relu_10 & Relu_11
		& Relu_12 & Relu_13
		& Relu_14 & Relu_15
		& Relu_16 & Relu_17
		& Relu_18 & Relu_19
                & Relu_20 & Relu_21
		& Relu_22 & Relu_23
		& Relu_24 & Relu_25
		& Relu_26 & Relu_27
		& Relu_28 & Relu_29
                & Relu_30 & Relu_31 ;

---------------------------------------------------------
pooling_lay2_inst : pooling_lay2 port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> ReluAll,
   WR_N	=> P_WR_N,
   HCO28	=> CO28,
   Poolout	=> Poolout);

pool_00 <= Poolout( 159 downto 155 );
pool_01 <= Poolout( 154 downto 150 );
pool_02 <= Poolout( 149 downto 145 );
pool_03 <= Poolout( 144 downto 140 );

  LAY2OUT <= Poolout;

end RTL;

