-- The layer5 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/07/01

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer5 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  CO9 : in std_logic_vector(3 downto 0); 
  CnnStart : IN std_logic;
  in5ChSel : in std_logic_vector(6 downto 0);
  W5Addr_H : in std_logic_vector(3 downto 0);
  RAM_X4_RD_ADDR : in std_logic_vector(3 downto 0);
  RAM_X5_WR_ADDR: out std_logic_vector(6 downto 0);
  RAM_X5_WR_N: out std_logic;
  LAY5IN : in std_logic_vector(639 downto 0); 
  LAY5OUT : out std_logic_vector(4 downto 0)); 
end conv_layer5;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer5 is

component ROM_W5_00
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_01
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_02
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_03
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_04
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_05
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_06
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_07
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_08
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_09
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_10
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_11
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_12
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_13
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_14
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component ROM_W5_15
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(15 downto 0) );
end component;

component dotpro_w5
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(1 downto 0);
  CO9is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal W5Data_00, W5Data_01, W5Data_02, W5Data_03, W5Data_04, W5Data_05, W5Data_06, W5Data_07, W5Data_08, W5Data_09: std_logic_vector(15 downto 0);
signal W5Data_10, W5Data_11, W5Data_12, W5Data_13, W5Data_14, W5Data_15: std_logic_vector(15 downto 0);

signal romdata_00, romdata_01, romdata_02, romdata_03, romdata_04, romdata_05, romdata_06, romdata_07, romdata_08, romdata_09: std_logic_vector(1 downto 0);
signal romdata_10, romdata_11, romdata_12, romdata_13, romdata_14, romdata_15: std_logic_vector(1 downto 0);

signal W5Addr : std_logic_vector(10 downto 0);
signal W5Addr_H_dly : std_logic_vector(3 downto 0);
signal W5Addr_H_m1 : std_logic_vector(3 downto 0);
signal W5DataSel : std_logic_vector(3 downto 0);

--signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(1 downto 0);
--signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15: std_logic_vector(1 downto 0);

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09: std_logic_vector(13 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15: std_logic_vector(13 downto 0);

signal MacShift_00, MacShift_01, MacShift_02, MacShift_03, MacShift_04, MacShift_05, MacShift_06, MacShift_07, MacShift_08, MacShift_09: std_logic_vector(17 downto 0);
signal MacShift_10, MacShift_11, MacShift_12, MacShift_13, MacShift_14, MacShift_15: std_logic_vector(17 downto 0);

signal BIAS_000, BIAS_001, BIAS_002, BIAS_003, BIAS_004, BIAS_005, BIAS_006, BIAS_007, BIAS_008, BIAS_009: std_logic_vector(17 downto 0);
signal BIAS_010, BIAS_011, BIAS_012, BIAS_013, BIAS_014, BIAS_015, BIAS_016, BIAS_017, BIAS_018, BIAS_019: std_logic_vector(17 downto 0);
signal BIAS_020, BIAS_021, BIAS_022, BIAS_023, BIAS_024, BIAS_025, BIAS_026, BIAS_027, BIAS_028, BIAS_029: std_logic_vector(17 downto 0);
signal BIAS_030, BIAS_031, BIAS_032, BIAS_033, BIAS_034, BIAS_035, BIAS_036, BIAS_037, BIAS_038, BIAS_039: std_logic_vector(17 downto 0);
signal BIAS_040, BIAS_041, BIAS_042, BIAS_043, BIAS_044, BIAS_045, BIAS_046, BIAS_047, BIAS_048, BIAS_049: std_logic_vector(17 downto 0);
signal BIAS_050, BIAS_051, BIAS_052, BIAS_053, BIAS_054, BIAS_055, BIAS_056, BIAS_057, BIAS_058, BIAS_059: std_logic_vector(17 downto 0);
signal BIAS_060, BIAS_061, BIAS_062, BIAS_063, BIAS_064, BIAS_065, BIAS_066, BIAS_067, BIAS_068, BIAS_069: std_logic_vector(17 downto 0);
signal BIAS_070, BIAS_071, BIAS_072, BIAS_073, BIAS_074, BIAS_075, BIAS_076, BIAS_077, BIAS_078, BIAS_079: std_logic_vector(17 downto 0);
signal BIAS_080, BIAS_081, BIAS_082, BIAS_083, BIAS_084, BIAS_085, BIAS_086, BIAS_087, BIAS_088, BIAS_089: std_logic_vector(17 downto 0);
signal BIAS_090, BIAS_091, BIAS_092, BIAS_093, BIAS_094, BIAS_095, BIAS_096, BIAS_097, BIAS_098, BIAS_099: std_logic_vector(17 downto 0);
signal BIAS_100, BIAS_101, BIAS_102, BIAS_103, BIAS_104, BIAS_105, BIAS_106, BIAS_107, BIAS_108, BIAS_109: std_logic_vector(17 downto 0);
signal BIAS_110, BIAS_111, BIAS_112, BIAS_113, BIAS_114, BIAS_115, BIAS_116, BIAS_117, BIAS_118, BIAS_119: std_logic_vector(17 downto 0);
signal BIAS_120, BIAS_121, BIAS_122, BIAS_123, BIAS_124, BIAS_125, BIAS_126, BIAS_127: std_logic_vector(17 downto 0);

signal BIAS_SEL_00, BIAS_SEL_01, BIAS_SEL_02, BIAS_SEL_03, BIAS_SEL_04, BIAS_SEL_05, BIAS_SEL_06, BIAS_SEL_07, BIAS_SEL_08, BIAS_SEL_09: std_logic_vector(17 downto 0);
signal BIAS_SEL_10, BIAS_SEL_11, BIAS_SEL_12, BIAS_SEL_13, BIAS_SEL_14, BIAS_SEL_15: std_logic_vector(17 downto 0);

signal plusBias_00, plusBias_01, plusBias_02, plusBias_03, plusBias_04, plusBias_05, plusBias_06, plusBias_07, plusBias_08, plusBias_09: std_logic_vector(17 downto 0);
signal plusBias_10, plusBias_11, plusBias_12, plusBias_13, plusBias_14, plusBias_15: std_logic_vector(17 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09: std_logic_vector(9 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15: std_logic_vector(9 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09: std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15: std_logic_vector(4 downto 0);

signal ReluAll: std_logic_vector(4 downto 0);

signal CO9is0 : std_logic;
signal CO9is0_sig : std_logic;

signal inch : std_logic_vector(4 downto 0);
signal in5ChSel_dly : std_logic_vector(6 downto 0);

begin

inch <= 
LAY5IN( 639 downto 635 ) when in5ChSel_dly = "0000000" else
LAY5IN( 634 downto 630 ) when in5ChSel_dly = "0000001" else
LAY5IN( 629 downto 625 ) when in5ChSel_dly = "0000010" else
LAY5IN( 624 downto 620 ) when in5ChSel_dly = "0000011" else
LAY5IN( 619 downto 615 ) when in5ChSel_dly = "0000100" else
LAY5IN( 614 downto 610 ) when in5ChSel_dly = "0000101" else
LAY5IN( 609 downto 605 ) when in5ChSel_dly = "0000110" else
LAY5IN( 604 downto 600 ) when in5ChSel_dly = "0000111" else
LAY5IN( 599 downto 595 ) when in5ChSel_dly = "0001000" else
LAY5IN( 594 downto 590 ) when in5ChSel_dly = "0001001" else
LAY5IN( 589 downto 585 ) when in5ChSel_dly = "0001010" else
LAY5IN( 584 downto 580 ) when in5ChSel_dly = "0001011" else
LAY5IN( 579 downto 575 ) when in5ChSel_dly = "0001100" else
LAY5IN( 574 downto 570 ) when in5ChSel_dly = "0001101" else
LAY5IN( 569 downto 565 ) when in5ChSel_dly = "0001110" else
LAY5IN( 564 downto 560 ) when in5ChSel_dly = "0001111" else
LAY5IN( 559 downto 555 ) when in5ChSel_dly = "0010000" else
LAY5IN( 554 downto 550 ) when in5ChSel_dly = "0010001" else
LAY5IN( 549 downto 545 ) when in5ChSel_dly = "0010010" else
LAY5IN( 544 downto 540 ) when in5ChSel_dly = "0010011" else
LAY5IN( 539 downto 535 ) when in5ChSel_dly = "0010100" else
LAY5IN( 534 downto 530 ) when in5ChSel_dly = "0010101" else
LAY5IN( 529 downto 525 ) when in5ChSel_dly = "0010110" else
LAY5IN( 524 downto 520 ) when in5ChSel_dly = "0010111" else
LAY5IN( 519 downto 515 ) when in5ChSel_dly = "0011000" else
LAY5IN( 514 downto 510 ) when in5ChSel_dly = "0011001" else
LAY5IN( 509 downto 505 ) when in5ChSel_dly = "0011010" else
LAY5IN( 504 downto 500 ) when in5ChSel_dly = "0011011" else
LAY5IN( 499 downto 495 ) when in5ChSel_dly = "0011100" else
LAY5IN( 494 downto 490 ) when in5ChSel_dly = "0011101" else
LAY5IN( 489 downto 485 ) when in5ChSel_dly = "0011110" else
LAY5IN( 484 downto 480 ) when in5ChSel_dly = "0011111" else
LAY5IN( 479 downto 475 ) when in5ChSel_dly = "0100000" else
LAY5IN( 474 downto 470 ) when in5ChSel_dly = "0100001" else
LAY5IN( 469 downto 465 ) when in5ChSel_dly = "0100010" else
LAY5IN( 464 downto 460 ) when in5ChSel_dly = "0100011" else
LAY5IN( 459 downto 455 ) when in5ChSel_dly = "0100100" else
LAY5IN( 454 downto 450 ) when in5ChSel_dly = "0100101" else
LAY5IN( 449 downto 445 ) when in5ChSel_dly = "0100110" else
LAY5IN( 444 downto 440 ) when in5ChSel_dly = "0100111" else
LAY5IN( 439 downto 435 ) when in5ChSel_dly = "0101000" else
LAY5IN( 434 downto 430 ) when in5ChSel_dly = "0101001" else
LAY5IN( 429 downto 425 ) when in5ChSel_dly = "0101010" else
LAY5IN( 424 downto 420 ) when in5ChSel_dly = "0101011" else
LAY5IN( 419 downto 415 ) when in5ChSel_dly = "0101100" else
LAY5IN( 414 downto 410 ) when in5ChSel_dly = "0101101" else
LAY5IN( 409 downto 405 ) when in5ChSel_dly = "0101110" else
LAY5IN( 404 downto 400 ) when in5ChSel_dly = "0101111" else
LAY5IN( 399 downto 395 ) when in5ChSel_dly = "0110000" else
LAY5IN( 394 downto 390 ) when in5ChSel_dly = "0110001" else
LAY5IN( 389 downto 385 ) when in5ChSel_dly = "0110010" else
LAY5IN( 384 downto 380 ) when in5ChSel_dly = "0110011" else
LAY5IN( 379 downto 375 ) when in5ChSel_dly = "0110100" else
LAY5IN( 374 downto 370 ) when in5ChSel_dly = "0110101" else
LAY5IN( 369 downto 365 ) when in5ChSel_dly = "0110110" else
LAY5IN( 364 downto 360 ) when in5ChSel_dly = "0110111" else
LAY5IN( 359 downto 355 ) when in5ChSel_dly = "0111000" else
LAY5IN( 354 downto 350 ) when in5ChSel_dly = "0111001" else
LAY5IN( 349 downto 345 ) when in5ChSel_dly = "0111010" else
LAY5IN( 344 downto 340 ) when in5ChSel_dly = "0111011" else
LAY5IN( 339 downto 335 ) when in5ChSel_dly = "0111100" else
LAY5IN( 334 downto 330 ) when in5ChSel_dly = "0111101" else
LAY5IN( 329 downto 325 ) when in5ChSel_dly = "0111110" else
LAY5IN( 324 downto 320 ) when in5ChSel_dly = "0111111" else
LAY5IN( 319 downto 315 ) when in5ChSel_dly = "1000000" else
LAY5IN( 314 downto 310 ) when in5ChSel_dly = "1000001" else
LAY5IN( 309 downto 305 ) when in5ChSel_dly = "1000010" else
LAY5IN( 304 downto 300 ) when in5ChSel_dly = "1000011" else
LAY5IN( 299 downto 295 ) when in5ChSel_dly = "1000100" else
LAY5IN( 294 downto 290 ) when in5ChSel_dly = "1000101" else
LAY5IN( 289 downto 285 ) when in5ChSel_dly = "1000110" else
LAY5IN( 284 downto 280 ) when in5ChSel_dly = "1000111" else
LAY5IN( 279 downto 275 ) when in5ChSel_dly = "1001000" else
LAY5IN( 274 downto 270 ) when in5ChSel_dly = "1001001" else
LAY5IN( 269 downto 265 ) when in5ChSel_dly = "1001010" else
LAY5IN( 264 downto 260 ) when in5ChSel_dly = "1001011" else
LAY5IN( 259 downto 255 ) when in5ChSel_dly = "1001100" else
LAY5IN( 254 downto 250 ) when in5ChSel_dly = "1001101" else
LAY5IN( 249 downto 245 ) when in5ChSel_dly = "1001110" else
LAY5IN( 244 downto 240 ) when in5ChSel_dly = "1001111" else
LAY5IN( 239 downto 235 ) when in5ChSel_dly = "1010000" else
LAY5IN( 234 downto 230 ) when in5ChSel_dly = "1010001" else
LAY5IN( 229 downto 225 ) when in5ChSel_dly = "1010010" else
LAY5IN( 224 downto 220 ) when in5ChSel_dly = "1010011" else
LAY5IN( 219 downto 215 ) when in5ChSel_dly = "1010100" else
LAY5IN( 214 downto 210 ) when in5ChSel_dly = "1010101" else
LAY5IN( 209 downto 205 ) when in5ChSel_dly = "1010110" else
LAY5IN( 204 downto 200 ) when in5ChSel_dly = "1010111" else
LAY5IN( 199 downto 195 ) when in5ChSel_dly = "1011000" else
LAY5IN( 194 downto 190 ) when in5ChSel_dly = "1011001" else
LAY5IN( 189 downto 185 ) when in5ChSel_dly = "1011010" else
LAY5IN( 184 downto 180 ) when in5ChSel_dly = "1011011" else
LAY5IN( 179 downto 175 ) when in5ChSel_dly = "1011100" else
LAY5IN( 174 downto 170 ) when in5ChSel_dly = "1011101" else
LAY5IN( 169 downto 165 ) when in5ChSel_dly = "1011110" else
LAY5IN( 164 downto 160 ) when in5ChSel_dly = "1011111" else
LAY5IN( 159 downto 155 ) when in5ChSel_dly = "1100000" else
LAY5IN( 154 downto 150 ) when in5ChSel_dly = "1100001" else
LAY5IN( 149 downto 145 ) when in5ChSel_dly = "1100010" else
LAY5IN( 144 downto 140 ) when in5ChSel_dly = "1100011" else
LAY5IN( 139 downto 135 ) when in5ChSel_dly = "1100100" else
LAY5IN( 134 downto 130 ) when in5ChSel_dly = "1100101" else
LAY5IN( 129 downto 125 ) when in5ChSel_dly = "1100110" else
LAY5IN( 124 downto 120 ) when in5ChSel_dly = "1100111" else
LAY5IN( 119 downto 115 ) when in5ChSel_dly = "1101000" else
LAY5IN( 114 downto 110 ) when in5ChSel_dly = "1101001" else
LAY5IN( 109 downto 105 ) when in5ChSel_dly = "1101010" else
LAY5IN( 104 downto 100 ) when in5ChSel_dly = "1101011" else
LAY5IN( 99 downto 95 ) when in5ChSel_dly = "1101100" else
LAY5IN( 94 downto 90 ) when in5ChSel_dly = "1101101" else
LAY5IN( 89 downto 85 ) when in5ChSel_dly = "1101110" else
LAY5IN( 84 downto 80 ) when in5ChSel_dly = "1101111" else
LAY5IN( 79 downto 75 ) when in5ChSel_dly = "1110000" else
LAY5IN( 74 downto 70 ) when in5ChSel_dly = "1110001" else
LAY5IN( 69 downto 65 ) when in5ChSel_dly = "1110010" else
LAY5IN( 64 downto 60 ) when in5ChSel_dly = "1110011" else
LAY5IN( 59 downto 55 ) when in5ChSel_dly = "1110100" else
LAY5IN( 54 downto 50 ) when in5ChSel_dly = "1110101" else
LAY5IN( 49 downto 45 ) when in5ChSel_dly = "1110110" else
LAY5IN( 44 downto 40 ) when in5ChSel_dly = "1110111" else
LAY5IN( 39 downto 35 ) when in5ChSel_dly = "1111000" else
LAY5IN( 34 downto 30 ) when in5ChSel_dly = "1111001" else
LAY5IN( 29 downto 25 ) when in5ChSel_dly = "1111010" else
LAY5IN( 24 downto 20 ) when in5ChSel_dly = "1111011" else
LAY5IN( 19 downto 15 ) when in5ChSel_dly = "1111100" else
LAY5IN( 14 downto 10 ) when in5ChSel_dly = "1111101" else
LAY5IN( 9 downto 5 ) when in5ChSel_dly = "1111110" else
LAY5IN( 4 downto 0 ) ; -- when in5ChSel_dly = "1111111" else


---------------------------------------------------------------------------------
  --W5Addr <= W5Addr_H(2 downto 0) & RAM_X4_RD_ADDR & in5ChSel;
  W5Addr <= RAM_X4_RD_ADDR & in5ChSel; -- 2023/05/25 for M10k usage

--  CO9is0_sig <= '1' when W5Addr(10 downto 0) = "11111111111" or CnnStart = '1' else '0';
  CO9is0_sig <= '1' when W5Addr(10 downto 0) = "00000000000" else '0';

  W5Addr_H_m1 <= W5Addr_H - 1;
  RAM_X5_WR_ADDR <= RAM_X4_RD_ADDR & W5Addr_H_m1(2 downto 0); -- 2023/06/17 from W5Addr_H_dly

  RAM_X5_WR_N <= '0' when W5Addr_H_m1(3) = '0' and in5ChSel = "1111111" else '1';

--------------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                CO9is0 <= '0';
        elsif( CLK'event and CLK = '1' ) then
                CO9is0 <= CO9is0_sig;
        end if;
end process;

---------------------------------------------------------------------------------
ROM_W5_00_inst : ROM_W5_00 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_00 );

ROM_W5_01_inst : ROM_W5_01 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_01 );

ROM_W5_02_inst : ROM_W5_02 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_02 );

ROM_W5_03_inst : ROM_W5_03 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_03 );

ROM_W5_04_inst : ROM_W5_04 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_04 );

ROM_W5_05_inst : ROM_W5_05 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_05 );

ROM_W5_06_inst : ROM_W5_06 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_06 );

ROM_W5_07_inst : ROM_W5_07 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_07 );

ROM_W5_08_inst : ROM_W5_08 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_08 );

ROM_W5_09_inst : ROM_W5_09 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_09 );

ROM_W5_10_inst : ROM_W5_10 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_10 );

ROM_W5_11_inst : ROM_W5_11 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_11 );

ROM_W5_12_inst : ROM_W5_12 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_12 );

ROM_W5_13_inst : ROM_W5_13 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_13 );

ROM_W5_14_inst : ROM_W5_14 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_14 );

ROM_W5_15_inst : ROM_W5_15 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data_15 );

-------------------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    in5ChSel_dly <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    in5ChSel_dly <= in5ChSel;
  end if;
end process;

---------------------------------------------------------------------------------
  romdata_00 <=  
      W5Data_00(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_00(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_00(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_00(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_00(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_00(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_00(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_00(1 downto 0); 

  romdata_01 <=  
      W5Data_01(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_01(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_01(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_01(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_01(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_01(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_01(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_01(1 downto 0); 

  romdata_02 <=  
      W5Data_02(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_02(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_02(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_02(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_02(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_02(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_02(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_02(1 downto 0); 

  romdata_03 <=  
      W5Data_03(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_03(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_03(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_03(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_03(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_03(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_03(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_03(1 downto 0); 

  romdata_04 <=  
      W5Data_04(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_04(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_04(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_04(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_04(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_04(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_04(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_04(1 downto 0); 

  romdata_05 <=  
      W5Data_05(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_05(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_05(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_05(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_05(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_05(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_05(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_05(1 downto 0); 

  romdata_06 <=  
      W5Data_06(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_06(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_06(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_06(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_06(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_06(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_06(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_06(1 downto 0); 

  romdata_07 <=  
      W5Data_07(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_07(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_07(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_07(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_07(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_07(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_07(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_07(1 downto 0); 

  romdata_08 <=  
      W5Data_08(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_08(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_08(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_08(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_08(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_08(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_08(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_08(1 downto 0); 

  romdata_09 <=  
      W5Data_09(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_09(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_09(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_09(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_09(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_09(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_09(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_09(1 downto 0); 

  romdata_10 <=  
      W5Data_10(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_10(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_10(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_10(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_10(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_10(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_10(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_10(1 downto 0); 

  romdata_11 <=  
      W5Data_11(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_11(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_11(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_11(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_11(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_11(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_11(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_11(1 downto 0); 

  romdata_12 <=  
      W5Data_12(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_12(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_12(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_12(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_12(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_12(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_12(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_12(1 downto 0); 

  romdata_13 <=  
      W5Data_13(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_13(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_13(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_13(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_13(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_13(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_13(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_13(1 downto 0); 

  romdata_14 <=  
      W5Data_14(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_14(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_14(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_14(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_14(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_14(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_14(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_14(1 downto 0); 

  romdata_15 <=  
      W5Data_15(15 downto 14) when W5DataSel(2 downto 0) = "000" else 
      W5Data_15(13 downto 12) when W5DataSel(2 downto 0) = "001" else 
      W5Data_15(11 downto 10) when W5DataSel(2 downto 0) = "010" else 
      W5Data_15(9 downto 8) when W5DataSel(2 downto 0) = "011" else 
      W5Data_15(7 downto 6) when W5DataSel(2 downto 0) = "100" else 
      W5Data_15(5 downto 4) when W5DataSel(2 downto 0) = "101" else 
      W5Data_15(3 downto 2) when W5DataSel(2 downto 0) = "110" else 
      W5Data_15(1 downto 0); 

---------------------------------------------------------------------------------
dotpro_00 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_00 );

dotpro_01 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_01 );

dotpro_02 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_02 );

dotpro_03 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_03 );

dotpro_04 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_04,
  CO9is0 => CO9is0,
  MATMULOUT => mac_04 );

dotpro_05 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_05,
  CO9is0 => CO9is0,
  MATMULOUT => mac_05 );

dotpro_06 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_06,
  CO9is0 => CO9is0,
  MATMULOUT => mac_06 );

dotpro_07 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_07,
  CO9is0 => CO9is0,
  MATMULOUT => mac_07 );

dotpro_08 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_08,
  CO9is0 => CO9is0,
  MATMULOUT => mac_08 );

dotpro_09 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_09,
  CO9is0 => CO9is0,
  MATMULOUT => mac_09 );

dotpro_10 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_10,
  CO9is0 => CO9is0,
  MATMULOUT => mac_10 );

dotpro_11 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_11,
  CO9is0 => CO9is0,
  MATMULOUT => mac_11 );

dotpro_12 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_12,
  CO9is0 => CO9is0,
  MATMULOUT => mac_12 );

dotpro_13 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_13,
  CO9is0 => CO9is0,
  MATMULOUT => mac_13 );

dotpro_14 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_14,
  CO9is0 => CO9is0,
  MATMULOUT => mac_14 );

dotpro_15 : dotpro_w5 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch,
  MatRomData => romdata_15,
  CO9is0 => CO9is0,
  MATMULOUT => mac_15 );


---------------------------------------------------------------------------------
  MacShift_00 <= mac_00 & "0000"; 
  MacShift_01 <= mac_01 & "0000"; 
  MacShift_02 <= mac_02 & "0000"; 
  MacShift_03 <= mac_03 & "0000"; 
  MacShift_04 <= mac_04 & "0000"; 
  MacShift_05 <= mac_05 & "0000"; 
  MacShift_06 <= mac_06 & "0000"; 
  MacShift_07 <= mac_07 & "0000"; 
  MacShift_08 <= mac_08 & "0000"; 
  MacShift_09 <= mac_09 & "0000"; 
  MacShift_10 <= mac_10 & "0000"; 
  MacShift_11 <= mac_11 & "0000"; 
  MacShift_12 <= mac_12 & "0000"; 
  MacShift_13 <= mac_13 & "0000"; 
  MacShift_14 <= mac_14 & "0000"; 
  MacShift_15 <= mac_15 & "0000"; 

  BIAS_000 <= CONV_std_logic_vector(-1006 , 18);
  BIAS_001 <= CONV_std_logic_vector(-370 , 18);
  BIAS_002 <= CONV_std_logic_vector( 1455 , 18);
  BIAS_003 <= CONV_std_logic_vector( 277 , 18);
  BIAS_004 <= CONV_std_logic_vector( 1096 , 18);
  BIAS_005 <= CONV_std_logic_vector(-113 , 18);
  BIAS_006 <= CONV_std_logic_vector(-565 , 18);
  BIAS_007 <= CONV_std_logic_vector(-126 , 18);
  BIAS_008 <= CONV_std_logic_vector(-866 , 18);
  BIAS_009 <= CONV_std_logic_vector(-166 , 18);
  BIAS_010 <= CONV_std_logic_vector(-612 , 18);
  BIAS_011 <= CONV_std_logic_vector(-807 , 18);
  BIAS_012 <= CONV_std_logic_vector(-807 , 18);
  BIAS_013 <= CONV_std_logic_vector(-513 , 18);
  BIAS_014 <= CONV_std_logic_vector(-730 , 18);
  BIAS_015 <= CONV_std_logic_vector( 191 , 18);
  BIAS_016 <= CONV_std_logic_vector(-281 , 18);
  BIAS_017 <= CONV_std_logic_vector(-887 , 18);
  BIAS_018 <= CONV_std_logic_vector(-147 , 18);
  BIAS_019 <= CONV_std_logic_vector(-933 , 18);
  BIAS_020 <= CONV_std_logic_vector( 267 , 18);
  BIAS_021 <= CONV_std_logic_vector(-1174 , 18);
  BIAS_022 <= CONV_std_logic_vector( 258 , 18);
  BIAS_023 <= CONV_std_logic_vector( 712 , 18);
  BIAS_024 <= CONV_std_logic_vector(-181 , 18);
  BIAS_025 <= CONV_std_logic_vector(-182 , 18);
  BIAS_026 <= CONV_std_logic_vector(-395 , 18);
  BIAS_027 <= CONV_std_logic_vector(-749 , 18);
  BIAS_028 <= CONV_std_logic_vector(-960 , 18);
  BIAS_029 <= CONV_std_logic_vector( 6 , 18);
  BIAS_030 <= CONV_std_logic_vector( 100 , 18);
  BIAS_031 <= CONV_std_logic_vector( 767 , 18);
  BIAS_032 <= CONV_std_logic_vector(-370 , 18);
  BIAS_033 <= CONV_std_logic_vector(-1278 , 18);
  BIAS_034 <= CONV_std_logic_vector(-71 , 18);
  BIAS_035 <= CONV_std_logic_vector(-568 , 18);
  BIAS_036 <= CONV_std_logic_vector( 101 , 18);
  BIAS_037 <= CONV_std_logic_vector(-232 , 18);
  BIAS_038 <= CONV_std_logic_vector(-694 , 18);
  BIAS_039 <= CONV_std_logic_vector( 51 , 18);
  BIAS_040 <= CONV_std_logic_vector(-708 , 18);
  BIAS_041 <= CONV_std_logic_vector( 849 , 18);
  BIAS_042 <= CONV_std_logic_vector(-888 , 18);
  BIAS_043 <= CONV_std_logic_vector( 848 , 18);
  BIAS_044 <= CONV_std_logic_vector(-300 , 18);
  BIAS_045 <= CONV_std_logic_vector( 273 , 18);
  BIAS_046 <= CONV_std_logic_vector( 454 , 18);
  BIAS_047 <= CONV_std_logic_vector( 395 , 18);
  BIAS_048 <= CONV_std_logic_vector( 627 , 18);
  BIAS_049 <= CONV_std_logic_vector(-192 , 18);
  BIAS_050 <= CONV_std_logic_vector(-881 , 18);
  BIAS_051 <= CONV_std_logic_vector(-508 , 18);
  BIAS_052 <= CONV_std_logic_vector( 15 , 18);
  BIAS_053 <= CONV_std_logic_vector(-707 , 18);
  BIAS_054 <= CONV_std_logic_vector(-769 , 18);
  BIAS_055 <= CONV_std_logic_vector( 58 , 18);
  BIAS_056 <= CONV_std_logic_vector( 756 , 18);
  BIAS_057 <= CONV_std_logic_vector(-59 , 18);
  BIAS_058 <= CONV_std_logic_vector(-574 , 18);
  BIAS_059 <= CONV_std_logic_vector( 240 , 18);
  BIAS_060 <= CONV_std_logic_vector( 179 , 18);
  BIAS_061 <= CONV_std_logic_vector(-163 , 18);
  BIAS_062 <= CONV_std_logic_vector(-341 , 18);
  BIAS_063 <= CONV_std_logic_vector( 121 , 18);
  BIAS_064 <= CONV_std_logic_vector(-706 , 18);
  BIAS_065 <= CONV_std_logic_vector(-1201 , 18);
  BIAS_066 <= CONV_std_logic_vector( 418 , 18);
  BIAS_067 <= CONV_std_logic_vector( 284 , 18);
  BIAS_068 <= CONV_std_logic_vector(-207 , 18);
  BIAS_069 <= CONV_std_logic_vector(-488 , 18);
  BIAS_070 <= CONV_std_logic_vector(-1068 , 18);
  BIAS_071 <= CONV_std_logic_vector( 28 , 18);
  BIAS_072 <= CONV_std_logic_vector(-280 , 18);
  BIAS_073 <= CONV_std_logic_vector(-203 , 18);
  BIAS_074 <= CONV_std_logic_vector(-334 , 18);
  BIAS_075 <= CONV_std_logic_vector(-361 , 18);
  BIAS_076 <= CONV_std_logic_vector( 388 , 18);
  BIAS_077 <= CONV_std_logic_vector(-225 , 18);
  BIAS_078 <= CONV_std_logic_vector( 138 , 18);
  BIAS_079 <= CONV_std_logic_vector( 239 , 18);
  BIAS_080 <= CONV_std_logic_vector(-1191 , 18);
  BIAS_081 <= CONV_std_logic_vector(-97 , 18);
  BIAS_082 <= CONV_std_logic_vector( 713 , 18);
  BIAS_083 <= CONV_std_logic_vector( 395 , 18);
  BIAS_084 <= CONV_std_logic_vector(-596 , 18);
  BIAS_085 <= CONV_std_logic_vector(-538 , 18);
  BIAS_086 <= CONV_std_logic_vector( 862 , 18);
  BIAS_087 <= CONV_std_logic_vector(-273 , 18);
  BIAS_088 <= CONV_std_logic_vector(-922 , 18);
  BIAS_089 <= CONV_std_logic_vector(-472 , 18);
  BIAS_090 <= CONV_std_logic_vector(-581 , 18);
  BIAS_091 <= CONV_std_logic_vector( 135 , 18);
  BIAS_092 <= CONV_std_logic_vector( 162 , 18);
  BIAS_093 <= CONV_std_logic_vector(-788 , 18);
  BIAS_094 <= CONV_std_logic_vector(-1144 , 18);
  BIAS_095 <= CONV_std_logic_vector(-822 , 18);
  BIAS_096 <= CONV_std_logic_vector( 99 , 18);
  BIAS_097 <= CONV_std_logic_vector(-335 , 18);
  BIAS_098 <= CONV_std_logic_vector( 304 , 18);
  BIAS_099 <= CONV_std_logic_vector(-302 , 18);
  BIAS_100 <= CONV_std_logic_vector(-272 , 18);
  BIAS_101 <= CONV_std_logic_vector(-260 , 18);
  BIAS_102 <= CONV_std_logic_vector( 760 , 18);
  BIAS_103 <= CONV_std_logic_vector(-739 , 18);
  BIAS_104 <= CONV_std_logic_vector(-778 , 18);
  BIAS_105 <= CONV_std_logic_vector(-490 , 18);
  BIAS_106 <= CONV_std_logic_vector(-598 , 18);
  BIAS_107 <= CONV_std_logic_vector( 265 , 18);
  BIAS_108 <= CONV_std_logic_vector(-9 , 18);
  BIAS_109 <= CONV_std_logic_vector(-53 , 18);
  BIAS_110 <= CONV_std_logic_vector(-103 , 18);
  BIAS_111 <= CONV_std_logic_vector( 379 , 18);
  BIAS_112 <= CONV_std_logic_vector( 0 , 18);
  BIAS_113 <= CONV_std_logic_vector( 199 , 18);
  BIAS_114 <= CONV_std_logic_vector(-925 , 18);
  BIAS_115 <= CONV_std_logic_vector( 110 , 18);
  BIAS_116 <= CONV_std_logic_vector( 300 , 18);
  BIAS_117 <= CONV_std_logic_vector(-390 , 18);
  BIAS_118 <= CONV_std_logic_vector(-1021 , 18);
  BIAS_119 <= CONV_std_logic_vector(-216 , 18);
  BIAS_120 <= CONV_std_logic_vector(-927 , 18);
  BIAS_121 <= CONV_std_logic_vector( 349 , 18);
  BIAS_122 <= CONV_std_logic_vector( 298 , 18);
  BIAS_123 <= CONV_std_logic_vector(-903 , 18);
  BIAS_124 <= CONV_std_logic_vector(-587 , 18);
  BIAS_125 <= CONV_std_logic_vector(-157 , 18);
  BIAS_126 <= CONV_std_logic_vector(-455 , 18);
  BIAS_127 <= CONV_std_logic_vector( 909 , 18);

--------------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                W5DataSel <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                W5DataSel <= W5Addr_H;
        end if;
end process;

process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                W5Addr_H_dly <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                if(CO9is0_sig = '1') then 
                      W5Addr_H_dly <= W5Addr_H;
                end if;
        end if;
end process;

BIAS_SEL_00 <= 
    BIAS_000 when W5Addr_H_m1 = "0000" else
    BIAS_001 when W5Addr_H_m1 = "0001" else
    BIAS_002 when W5Addr_H_m1 = "0010" else
    BIAS_003 when W5Addr_H_m1 = "0011" else
    BIAS_004 when W5Addr_H_m1 = "0100" else
    BIAS_005 when W5Addr_H_m1 = "0101" else
    BIAS_006 when W5Addr_H_m1 = "0110" else
    BIAS_007 ;

BIAS_SEL_01 <= 
    BIAS_008 when W5Addr_H_m1 = "0000" else
    BIAS_009 when W5Addr_H_m1 = "0001" else
    BIAS_010 when W5Addr_H_m1 = "0010" else
    BIAS_011 when W5Addr_H_m1 = "0011" else
    BIAS_012 when W5Addr_H_m1 = "0100" else
    BIAS_013 when W5Addr_H_m1 = "0101" else
    BIAS_014 when W5Addr_H_m1 = "0110" else
    BIAS_015 ;

BIAS_SEL_02 <= 
    BIAS_016 when W5Addr_H_m1 = "0000" else
    BIAS_017 when W5Addr_H_m1 = "0001" else
    BIAS_018 when W5Addr_H_m1 = "0010" else
    BIAS_019 when W5Addr_H_m1 = "0011" else
    BIAS_020 when W5Addr_H_m1 = "0100" else
    BIAS_021 when W5Addr_H_m1 = "0101" else
    BIAS_022 when W5Addr_H_m1 = "0110" else
    BIAS_023 ;

BIAS_SEL_03 <= 
    BIAS_024 when W5Addr_H_m1 = "0000" else
    BIAS_025 when W5Addr_H_m1 = "0001" else
    BIAS_026 when W5Addr_H_m1 = "0010" else
    BIAS_027 when W5Addr_H_m1 = "0011" else
    BIAS_028 when W5Addr_H_m1 = "0100" else
    BIAS_029 when W5Addr_H_m1 = "0101" else
    BIAS_030 when W5Addr_H_m1 = "0110" else
    BIAS_031 ;

BIAS_SEL_04 <= 
    BIAS_032 when W5Addr_H_m1 = "0000" else
    BIAS_033 when W5Addr_H_m1 = "0001" else
    BIAS_034 when W5Addr_H_m1 = "0010" else
    BIAS_035 when W5Addr_H_m1 = "0011" else
    BIAS_036 when W5Addr_H_m1 = "0100" else
    BIAS_037 when W5Addr_H_m1 = "0101" else
    BIAS_038 when W5Addr_H_m1 = "0110" else
    BIAS_039 ;

BIAS_SEL_05 <= 
    BIAS_040 when W5Addr_H_m1 = "0000" else
    BIAS_041 when W5Addr_H_m1 = "0001" else
    BIAS_042 when W5Addr_H_m1 = "0010" else
    BIAS_043 when W5Addr_H_m1 = "0011" else
    BIAS_044 when W5Addr_H_m1 = "0100" else
    BIAS_045 when W5Addr_H_m1 = "0101" else
    BIAS_046 when W5Addr_H_m1 = "0110" else
    BIAS_047 ;

BIAS_SEL_06 <= 
    BIAS_048 when W5Addr_H_m1 = "0000" else
    BIAS_049 when W5Addr_H_m1 = "0001" else
    BIAS_050 when W5Addr_H_m1 = "0010" else
    BIAS_051 when W5Addr_H_m1 = "0011" else
    BIAS_052 when W5Addr_H_m1 = "0100" else
    BIAS_053 when W5Addr_H_m1 = "0101" else
    BIAS_054 when W5Addr_H_m1 = "0110" else
    BIAS_055 ;

BIAS_SEL_07 <= 
    BIAS_056 when W5Addr_H_m1 = "0000" else
    BIAS_057 when W5Addr_H_m1 = "0001" else
    BIAS_058 when W5Addr_H_m1 = "0010" else
    BIAS_059 when W5Addr_H_m1 = "0011" else
    BIAS_060 when W5Addr_H_m1 = "0100" else
    BIAS_061 when W5Addr_H_m1 = "0101" else
    BIAS_062 when W5Addr_H_m1 = "0110" else
    BIAS_063 ;

BIAS_SEL_08 <= 
    BIAS_064 when W5Addr_H_m1 = "0000" else
    BIAS_065 when W5Addr_H_m1 = "0001" else
    BIAS_066 when W5Addr_H_m1 = "0010" else
    BIAS_067 when W5Addr_H_m1 = "0011" else
    BIAS_068 when W5Addr_H_m1 = "0100" else
    BIAS_069 when W5Addr_H_m1 = "0101" else
    BIAS_070 when W5Addr_H_m1 = "0110" else
    BIAS_071 ;

BIAS_SEL_09 <= 
    BIAS_072 when W5Addr_H_m1 = "0000" else
    BIAS_073 when W5Addr_H_m1 = "0001" else
    BIAS_074 when W5Addr_H_m1 = "0010" else
    BIAS_075 when W5Addr_H_m1 = "0011" else
    BIAS_076 when W5Addr_H_m1 = "0100" else
    BIAS_077 when W5Addr_H_m1 = "0101" else
    BIAS_078 when W5Addr_H_m1 = "0110" else
    BIAS_079 ;

BIAS_SEL_10 <= 
    BIAS_080 when W5Addr_H_m1 = "0000" else
    BIAS_081 when W5Addr_H_m1 = "0001" else
    BIAS_082 when W5Addr_H_m1 = "0010" else
    BIAS_083 when W5Addr_H_m1 = "0011" else
    BIAS_084 when W5Addr_H_m1 = "0100" else
    BIAS_085 when W5Addr_H_m1 = "0101" else
    BIAS_086 when W5Addr_H_m1 = "0110" else
    BIAS_087 ;

BIAS_SEL_11 <= 
    BIAS_088 when W5Addr_H_m1 = "0000" else
    BIAS_089 when W5Addr_H_m1 = "0001" else
    BIAS_090 when W5Addr_H_m1 = "0010" else
    BIAS_091 when W5Addr_H_m1 = "0011" else
    BIAS_092 when W5Addr_H_m1 = "0100" else
    BIAS_093 when W5Addr_H_m1 = "0101" else
    BIAS_094 when W5Addr_H_m1 = "0110" else
    BIAS_095 ;

BIAS_SEL_12 <= 
    BIAS_096 when W5Addr_H_m1 = "0000" else
    BIAS_097 when W5Addr_H_m1 = "0001" else
    BIAS_098 when W5Addr_H_m1 = "0010" else
    BIAS_099 when W5Addr_H_m1 = "0011" else
    BIAS_100 when W5Addr_H_m1 = "0100" else
    BIAS_101 when W5Addr_H_m1 = "0101" else
    BIAS_102 when W5Addr_H_m1 = "0110" else
    BIAS_103 ;

BIAS_SEL_13 <= 
    BIAS_104 when W5Addr_H_m1 = "0000" else
    BIAS_105 when W5Addr_H_m1 = "0001" else
    BIAS_106 when W5Addr_H_m1 = "0010" else
    BIAS_107 when W5Addr_H_m1 = "0011" else
    BIAS_108 when W5Addr_H_m1 = "0100" else
    BIAS_109 when W5Addr_H_m1 = "0101" else
    BIAS_110 when W5Addr_H_m1 = "0110" else
    BIAS_111 ;

BIAS_SEL_14 <= 
    BIAS_112 when W5Addr_H_m1 = "0000" else
    BIAS_113 when W5Addr_H_m1 = "0001" else
    BIAS_114 when W5Addr_H_m1 = "0010" else
    BIAS_115 when W5Addr_H_m1 = "0011" else
    BIAS_116 when W5Addr_H_m1 = "0100" else
    BIAS_117 when W5Addr_H_m1 = "0101" else
    BIAS_118 when W5Addr_H_m1 = "0110" else
    BIAS_119 ;

BIAS_SEL_15 <= 
    BIAS_120 when W5Addr_H_m1 = "0000" else
    BIAS_121 when W5Addr_H_m1 = "0001" else
    BIAS_122 when W5Addr_H_m1 = "0010" else
    BIAS_123 when W5Addr_H_m1 = "0011" else
    BIAS_124 when W5Addr_H_m1 = "0100" else
    BIAS_125 when W5Addr_H_m1 = "0101" else
    BIAS_126 when W5Addr_H_m1 = "0110" else
    BIAS_127 ;


  plusBias_00 <= MacShift_00 + BIAS_SEL_00;
  truncate_00 <= plusBias_00(17 downto 8);

  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);


  plusBias_01 <= MacShift_01 + BIAS_SEL_01;
  truncate_01 <= plusBias_01(17 downto 8);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);


  plusBias_02 <= MacShift_02 + BIAS_SEL_02;
  truncate_02 <= plusBias_02(17 downto 8);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);


  plusBias_03 <= MacShift_03 + BIAS_SEL_03;
  truncate_03 <= plusBias_03(17 downto 8);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);


  plusBias_04 <= MacShift_04 + BIAS_SEL_04;
  truncate_04 <= plusBias_04(17 downto 8);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);


  plusBias_05 <= MacShift_05 + BIAS_SEL_05;
  truncate_05 <= plusBias_05(17 downto 8);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);


  plusBias_06 <= MacShift_06 + BIAS_SEL_06;
  truncate_06 <= plusBias_06(17 downto 8);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);


  plusBias_07 <= MacShift_07 + BIAS_SEL_07;
  truncate_07 <= plusBias_07(17 downto 8);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);


  plusBias_08 <= MacShift_08 + BIAS_SEL_08;
  truncate_08 <= plusBias_08(17 downto 8);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);


  plusBias_09 <= MacShift_09 + BIAS_SEL_09;
  truncate_09 <= plusBias_09(17 downto 8);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);


  plusBias_10 <= MacShift_10 + BIAS_SEL_10;
  truncate_10 <= plusBias_10(17 downto 8);

  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);


  plusBias_11 <= MacShift_11 + BIAS_SEL_11;
  truncate_11 <= plusBias_11(17 downto 8);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);


  plusBias_12 <= MacShift_12 + BIAS_SEL_12;
  truncate_12 <= plusBias_12(17 downto 8);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);


  plusBias_13 <= MacShift_13 + BIAS_SEL_13;
  truncate_13 <= plusBias_13(17 downto 8);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);


  plusBias_14 <= MacShift_14 + BIAS_SEL_14;
  truncate_14 <= plusBias_14(17 downto 8);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);


  plusBias_15 <= MacShift_15 + BIAS_SEL_15;
  truncate_15 <= plusBias_15(17 downto 8);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);


ReluAll <= 
        Relu_00 when RAM_X4_RD_ADDR = "0000" else
        Relu_01 when RAM_X4_RD_ADDR = "0001" else
        Relu_02 when RAM_X4_RD_ADDR = "0010" else
        Relu_03 when RAM_X4_RD_ADDR = "0011" else
        Relu_04 when RAM_X4_RD_ADDR = "0100" else
        Relu_05 when RAM_X4_RD_ADDR = "0101" else
        Relu_06 when RAM_X4_RD_ADDR = "0110" else
        Relu_07 when RAM_X4_RD_ADDR = "0111" else
        Relu_08 when RAM_X4_RD_ADDR = "1000" else
        Relu_09 when RAM_X4_RD_ADDR = "1001" else
        Relu_10 when RAM_X4_RD_ADDR = "1010" else
        Relu_11 when RAM_X4_RD_ADDR = "1011" else
        Relu_12 when RAM_X4_RD_ADDR = "1100" else
        Relu_13 when RAM_X4_RD_ADDR = "1101" else
        Relu_14 when RAM_X4_RD_ADDR = "1110" else
        Relu_15 ; -- when RAM_X4_RD_ADDR = "1111" else

---------------------------------------------------------
  LAY5OUT <= ReluAll;

end RTL;

