-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2019/02/13

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity judge is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  HAIKEIMODE : in std_logic;                          -- reset (active low)
  AIN : in std_logic_vector(13 downto 0); 
  NUMBER : in std_logic_vector(5 downto 0); 
  GATE_N : in std_logic; 
  START : in std_logic; 
  DISPJUDGE : in std_logic;
  THRESH : in std_logic_vector(13 downto 0); 
  ANS_WR_ADDR : in std_logic_vector(8 downto 0); 
  ANS_WR_N : out std_logic;
  ANSPOS : out std_logic_vector(8 downto 0); 
  ANSVALUE : out std_logic_vector(13 downto 0);
  ANSWER : Out std_logic_vector(5 downto 0) );  
end judge;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of judge is

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal MAXDATA : std_logic_vector(13 downto 0);
signal b_bt_a : std_logic;
signal MAXN : std_logic_vector(5 downto 0);
signal ANS_WR_N_sig : std_logic;
signal START_dly, START_edge : std_logic;
signal data_bt_100 : std_logic;
signal addout, acc, AIN_abs : std_logic_vector(15 downto 0);
signal GATE : std_logic;
signal THRESH_m4 : std_logic_vector(13 downto 0);
signal disp_haikei : std_logic;

begin

  GATE <= not GATE_N;
 
  AIN_abs <= "000" & AIN(12 downto 0) when AIN(13) = '0' 
		else "000" & (not AIN(12)) & (not AIN(11)) & (not AIN(10)) 
		& (not AIN(9)) & (not AIN(8)) & (not AIN(7)) & (not AIN(6)) & (not AIN(5)) 
		& (not AIN(4)) & (not AIN(3)) & (not AIN(2)) & (not AIN(1)) & (not AIN(0));

  addout <= AIN_abs + acc;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		acc <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(START_edge = '1') then
			acc <= (others => '0');
		elsif(GATE = '1') then
			acc <= addout;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		START_dly <= '1';
	elsif( CLK'event and CLK = '1' ) then
		START_dly <= START;
	end if;  
end process;

  START_edge <= '1' when START = '1' and START_dly = '0' else '0';

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		MAXDATA <= (others => '0');--"10000000000000";
	elsif( CLK'event and CLK = '1' ) then
		if(START_edge = '1') then
			MAXDATA <= (others => '0');--"10000000000000";
		elsif(GATE = '1' and b_bt_a = '1') then
			MAXDATA <= AIN;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		MAXN <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(START_edge = '1') then
			MAXN <= (others => '1');
		elsif(GATE = '1' and b_bt_a = '1') then
			MAXN <= NUMBER;
		end if;
	end if;  
end process;

---------------------------------------------------------
  b_bt_a <= '1' when AIN > MAXDATA else '0';

--  data_bt_100 <= '1' when MAXDATA >= THRESH else '0';
  data_bt_100 <= '1' when (MAXDATA >= THRESH and MAXN /= "000000") 
                       or (MAXDATA >= THRESH_m4 and MAXN = "000000") 
                  else '0';

  THRESH_m4 <= THRESH;--(12 downto 0) & "0";

  ANS_WR_N_sig <= '0' when START = '1' and DISPJUDGE = '1' else '1'; -- added dispjudge 2020/06/16

-----------------------------------------------------------
--process( RST_N, CLK )  
--begin 
--	if( RST_N = '0' ) then
--		ANS_WR_N <= '0';
--	elsif( CLK'event and CLK = '1' ) then
		ANS_WR_N <= ANS_WR_N_sig; -- removed latch 2023/06/26
--	end if;  
--end process;

-- to hide HAIKEI detection 2023/10/14
  disp_haikei <= '0' when HAIKEIMODE = '1' and 
           not (MAXN = "000000" or MAXN = "000001" or MAXN = "000010" or MAXN = "000011" or MAXN = "000100" or MAXN = "000101" or MAXN = "000110" or MAXN = "000111" or MAXN = "001000" or MAXN = "001001") else '1';
  
---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ANSWER <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(NUMBER = 43) then
			if(data_bt_100 = '1' and disp_haikei = '1') then
				ANSWER <= MAXN;
			else 
				ANSWER <= "111111";
			end if;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ANSVALUE <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(NUMBER = 43) then
			ANSVALUE <= MAXDATA;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		ANSPOS <= (others => '1');
	elsif( CLK'event and CLK = '1' ) then
		if(NUMBER = 43) then
			ANSPOS <= ANS_WR_ADDR;
		end if;
	end if;  
end process;

end RTL;

