-- The layer4 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/07/01

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer4 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  P_WR_ADDR : in std_logic_vector(3 downto 0); 
  P_WR_N : in std_logic; 
  CO9 : in std_logic_vector(3 downto 0); 
  CnnStart : IN std_logic;
  in3ChSel : in std_logic_vector(3 downto 0);
  LAY4IN : in std_logic_vector(319 downto 0); 
  LAY4OUT : out std_logic_vector(639 downto 0)); 
end conv_layer4;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer4 is

component ROM_W4_00
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(7 downto 0);
          DATA : Out std_logic_vector(255 downto 0) );
end component;

component ROM_W4_01
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(7 downto 0);
          DATA : Out std_logic_vector(255 downto 0) );
end component;

component ROM_W4_02
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(7 downto 0);
          DATA : Out std_logic_vector(255 downto 0) );
end component;

component ROM_W4_03
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(7 downto 0);
          DATA : Out std_logic_vector(255 downto 0) );
end component;

component dotpro_w4
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(1 downto 0);
  CO9is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

component pooling_lay4
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(639 downto 0);
  WR_N : in std_logic;
  WR_ADDR : in std_logic_vector(3 downto 0);
  Poolout : Out std_logic_vector(639 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal W4Data_00, W4Data_01, W4Data_02, W4Data_03 : std_logic_vector(255 downto 0);

signal W4Addr : std_logic_vector(7 downto 0);

signal coeff_000_00 : std_logic_vector(1 downto 0);
signal mac_000_00 : std_logic_vector(13 downto 0);

signal coeff_000_01 : std_logic_vector(1 downto 0);
signal mac_000_01 : std_logic_vector(13 downto 0);

signal coeff_000_02 : std_logic_vector(1 downto 0);
signal mac_000_02 : std_logic_vector(13 downto 0);

signal coeff_000_03 : std_logic_vector(1 downto 0);
signal mac_000_03 : std_logic_vector(13 downto 0);

signal coeff_001_00 : std_logic_vector(1 downto 0);
signal mac_001_00 : std_logic_vector(13 downto 0);

signal coeff_001_01 : std_logic_vector(1 downto 0);
signal mac_001_01 : std_logic_vector(13 downto 0);

signal coeff_001_02 : std_logic_vector(1 downto 0);
signal mac_001_02 : std_logic_vector(13 downto 0);

signal coeff_001_03 : std_logic_vector(1 downto 0);
signal mac_001_03 : std_logic_vector(13 downto 0);

signal coeff_002_00 : std_logic_vector(1 downto 0);
signal mac_002_00 : std_logic_vector(13 downto 0);

signal coeff_002_01 : std_logic_vector(1 downto 0);
signal mac_002_01 : std_logic_vector(13 downto 0);

signal coeff_002_02 : std_logic_vector(1 downto 0);
signal mac_002_02 : std_logic_vector(13 downto 0);

signal coeff_002_03 : std_logic_vector(1 downto 0);
signal mac_002_03 : std_logic_vector(13 downto 0);

signal coeff_003_00 : std_logic_vector(1 downto 0);
signal mac_003_00 : std_logic_vector(13 downto 0);

signal coeff_003_01 : std_logic_vector(1 downto 0);
signal mac_003_01 : std_logic_vector(13 downto 0);

signal coeff_003_02 : std_logic_vector(1 downto 0);
signal mac_003_02 : std_logic_vector(13 downto 0);

signal coeff_003_03 : std_logic_vector(1 downto 0);
signal mac_003_03 : std_logic_vector(13 downto 0);

signal coeff_004_00 : std_logic_vector(1 downto 0);
signal mac_004_00 : std_logic_vector(13 downto 0);

signal coeff_004_01 : std_logic_vector(1 downto 0);
signal mac_004_01 : std_logic_vector(13 downto 0);

signal coeff_004_02 : std_logic_vector(1 downto 0);
signal mac_004_02 : std_logic_vector(13 downto 0);

signal coeff_004_03 : std_logic_vector(1 downto 0);
signal mac_004_03 : std_logic_vector(13 downto 0);

signal coeff_005_00 : std_logic_vector(1 downto 0);
signal mac_005_00 : std_logic_vector(13 downto 0);

signal coeff_005_01 : std_logic_vector(1 downto 0);
signal mac_005_01 : std_logic_vector(13 downto 0);

signal coeff_005_02 : std_logic_vector(1 downto 0);
signal mac_005_02 : std_logic_vector(13 downto 0);

signal coeff_005_03 : std_logic_vector(1 downto 0);
signal mac_005_03 : std_logic_vector(13 downto 0);

signal coeff_006_00 : std_logic_vector(1 downto 0);
signal mac_006_00 : std_logic_vector(13 downto 0);

signal coeff_006_01 : std_logic_vector(1 downto 0);
signal mac_006_01 : std_logic_vector(13 downto 0);

signal coeff_006_02 : std_logic_vector(1 downto 0);
signal mac_006_02 : std_logic_vector(13 downto 0);

signal coeff_006_03 : std_logic_vector(1 downto 0);
signal mac_006_03 : std_logic_vector(13 downto 0);

signal coeff_007_00 : std_logic_vector(1 downto 0);
signal mac_007_00 : std_logic_vector(13 downto 0);

signal coeff_007_01 : std_logic_vector(1 downto 0);
signal mac_007_01 : std_logic_vector(13 downto 0);

signal coeff_007_02 : std_logic_vector(1 downto 0);
signal mac_007_02 : std_logic_vector(13 downto 0);

signal coeff_007_03 : std_logic_vector(1 downto 0);
signal mac_007_03 : std_logic_vector(13 downto 0);

signal coeff_008_00 : std_logic_vector(1 downto 0);
signal mac_008_00 : std_logic_vector(13 downto 0);

signal coeff_008_01 : std_logic_vector(1 downto 0);
signal mac_008_01 : std_logic_vector(13 downto 0);

signal coeff_008_02 : std_logic_vector(1 downto 0);
signal mac_008_02 : std_logic_vector(13 downto 0);

signal coeff_008_03 : std_logic_vector(1 downto 0);
signal mac_008_03 : std_logic_vector(13 downto 0);

signal coeff_009_00 : std_logic_vector(1 downto 0);
signal mac_009_00 : std_logic_vector(13 downto 0);

signal coeff_009_01 : std_logic_vector(1 downto 0);
signal mac_009_01 : std_logic_vector(13 downto 0);

signal coeff_009_02 : std_logic_vector(1 downto 0);
signal mac_009_02 : std_logic_vector(13 downto 0);

signal coeff_009_03 : std_logic_vector(1 downto 0);
signal mac_009_03 : std_logic_vector(13 downto 0);

signal coeff_010_00 : std_logic_vector(1 downto 0);
signal mac_010_00 : std_logic_vector(13 downto 0);

signal coeff_010_01 : std_logic_vector(1 downto 0);
signal mac_010_01 : std_logic_vector(13 downto 0);

signal coeff_010_02 : std_logic_vector(1 downto 0);
signal mac_010_02 : std_logic_vector(13 downto 0);

signal coeff_010_03 : std_logic_vector(1 downto 0);
signal mac_010_03 : std_logic_vector(13 downto 0);

signal coeff_011_00 : std_logic_vector(1 downto 0);
signal mac_011_00 : std_logic_vector(13 downto 0);

signal coeff_011_01 : std_logic_vector(1 downto 0);
signal mac_011_01 : std_logic_vector(13 downto 0);

signal coeff_011_02 : std_logic_vector(1 downto 0);
signal mac_011_02 : std_logic_vector(13 downto 0);

signal coeff_011_03 : std_logic_vector(1 downto 0);
signal mac_011_03 : std_logic_vector(13 downto 0);

signal coeff_012_00 : std_logic_vector(1 downto 0);
signal mac_012_00 : std_logic_vector(13 downto 0);

signal coeff_012_01 : std_logic_vector(1 downto 0);
signal mac_012_01 : std_logic_vector(13 downto 0);

signal coeff_012_02 : std_logic_vector(1 downto 0);
signal mac_012_02 : std_logic_vector(13 downto 0);

signal coeff_012_03 : std_logic_vector(1 downto 0);
signal mac_012_03 : std_logic_vector(13 downto 0);

signal coeff_013_00 : std_logic_vector(1 downto 0);
signal mac_013_00 : std_logic_vector(13 downto 0);

signal coeff_013_01 : std_logic_vector(1 downto 0);
signal mac_013_01 : std_logic_vector(13 downto 0);

signal coeff_013_02 : std_logic_vector(1 downto 0);
signal mac_013_02 : std_logic_vector(13 downto 0);

signal coeff_013_03 : std_logic_vector(1 downto 0);
signal mac_013_03 : std_logic_vector(13 downto 0);

signal coeff_014_00 : std_logic_vector(1 downto 0);
signal mac_014_00 : std_logic_vector(13 downto 0);

signal coeff_014_01 : std_logic_vector(1 downto 0);
signal mac_014_01 : std_logic_vector(13 downto 0);

signal coeff_014_02 : std_logic_vector(1 downto 0);
signal mac_014_02 : std_logic_vector(13 downto 0);

signal coeff_014_03 : std_logic_vector(1 downto 0);
signal mac_014_03 : std_logic_vector(13 downto 0);

signal coeff_015_00 : std_logic_vector(1 downto 0);
signal mac_015_00 : std_logic_vector(13 downto 0);

signal coeff_015_01 : std_logic_vector(1 downto 0);
signal mac_015_01 : std_logic_vector(13 downto 0);

signal coeff_015_02 : std_logic_vector(1 downto 0);
signal mac_015_02 : std_logic_vector(13 downto 0);

signal coeff_015_03 : std_logic_vector(1 downto 0);
signal mac_015_03 : std_logic_vector(13 downto 0);

signal coeff_016_00 : std_logic_vector(1 downto 0);
signal mac_016_00 : std_logic_vector(13 downto 0);

signal coeff_016_01 : std_logic_vector(1 downto 0);
signal mac_016_01 : std_logic_vector(13 downto 0);

signal coeff_016_02 : std_logic_vector(1 downto 0);
signal mac_016_02 : std_logic_vector(13 downto 0);

signal coeff_016_03 : std_logic_vector(1 downto 0);
signal mac_016_03 : std_logic_vector(13 downto 0);

signal coeff_017_00 : std_logic_vector(1 downto 0);
signal mac_017_00 : std_logic_vector(13 downto 0);

signal coeff_017_01 : std_logic_vector(1 downto 0);
signal mac_017_01 : std_logic_vector(13 downto 0);

signal coeff_017_02 : std_logic_vector(1 downto 0);
signal mac_017_02 : std_logic_vector(13 downto 0);

signal coeff_017_03 : std_logic_vector(1 downto 0);
signal mac_017_03 : std_logic_vector(13 downto 0);

signal coeff_018_00 : std_logic_vector(1 downto 0);
signal mac_018_00 : std_logic_vector(13 downto 0);

signal coeff_018_01 : std_logic_vector(1 downto 0);
signal mac_018_01 : std_logic_vector(13 downto 0);

signal coeff_018_02 : std_logic_vector(1 downto 0);
signal mac_018_02 : std_logic_vector(13 downto 0);

signal coeff_018_03 : std_logic_vector(1 downto 0);
signal mac_018_03 : std_logic_vector(13 downto 0);

signal coeff_019_00 : std_logic_vector(1 downto 0);
signal mac_019_00 : std_logic_vector(13 downto 0);

signal coeff_019_01 : std_logic_vector(1 downto 0);
signal mac_019_01 : std_logic_vector(13 downto 0);

signal coeff_019_02 : std_logic_vector(1 downto 0);
signal mac_019_02 : std_logic_vector(13 downto 0);

signal coeff_019_03 : std_logic_vector(1 downto 0);
signal mac_019_03 : std_logic_vector(13 downto 0);

signal coeff_020_00 : std_logic_vector(1 downto 0);
signal mac_020_00 : std_logic_vector(13 downto 0);

signal coeff_020_01 : std_logic_vector(1 downto 0);
signal mac_020_01 : std_logic_vector(13 downto 0);

signal coeff_020_02 : std_logic_vector(1 downto 0);
signal mac_020_02 : std_logic_vector(13 downto 0);

signal coeff_020_03 : std_logic_vector(1 downto 0);
signal mac_020_03 : std_logic_vector(13 downto 0);

signal coeff_021_00 : std_logic_vector(1 downto 0);
signal mac_021_00 : std_logic_vector(13 downto 0);

signal coeff_021_01 : std_logic_vector(1 downto 0);
signal mac_021_01 : std_logic_vector(13 downto 0);

signal coeff_021_02 : std_logic_vector(1 downto 0);
signal mac_021_02 : std_logic_vector(13 downto 0);

signal coeff_021_03 : std_logic_vector(1 downto 0);
signal mac_021_03 : std_logic_vector(13 downto 0);

signal coeff_022_00 : std_logic_vector(1 downto 0);
signal mac_022_00 : std_logic_vector(13 downto 0);

signal coeff_022_01 : std_logic_vector(1 downto 0);
signal mac_022_01 : std_logic_vector(13 downto 0);

signal coeff_022_02 : std_logic_vector(1 downto 0);
signal mac_022_02 : std_logic_vector(13 downto 0);

signal coeff_022_03 : std_logic_vector(1 downto 0);
signal mac_022_03 : std_logic_vector(13 downto 0);

signal coeff_023_00 : std_logic_vector(1 downto 0);
signal mac_023_00 : std_logic_vector(13 downto 0);

signal coeff_023_01 : std_logic_vector(1 downto 0);
signal mac_023_01 : std_logic_vector(13 downto 0);

signal coeff_023_02 : std_logic_vector(1 downto 0);
signal mac_023_02 : std_logic_vector(13 downto 0);

signal coeff_023_03 : std_logic_vector(1 downto 0);
signal mac_023_03 : std_logic_vector(13 downto 0);

signal coeff_024_00 : std_logic_vector(1 downto 0);
signal mac_024_00 : std_logic_vector(13 downto 0);

signal coeff_024_01 : std_logic_vector(1 downto 0);
signal mac_024_01 : std_logic_vector(13 downto 0);

signal coeff_024_02 : std_logic_vector(1 downto 0);
signal mac_024_02 : std_logic_vector(13 downto 0);

signal coeff_024_03 : std_logic_vector(1 downto 0);
signal mac_024_03 : std_logic_vector(13 downto 0);

signal coeff_025_00 : std_logic_vector(1 downto 0);
signal mac_025_00 : std_logic_vector(13 downto 0);

signal coeff_025_01 : std_logic_vector(1 downto 0);
signal mac_025_01 : std_logic_vector(13 downto 0);

signal coeff_025_02 : std_logic_vector(1 downto 0);
signal mac_025_02 : std_logic_vector(13 downto 0);

signal coeff_025_03 : std_logic_vector(1 downto 0);
signal mac_025_03 : std_logic_vector(13 downto 0);

signal coeff_026_00 : std_logic_vector(1 downto 0);
signal mac_026_00 : std_logic_vector(13 downto 0);

signal coeff_026_01 : std_logic_vector(1 downto 0);
signal mac_026_01 : std_logic_vector(13 downto 0);

signal coeff_026_02 : std_logic_vector(1 downto 0);
signal mac_026_02 : std_logic_vector(13 downto 0);

signal coeff_026_03 : std_logic_vector(1 downto 0);
signal mac_026_03 : std_logic_vector(13 downto 0);

signal coeff_027_00 : std_logic_vector(1 downto 0);
signal mac_027_00 : std_logic_vector(13 downto 0);

signal coeff_027_01 : std_logic_vector(1 downto 0);
signal mac_027_01 : std_logic_vector(13 downto 0);

signal coeff_027_02 : std_logic_vector(1 downto 0);
signal mac_027_02 : std_logic_vector(13 downto 0);

signal coeff_027_03 : std_logic_vector(1 downto 0);
signal mac_027_03 : std_logic_vector(13 downto 0);

signal coeff_028_00 : std_logic_vector(1 downto 0);
signal mac_028_00 : std_logic_vector(13 downto 0);

signal coeff_028_01 : std_logic_vector(1 downto 0);
signal mac_028_01 : std_logic_vector(13 downto 0);

signal coeff_028_02 : std_logic_vector(1 downto 0);
signal mac_028_02 : std_logic_vector(13 downto 0);

signal coeff_028_03 : std_logic_vector(1 downto 0);
signal mac_028_03 : std_logic_vector(13 downto 0);

signal coeff_029_00 : std_logic_vector(1 downto 0);
signal mac_029_00 : std_logic_vector(13 downto 0);

signal coeff_029_01 : std_logic_vector(1 downto 0);
signal mac_029_01 : std_logic_vector(13 downto 0);

signal coeff_029_02 : std_logic_vector(1 downto 0);
signal mac_029_02 : std_logic_vector(13 downto 0);

signal coeff_029_03 : std_logic_vector(1 downto 0);
signal mac_029_03 : std_logic_vector(13 downto 0);

signal coeff_030_00 : std_logic_vector(1 downto 0);
signal mac_030_00 : std_logic_vector(13 downto 0);

signal coeff_030_01 : std_logic_vector(1 downto 0);
signal mac_030_01 : std_logic_vector(13 downto 0);

signal coeff_030_02 : std_logic_vector(1 downto 0);
signal mac_030_02 : std_logic_vector(13 downto 0);

signal coeff_030_03 : std_logic_vector(1 downto 0);
signal mac_030_03 : std_logic_vector(13 downto 0);

signal coeff_031_00 : std_logic_vector(1 downto 0);
signal mac_031_00 : std_logic_vector(13 downto 0);

signal coeff_031_01 : std_logic_vector(1 downto 0);
signal mac_031_01 : std_logic_vector(13 downto 0);

signal coeff_031_02 : std_logic_vector(1 downto 0);
signal mac_031_02 : std_logic_vector(13 downto 0);

signal coeff_031_03 : std_logic_vector(1 downto 0);
signal mac_031_03 : std_logic_vector(13 downto 0);

signal coeff_032_00 : std_logic_vector(1 downto 0);
signal mac_032_00 : std_logic_vector(13 downto 0);

signal coeff_032_01 : std_logic_vector(1 downto 0);
signal mac_032_01 : std_logic_vector(13 downto 0);

signal coeff_032_02 : std_logic_vector(1 downto 0);
signal mac_032_02 : std_logic_vector(13 downto 0);

signal coeff_032_03 : std_logic_vector(1 downto 0);
signal mac_032_03 : std_logic_vector(13 downto 0);

signal coeff_033_00 : std_logic_vector(1 downto 0);
signal mac_033_00 : std_logic_vector(13 downto 0);

signal coeff_033_01 : std_logic_vector(1 downto 0);
signal mac_033_01 : std_logic_vector(13 downto 0);

signal coeff_033_02 : std_logic_vector(1 downto 0);
signal mac_033_02 : std_logic_vector(13 downto 0);

signal coeff_033_03 : std_logic_vector(1 downto 0);
signal mac_033_03 : std_logic_vector(13 downto 0);

signal coeff_034_00 : std_logic_vector(1 downto 0);
signal mac_034_00 : std_logic_vector(13 downto 0);

signal coeff_034_01 : std_logic_vector(1 downto 0);
signal mac_034_01 : std_logic_vector(13 downto 0);

signal coeff_034_02 : std_logic_vector(1 downto 0);
signal mac_034_02 : std_logic_vector(13 downto 0);

signal coeff_034_03 : std_logic_vector(1 downto 0);
signal mac_034_03 : std_logic_vector(13 downto 0);

signal coeff_035_00 : std_logic_vector(1 downto 0);
signal mac_035_00 : std_logic_vector(13 downto 0);

signal coeff_035_01 : std_logic_vector(1 downto 0);
signal mac_035_01 : std_logic_vector(13 downto 0);

signal coeff_035_02 : std_logic_vector(1 downto 0);
signal mac_035_02 : std_logic_vector(13 downto 0);

signal coeff_035_03 : std_logic_vector(1 downto 0);
signal mac_035_03 : std_logic_vector(13 downto 0);

signal coeff_036_00 : std_logic_vector(1 downto 0);
signal mac_036_00 : std_logic_vector(13 downto 0);

signal coeff_036_01 : std_logic_vector(1 downto 0);
signal mac_036_01 : std_logic_vector(13 downto 0);

signal coeff_036_02 : std_logic_vector(1 downto 0);
signal mac_036_02 : std_logic_vector(13 downto 0);

signal coeff_036_03 : std_logic_vector(1 downto 0);
signal mac_036_03 : std_logic_vector(13 downto 0);

signal coeff_037_00 : std_logic_vector(1 downto 0);
signal mac_037_00 : std_logic_vector(13 downto 0);

signal coeff_037_01 : std_logic_vector(1 downto 0);
signal mac_037_01 : std_logic_vector(13 downto 0);

signal coeff_037_02 : std_logic_vector(1 downto 0);
signal mac_037_02 : std_logic_vector(13 downto 0);

signal coeff_037_03 : std_logic_vector(1 downto 0);
signal mac_037_03 : std_logic_vector(13 downto 0);

signal coeff_038_00 : std_logic_vector(1 downto 0);
signal mac_038_00 : std_logic_vector(13 downto 0);

signal coeff_038_01 : std_logic_vector(1 downto 0);
signal mac_038_01 : std_logic_vector(13 downto 0);

signal coeff_038_02 : std_logic_vector(1 downto 0);
signal mac_038_02 : std_logic_vector(13 downto 0);

signal coeff_038_03 : std_logic_vector(1 downto 0);
signal mac_038_03 : std_logic_vector(13 downto 0);

signal coeff_039_00 : std_logic_vector(1 downto 0);
signal mac_039_00 : std_logic_vector(13 downto 0);

signal coeff_039_01 : std_logic_vector(1 downto 0);
signal mac_039_01 : std_logic_vector(13 downto 0);

signal coeff_039_02 : std_logic_vector(1 downto 0);
signal mac_039_02 : std_logic_vector(13 downto 0);

signal coeff_039_03 : std_logic_vector(1 downto 0);
signal mac_039_03 : std_logic_vector(13 downto 0);

signal coeff_040_00 : std_logic_vector(1 downto 0);
signal mac_040_00 : std_logic_vector(13 downto 0);

signal coeff_040_01 : std_logic_vector(1 downto 0);
signal mac_040_01 : std_logic_vector(13 downto 0);

signal coeff_040_02 : std_logic_vector(1 downto 0);
signal mac_040_02 : std_logic_vector(13 downto 0);

signal coeff_040_03 : std_logic_vector(1 downto 0);
signal mac_040_03 : std_logic_vector(13 downto 0);

signal coeff_041_00 : std_logic_vector(1 downto 0);
signal mac_041_00 : std_logic_vector(13 downto 0);

signal coeff_041_01 : std_logic_vector(1 downto 0);
signal mac_041_01 : std_logic_vector(13 downto 0);

signal coeff_041_02 : std_logic_vector(1 downto 0);
signal mac_041_02 : std_logic_vector(13 downto 0);

signal coeff_041_03 : std_logic_vector(1 downto 0);
signal mac_041_03 : std_logic_vector(13 downto 0);

signal coeff_042_00 : std_logic_vector(1 downto 0);
signal mac_042_00 : std_logic_vector(13 downto 0);

signal coeff_042_01 : std_logic_vector(1 downto 0);
signal mac_042_01 : std_logic_vector(13 downto 0);

signal coeff_042_02 : std_logic_vector(1 downto 0);
signal mac_042_02 : std_logic_vector(13 downto 0);

signal coeff_042_03 : std_logic_vector(1 downto 0);
signal mac_042_03 : std_logic_vector(13 downto 0);

signal coeff_043_00 : std_logic_vector(1 downto 0);
signal mac_043_00 : std_logic_vector(13 downto 0);

signal coeff_043_01 : std_logic_vector(1 downto 0);
signal mac_043_01 : std_logic_vector(13 downto 0);

signal coeff_043_02 : std_logic_vector(1 downto 0);
signal mac_043_02 : std_logic_vector(13 downto 0);

signal coeff_043_03 : std_logic_vector(1 downto 0);
signal mac_043_03 : std_logic_vector(13 downto 0);

signal coeff_044_00 : std_logic_vector(1 downto 0);
signal mac_044_00 : std_logic_vector(13 downto 0);

signal coeff_044_01 : std_logic_vector(1 downto 0);
signal mac_044_01 : std_logic_vector(13 downto 0);

signal coeff_044_02 : std_logic_vector(1 downto 0);
signal mac_044_02 : std_logic_vector(13 downto 0);

signal coeff_044_03 : std_logic_vector(1 downto 0);
signal mac_044_03 : std_logic_vector(13 downto 0);

signal coeff_045_00 : std_logic_vector(1 downto 0);
signal mac_045_00 : std_logic_vector(13 downto 0);

signal coeff_045_01 : std_logic_vector(1 downto 0);
signal mac_045_01 : std_logic_vector(13 downto 0);

signal coeff_045_02 : std_logic_vector(1 downto 0);
signal mac_045_02 : std_logic_vector(13 downto 0);

signal coeff_045_03 : std_logic_vector(1 downto 0);
signal mac_045_03 : std_logic_vector(13 downto 0);

signal coeff_046_00 : std_logic_vector(1 downto 0);
signal mac_046_00 : std_logic_vector(13 downto 0);

signal coeff_046_01 : std_logic_vector(1 downto 0);
signal mac_046_01 : std_logic_vector(13 downto 0);

signal coeff_046_02 : std_logic_vector(1 downto 0);
signal mac_046_02 : std_logic_vector(13 downto 0);

signal coeff_046_03 : std_logic_vector(1 downto 0);
signal mac_046_03 : std_logic_vector(13 downto 0);

signal coeff_047_00 : std_logic_vector(1 downto 0);
signal mac_047_00 : std_logic_vector(13 downto 0);

signal coeff_047_01 : std_logic_vector(1 downto 0);
signal mac_047_01 : std_logic_vector(13 downto 0);

signal coeff_047_02 : std_logic_vector(1 downto 0);
signal mac_047_02 : std_logic_vector(13 downto 0);

signal coeff_047_03 : std_logic_vector(1 downto 0);
signal mac_047_03 : std_logic_vector(13 downto 0);

signal coeff_048_00 : std_logic_vector(1 downto 0);
signal mac_048_00 : std_logic_vector(13 downto 0);

signal coeff_048_01 : std_logic_vector(1 downto 0);
signal mac_048_01 : std_logic_vector(13 downto 0);

signal coeff_048_02 : std_logic_vector(1 downto 0);
signal mac_048_02 : std_logic_vector(13 downto 0);

signal coeff_048_03 : std_logic_vector(1 downto 0);
signal mac_048_03 : std_logic_vector(13 downto 0);

signal coeff_049_00 : std_logic_vector(1 downto 0);
signal mac_049_00 : std_logic_vector(13 downto 0);

signal coeff_049_01 : std_logic_vector(1 downto 0);
signal mac_049_01 : std_logic_vector(13 downto 0);

signal coeff_049_02 : std_logic_vector(1 downto 0);
signal mac_049_02 : std_logic_vector(13 downto 0);

signal coeff_049_03 : std_logic_vector(1 downto 0);
signal mac_049_03 : std_logic_vector(13 downto 0);

signal coeff_050_00 : std_logic_vector(1 downto 0);
signal mac_050_00 : std_logic_vector(13 downto 0);

signal coeff_050_01 : std_logic_vector(1 downto 0);
signal mac_050_01 : std_logic_vector(13 downto 0);

signal coeff_050_02 : std_logic_vector(1 downto 0);
signal mac_050_02 : std_logic_vector(13 downto 0);

signal coeff_050_03 : std_logic_vector(1 downto 0);
signal mac_050_03 : std_logic_vector(13 downto 0);

signal coeff_051_00 : std_logic_vector(1 downto 0);
signal mac_051_00 : std_logic_vector(13 downto 0);

signal coeff_051_01 : std_logic_vector(1 downto 0);
signal mac_051_01 : std_logic_vector(13 downto 0);

signal coeff_051_02 : std_logic_vector(1 downto 0);
signal mac_051_02 : std_logic_vector(13 downto 0);

signal coeff_051_03 : std_logic_vector(1 downto 0);
signal mac_051_03 : std_logic_vector(13 downto 0);

signal coeff_052_00 : std_logic_vector(1 downto 0);
signal mac_052_00 : std_logic_vector(13 downto 0);

signal coeff_052_01 : std_logic_vector(1 downto 0);
signal mac_052_01 : std_logic_vector(13 downto 0);

signal coeff_052_02 : std_logic_vector(1 downto 0);
signal mac_052_02 : std_logic_vector(13 downto 0);

signal coeff_052_03 : std_logic_vector(1 downto 0);
signal mac_052_03 : std_logic_vector(13 downto 0);

signal coeff_053_00 : std_logic_vector(1 downto 0);
signal mac_053_00 : std_logic_vector(13 downto 0);

signal coeff_053_01 : std_logic_vector(1 downto 0);
signal mac_053_01 : std_logic_vector(13 downto 0);

signal coeff_053_02 : std_logic_vector(1 downto 0);
signal mac_053_02 : std_logic_vector(13 downto 0);

signal coeff_053_03 : std_logic_vector(1 downto 0);
signal mac_053_03 : std_logic_vector(13 downto 0);

signal coeff_054_00 : std_logic_vector(1 downto 0);
signal mac_054_00 : std_logic_vector(13 downto 0);

signal coeff_054_01 : std_logic_vector(1 downto 0);
signal mac_054_01 : std_logic_vector(13 downto 0);

signal coeff_054_02 : std_logic_vector(1 downto 0);
signal mac_054_02 : std_logic_vector(13 downto 0);

signal coeff_054_03 : std_logic_vector(1 downto 0);
signal mac_054_03 : std_logic_vector(13 downto 0);

signal coeff_055_00 : std_logic_vector(1 downto 0);
signal mac_055_00 : std_logic_vector(13 downto 0);

signal coeff_055_01 : std_logic_vector(1 downto 0);
signal mac_055_01 : std_logic_vector(13 downto 0);

signal coeff_055_02 : std_logic_vector(1 downto 0);
signal mac_055_02 : std_logic_vector(13 downto 0);

signal coeff_055_03 : std_logic_vector(1 downto 0);
signal mac_055_03 : std_logic_vector(13 downto 0);

signal coeff_056_00 : std_logic_vector(1 downto 0);
signal mac_056_00 : std_logic_vector(13 downto 0);

signal coeff_056_01 : std_logic_vector(1 downto 0);
signal mac_056_01 : std_logic_vector(13 downto 0);

signal coeff_056_02 : std_logic_vector(1 downto 0);
signal mac_056_02 : std_logic_vector(13 downto 0);

signal coeff_056_03 : std_logic_vector(1 downto 0);
signal mac_056_03 : std_logic_vector(13 downto 0);

signal coeff_057_00 : std_logic_vector(1 downto 0);
signal mac_057_00 : std_logic_vector(13 downto 0);

signal coeff_057_01 : std_logic_vector(1 downto 0);
signal mac_057_01 : std_logic_vector(13 downto 0);

signal coeff_057_02 : std_logic_vector(1 downto 0);
signal mac_057_02 : std_logic_vector(13 downto 0);

signal coeff_057_03 : std_logic_vector(1 downto 0);
signal mac_057_03 : std_logic_vector(13 downto 0);

signal coeff_058_00 : std_logic_vector(1 downto 0);
signal mac_058_00 : std_logic_vector(13 downto 0);

signal coeff_058_01 : std_logic_vector(1 downto 0);
signal mac_058_01 : std_logic_vector(13 downto 0);

signal coeff_058_02 : std_logic_vector(1 downto 0);
signal mac_058_02 : std_logic_vector(13 downto 0);

signal coeff_058_03 : std_logic_vector(1 downto 0);
signal mac_058_03 : std_logic_vector(13 downto 0);

signal coeff_059_00 : std_logic_vector(1 downto 0);
signal mac_059_00 : std_logic_vector(13 downto 0);

signal coeff_059_01 : std_logic_vector(1 downto 0);
signal mac_059_01 : std_logic_vector(13 downto 0);

signal coeff_059_02 : std_logic_vector(1 downto 0);
signal mac_059_02 : std_logic_vector(13 downto 0);

signal coeff_059_03 : std_logic_vector(1 downto 0);
signal mac_059_03 : std_logic_vector(13 downto 0);

signal coeff_060_00 : std_logic_vector(1 downto 0);
signal mac_060_00 : std_logic_vector(13 downto 0);

signal coeff_060_01 : std_logic_vector(1 downto 0);
signal mac_060_01 : std_logic_vector(13 downto 0);

signal coeff_060_02 : std_logic_vector(1 downto 0);
signal mac_060_02 : std_logic_vector(13 downto 0);

signal coeff_060_03 : std_logic_vector(1 downto 0);
signal mac_060_03 : std_logic_vector(13 downto 0);

signal coeff_061_00 : std_logic_vector(1 downto 0);
signal mac_061_00 : std_logic_vector(13 downto 0);

signal coeff_061_01 : std_logic_vector(1 downto 0);
signal mac_061_01 : std_logic_vector(13 downto 0);

signal coeff_061_02 : std_logic_vector(1 downto 0);
signal mac_061_02 : std_logic_vector(13 downto 0);

signal coeff_061_03 : std_logic_vector(1 downto 0);
signal mac_061_03 : std_logic_vector(13 downto 0);

signal coeff_062_00 : std_logic_vector(1 downto 0);
signal mac_062_00 : std_logic_vector(13 downto 0);

signal coeff_062_01 : std_logic_vector(1 downto 0);
signal mac_062_01 : std_logic_vector(13 downto 0);

signal coeff_062_02 : std_logic_vector(1 downto 0);
signal mac_062_02 : std_logic_vector(13 downto 0);

signal coeff_062_03 : std_logic_vector(1 downto 0);
signal mac_062_03 : std_logic_vector(13 downto 0);

signal coeff_063_00 : std_logic_vector(1 downto 0);
signal mac_063_00 : std_logic_vector(13 downto 0);

signal coeff_063_01 : std_logic_vector(1 downto 0);
signal mac_063_01 : std_logic_vector(13 downto 0);

signal coeff_063_02 : std_logic_vector(1 downto 0);
signal mac_063_02 : std_logic_vector(13 downto 0);

signal coeff_063_03 : std_logic_vector(1 downto 0);
signal mac_063_03 : std_logic_vector(13 downto 0);

signal coeff_064_00 : std_logic_vector(1 downto 0);
signal mac_064_00 : std_logic_vector(13 downto 0);

signal coeff_064_01 : std_logic_vector(1 downto 0);
signal mac_064_01 : std_logic_vector(13 downto 0);

signal coeff_064_02 : std_logic_vector(1 downto 0);
signal mac_064_02 : std_logic_vector(13 downto 0);

signal coeff_064_03 : std_logic_vector(1 downto 0);
signal mac_064_03 : std_logic_vector(13 downto 0);

signal coeff_065_00 : std_logic_vector(1 downto 0);
signal mac_065_00 : std_logic_vector(13 downto 0);

signal coeff_065_01 : std_logic_vector(1 downto 0);
signal mac_065_01 : std_logic_vector(13 downto 0);

signal coeff_065_02 : std_logic_vector(1 downto 0);
signal mac_065_02 : std_logic_vector(13 downto 0);

signal coeff_065_03 : std_logic_vector(1 downto 0);
signal mac_065_03 : std_logic_vector(13 downto 0);

signal coeff_066_00 : std_logic_vector(1 downto 0);
signal mac_066_00 : std_logic_vector(13 downto 0);

signal coeff_066_01 : std_logic_vector(1 downto 0);
signal mac_066_01 : std_logic_vector(13 downto 0);

signal coeff_066_02 : std_logic_vector(1 downto 0);
signal mac_066_02 : std_logic_vector(13 downto 0);

signal coeff_066_03 : std_logic_vector(1 downto 0);
signal mac_066_03 : std_logic_vector(13 downto 0);

signal coeff_067_00 : std_logic_vector(1 downto 0);
signal mac_067_00 : std_logic_vector(13 downto 0);

signal coeff_067_01 : std_logic_vector(1 downto 0);
signal mac_067_01 : std_logic_vector(13 downto 0);

signal coeff_067_02 : std_logic_vector(1 downto 0);
signal mac_067_02 : std_logic_vector(13 downto 0);

signal coeff_067_03 : std_logic_vector(1 downto 0);
signal mac_067_03 : std_logic_vector(13 downto 0);

signal coeff_068_00 : std_logic_vector(1 downto 0);
signal mac_068_00 : std_logic_vector(13 downto 0);

signal coeff_068_01 : std_logic_vector(1 downto 0);
signal mac_068_01 : std_logic_vector(13 downto 0);

signal coeff_068_02 : std_logic_vector(1 downto 0);
signal mac_068_02 : std_logic_vector(13 downto 0);

signal coeff_068_03 : std_logic_vector(1 downto 0);
signal mac_068_03 : std_logic_vector(13 downto 0);

signal coeff_069_00 : std_logic_vector(1 downto 0);
signal mac_069_00 : std_logic_vector(13 downto 0);

signal coeff_069_01 : std_logic_vector(1 downto 0);
signal mac_069_01 : std_logic_vector(13 downto 0);

signal coeff_069_02 : std_logic_vector(1 downto 0);
signal mac_069_02 : std_logic_vector(13 downto 0);

signal coeff_069_03 : std_logic_vector(1 downto 0);
signal mac_069_03 : std_logic_vector(13 downto 0);

signal coeff_070_00 : std_logic_vector(1 downto 0);
signal mac_070_00 : std_logic_vector(13 downto 0);

signal coeff_070_01 : std_logic_vector(1 downto 0);
signal mac_070_01 : std_logic_vector(13 downto 0);

signal coeff_070_02 : std_logic_vector(1 downto 0);
signal mac_070_02 : std_logic_vector(13 downto 0);

signal coeff_070_03 : std_logic_vector(1 downto 0);
signal mac_070_03 : std_logic_vector(13 downto 0);

signal coeff_071_00 : std_logic_vector(1 downto 0);
signal mac_071_00 : std_logic_vector(13 downto 0);

signal coeff_071_01 : std_logic_vector(1 downto 0);
signal mac_071_01 : std_logic_vector(13 downto 0);

signal coeff_071_02 : std_logic_vector(1 downto 0);
signal mac_071_02 : std_logic_vector(13 downto 0);

signal coeff_071_03 : std_logic_vector(1 downto 0);
signal mac_071_03 : std_logic_vector(13 downto 0);

signal coeff_072_00 : std_logic_vector(1 downto 0);
signal mac_072_00 : std_logic_vector(13 downto 0);

signal coeff_072_01 : std_logic_vector(1 downto 0);
signal mac_072_01 : std_logic_vector(13 downto 0);

signal coeff_072_02 : std_logic_vector(1 downto 0);
signal mac_072_02 : std_logic_vector(13 downto 0);

signal coeff_072_03 : std_logic_vector(1 downto 0);
signal mac_072_03 : std_logic_vector(13 downto 0);

signal coeff_073_00 : std_logic_vector(1 downto 0);
signal mac_073_00 : std_logic_vector(13 downto 0);

signal coeff_073_01 : std_logic_vector(1 downto 0);
signal mac_073_01 : std_logic_vector(13 downto 0);

signal coeff_073_02 : std_logic_vector(1 downto 0);
signal mac_073_02 : std_logic_vector(13 downto 0);

signal coeff_073_03 : std_logic_vector(1 downto 0);
signal mac_073_03 : std_logic_vector(13 downto 0);

signal coeff_074_00 : std_logic_vector(1 downto 0);
signal mac_074_00 : std_logic_vector(13 downto 0);

signal coeff_074_01 : std_logic_vector(1 downto 0);
signal mac_074_01 : std_logic_vector(13 downto 0);

signal coeff_074_02 : std_logic_vector(1 downto 0);
signal mac_074_02 : std_logic_vector(13 downto 0);

signal coeff_074_03 : std_logic_vector(1 downto 0);
signal mac_074_03 : std_logic_vector(13 downto 0);

signal coeff_075_00 : std_logic_vector(1 downto 0);
signal mac_075_00 : std_logic_vector(13 downto 0);

signal coeff_075_01 : std_logic_vector(1 downto 0);
signal mac_075_01 : std_logic_vector(13 downto 0);

signal coeff_075_02 : std_logic_vector(1 downto 0);
signal mac_075_02 : std_logic_vector(13 downto 0);

signal coeff_075_03 : std_logic_vector(1 downto 0);
signal mac_075_03 : std_logic_vector(13 downto 0);

signal coeff_076_00 : std_logic_vector(1 downto 0);
signal mac_076_00 : std_logic_vector(13 downto 0);

signal coeff_076_01 : std_logic_vector(1 downto 0);
signal mac_076_01 : std_logic_vector(13 downto 0);

signal coeff_076_02 : std_logic_vector(1 downto 0);
signal mac_076_02 : std_logic_vector(13 downto 0);

signal coeff_076_03 : std_logic_vector(1 downto 0);
signal mac_076_03 : std_logic_vector(13 downto 0);

signal coeff_077_00 : std_logic_vector(1 downto 0);
signal mac_077_00 : std_logic_vector(13 downto 0);

signal coeff_077_01 : std_logic_vector(1 downto 0);
signal mac_077_01 : std_logic_vector(13 downto 0);

signal coeff_077_02 : std_logic_vector(1 downto 0);
signal mac_077_02 : std_logic_vector(13 downto 0);

signal coeff_077_03 : std_logic_vector(1 downto 0);
signal mac_077_03 : std_logic_vector(13 downto 0);

signal coeff_078_00 : std_logic_vector(1 downto 0);
signal mac_078_00 : std_logic_vector(13 downto 0);

signal coeff_078_01 : std_logic_vector(1 downto 0);
signal mac_078_01 : std_logic_vector(13 downto 0);

signal coeff_078_02 : std_logic_vector(1 downto 0);
signal mac_078_02 : std_logic_vector(13 downto 0);

signal coeff_078_03 : std_logic_vector(1 downto 0);
signal mac_078_03 : std_logic_vector(13 downto 0);

signal coeff_079_00 : std_logic_vector(1 downto 0);
signal mac_079_00 : std_logic_vector(13 downto 0);

signal coeff_079_01 : std_logic_vector(1 downto 0);
signal mac_079_01 : std_logic_vector(13 downto 0);

signal coeff_079_02 : std_logic_vector(1 downto 0);
signal mac_079_02 : std_logic_vector(13 downto 0);

signal coeff_079_03 : std_logic_vector(1 downto 0);
signal mac_079_03 : std_logic_vector(13 downto 0);

signal coeff_080_00 : std_logic_vector(1 downto 0);
signal mac_080_00 : std_logic_vector(13 downto 0);

signal coeff_080_01 : std_logic_vector(1 downto 0);
signal mac_080_01 : std_logic_vector(13 downto 0);

signal coeff_080_02 : std_logic_vector(1 downto 0);
signal mac_080_02 : std_logic_vector(13 downto 0);

signal coeff_080_03 : std_logic_vector(1 downto 0);
signal mac_080_03 : std_logic_vector(13 downto 0);

signal coeff_081_00 : std_logic_vector(1 downto 0);
signal mac_081_00 : std_logic_vector(13 downto 0);

signal coeff_081_01 : std_logic_vector(1 downto 0);
signal mac_081_01 : std_logic_vector(13 downto 0);

signal coeff_081_02 : std_logic_vector(1 downto 0);
signal mac_081_02 : std_logic_vector(13 downto 0);

signal coeff_081_03 : std_logic_vector(1 downto 0);
signal mac_081_03 : std_logic_vector(13 downto 0);

signal coeff_082_00 : std_logic_vector(1 downto 0);
signal mac_082_00 : std_logic_vector(13 downto 0);

signal coeff_082_01 : std_logic_vector(1 downto 0);
signal mac_082_01 : std_logic_vector(13 downto 0);

signal coeff_082_02 : std_logic_vector(1 downto 0);
signal mac_082_02 : std_logic_vector(13 downto 0);

signal coeff_082_03 : std_logic_vector(1 downto 0);
signal mac_082_03 : std_logic_vector(13 downto 0);

signal coeff_083_00 : std_logic_vector(1 downto 0);
signal mac_083_00 : std_logic_vector(13 downto 0);

signal coeff_083_01 : std_logic_vector(1 downto 0);
signal mac_083_01 : std_logic_vector(13 downto 0);

signal coeff_083_02 : std_logic_vector(1 downto 0);
signal mac_083_02 : std_logic_vector(13 downto 0);

signal coeff_083_03 : std_logic_vector(1 downto 0);
signal mac_083_03 : std_logic_vector(13 downto 0);

signal coeff_084_00 : std_logic_vector(1 downto 0);
signal mac_084_00 : std_logic_vector(13 downto 0);

signal coeff_084_01 : std_logic_vector(1 downto 0);
signal mac_084_01 : std_logic_vector(13 downto 0);

signal coeff_084_02 : std_logic_vector(1 downto 0);
signal mac_084_02 : std_logic_vector(13 downto 0);

signal coeff_084_03 : std_logic_vector(1 downto 0);
signal mac_084_03 : std_logic_vector(13 downto 0);

signal coeff_085_00 : std_logic_vector(1 downto 0);
signal mac_085_00 : std_logic_vector(13 downto 0);

signal coeff_085_01 : std_logic_vector(1 downto 0);
signal mac_085_01 : std_logic_vector(13 downto 0);

signal coeff_085_02 : std_logic_vector(1 downto 0);
signal mac_085_02 : std_logic_vector(13 downto 0);

signal coeff_085_03 : std_logic_vector(1 downto 0);
signal mac_085_03 : std_logic_vector(13 downto 0);

signal coeff_086_00 : std_logic_vector(1 downto 0);
signal mac_086_00 : std_logic_vector(13 downto 0);

signal coeff_086_01 : std_logic_vector(1 downto 0);
signal mac_086_01 : std_logic_vector(13 downto 0);

signal coeff_086_02 : std_logic_vector(1 downto 0);
signal mac_086_02 : std_logic_vector(13 downto 0);

signal coeff_086_03 : std_logic_vector(1 downto 0);
signal mac_086_03 : std_logic_vector(13 downto 0);

signal coeff_087_00 : std_logic_vector(1 downto 0);
signal mac_087_00 : std_logic_vector(13 downto 0);

signal coeff_087_01 : std_logic_vector(1 downto 0);
signal mac_087_01 : std_logic_vector(13 downto 0);

signal coeff_087_02 : std_logic_vector(1 downto 0);
signal mac_087_02 : std_logic_vector(13 downto 0);

signal coeff_087_03 : std_logic_vector(1 downto 0);
signal mac_087_03 : std_logic_vector(13 downto 0);

signal coeff_088_00 : std_logic_vector(1 downto 0);
signal mac_088_00 : std_logic_vector(13 downto 0);

signal coeff_088_01 : std_logic_vector(1 downto 0);
signal mac_088_01 : std_logic_vector(13 downto 0);

signal coeff_088_02 : std_logic_vector(1 downto 0);
signal mac_088_02 : std_logic_vector(13 downto 0);

signal coeff_088_03 : std_logic_vector(1 downto 0);
signal mac_088_03 : std_logic_vector(13 downto 0);

signal coeff_089_00 : std_logic_vector(1 downto 0);
signal mac_089_00 : std_logic_vector(13 downto 0);

signal coeff_089_01 : std_logic_vector(1 downto 0);
signal mac_089_01 : std_logic_vector(13 downto 0);

signal coeff_089_02 : std_logic_vector(1 downto 0);
signal mac_089_02 : std_logic_vector(13 downto 0);

signal coeff_089_03 : std_logic_vector(1 downto 0);
signal mac_089_03 : std_logic_vector(13 downto 0);

signal coeff_090_00 : std_logic_vector(1 downto 0);
signal mac_090_00 : std_logic_vector(13 downto 0);

signal coeff_090_01 : std_logic_vector(1 downto 0);
signal mac_090_01 : std_logic_vector(13 downto 0);

signal coeff_090_02 : std_logic_vector(1 downto 0);
signal mac_090_02 : std_logic_vector(13 downto 0);

signal coeff_090_03 : std_logic_vector(1 downto 0);
signal mac_090_03 : std_logic_vector(13 downto 0);

signal coeff_091_00 : std_logic_vector(1 downto 0);
signal mac_091_00 : std_logic_vector(13 downto 0);

signal coeff_091_01 : std_logic_vector(1 downto 0);
signal mac_091_01 : std_logic_vector(13 downto 0);

signal coeff_091_02 : std_logic_vector(1 downto 0);
signal mac_091_02 : std_logic_vector(13 downto 0);

signal coeff_091_03 : std_logic_vector(1 downto 0);
signal mac_091_03 : std_logic_vector(13 downto 0);

signal coeff_092_00 : std_logic_vector(1 downto 0);
signal mac_092_00 : std_logic_vector(13 downto 0);

signal coeff_092_01 : std_logic_vector(1 downto 0);
signal mac_092_01 : std_logic_vector(13 downto 0);

signal coeff_092_02 : std_logic_vector(1 downto 0);
signal mac_092_02 : std_logic_vector(13 downto 0);

signal coeff_092_03 : std_logic_vector(1 downto 0);
signal mac_092_03 : std_logic_vector(13 downto 0);

signal coeff_093_00 : std_logic_vector(1 downto 0);
signal mac_093_00 : std_logic_vector(13 downto 0);

signal coeff_093_01 : std_logic_vector(1 downto 0);
signal mac_093_01 : std_logic_vector(13 downto 0);

signal coeff_093_02 : std_logic_vector(1 downto 0);
signal mac_093_02 : std_logic_vector(13 downto 0);

signal coeff_093_03 : std_logic_vector(1 downto 0);
signal mac_093_03 : std_logic_vector(13 downto 0);

signal coeff_094_00 : std_logic_vector(1 downto 0);
signal mac_094_00 : std_logic_vector(13 downto 0);

signal coeff_094_01 : std_logic_vector(1 downto 0);
signal mac_094_01 : std_logic_vector(13 downto 0);

signal coeff_094_02 : std_logic_vector(1 downto 0);
signal mac_094_02 : std_logic_vector(13 downto 0);

signal coeff_094_03 : std_logic_vector(1 downto 0);
signal mac_094_03 : std_logic_vector(13 downto 0);

signal coeff_095_00 : std_logic_vector(1 downto 0);
signal mac_095_00 : std_logic_vector(13 downto 0);

signal coeff_095_01 : std_logic_vector(1 downto 0);
signal mac_095_01 : std_logic_vector(13 downto 0);

signal coeff_095_02 : std_logic_vector(1 downto 0);
signal mac_095_02 : std_logic_vector(13 downto 0);

signal coeff_095_03 : std_logic_vector(1 downto 0);
signal mac_095_03 : std_logic_vector(13 downto 0);

signal coeff_096_00 : std_logic_vector(1 downto 0);
signal mac_096_00 : std_logic_vector(13 downto 0);

signal coeff_096_01 : std_logic_vector(1 downto 0);
signal mac_096_01 : std_logic_vector(13 downto 0);

signal coeff_096_02 : std_logic_vector(1 downto 0);
signal mac_096_02 : std_logic_vector(13 downto 0);

signal coeff_096_03 : std_logic_vector(1 downto 0);
signal mac_096_03 : std_logic_vector(13 downto 0);

signal coeff_097_00 : std_logic_vector(1 downto 0);
signal mac_097_00 : std_logic_vector(13 downto 0);

signal coeff_097_01 : std_logic_vector(1 downto 0);
signal mac_097_01 : std_logic_vector(13 downto 0);

signal coeff_097_02 : std_logic_vector(1 downto 0);
signal mac_097_02 : std_logic_vector(13 downto 0);

signal coeff_097_03 : std_logic_vector(1 downto 0);
signal mac_097_03 : std_logic_vector(13 downto 0);

signal coeff_098_00 : std_logic_vector(1 downto 0);
signal mac_098_00 : std_logic_vector(13 downto 0);

signal coeff_098_01 : std_logic_vector(1 downto 0);
signal mac_098_01 : std_logic_vector(13 downto 0);

signal coeff_098_02 : std_logic_vector(1 downto 0);
signal mac_098_02 : std_logic_vector(13 downto 0);

signal coeff_098_03 : std_logic_vector(1 downto 0);
signal mac_098_03 : std_logic_vector(13 downto 0);

signal coeff_099_00 : std_logic_vector(1 downto 0);
signal mac_099_00 : std_logic_vector(13 downto 0);

signal coeff_099_01 : std_logic_vector(1 downto 0);
signal mac_099_01 : std_logic_vector(13 downto 0);

signal coeff_099_02 : std_logic_vector(1 downto 0);
signal mac_099_02 : std_logic_vector(13 downto 0);

signal coeff_099_03 : std_logic_vector(1 downto 0);
signal mac_099_03 : std_logic_vector(13 downto 0);

signal coeff_100_00 : std_logic_vector(1 downto 0);
signal mac_100_00 : std_logic_vector(13 downto 0);

signal coeff_100_01 : std_logic_vector(1 downto 0);
signal mac_100_01 : std_logic_vector(13 downto 0);

signal coeff_100_02 : std_logic_vector(1 downto 0);
signal mac_100_02 : std_logic_vector(13 downto 0);

signal coeff_100_03 : std_logic_vector(1 downto 0);
signal mac_100_03 : std_logic_vector(13 downto 0);

signal coeff_101_00 : std_logic_vector(1 downto 0);
signal mac_101_00 : std_logic_vector(13 downto 0);

signal coeff_101_01 : std_logic_vector(1 downto 0);
signal mac_101_01 : std_logic_vector(13 downto 0);

signal coeff_101_02 : std_logic_vector(1 downto 0);
signal mac_101_02 : std_logic_vector(13 downto 0);

signal coeff_101_03 : std_logic_vector(1 downto 0);
signal mac_101_03 : std_logic_vector(13 downto 0);

signal coeff_102_00 : std_logic_vector(1 downto 0);
signal mac_102_00 : std_logic_vector(13 downto 0);

signal coeff_102_01 : std_logic_vector(1 downto 0);
signal mac_102_01 : std_logic_vector(13 downto 0);

signal coeff_102_02 : std_logic_vector(1 downto 0);
signal mac_102_02 : std_logic_vector(13 downto 0);

signal coeff_102_03 : std_logic_vector(1 downto 0);
signal mac_102_03 : std_logic_vector(13 downto 0);

signal coeff_103_00 : std_logic_vector(1 downto 0);
signal mac_103_00 : std_logic_vector(13 downto 0);

signal coeff_103_01 : std_logic_vector(1 downto 0);
signal mac_103_01 : std_logic_vector(13 downto 0);

signal coeff_103_02 : std_logic_vector(1 downto 0);
signal mac_103_02 : std_logic_vector(13 downto 0);

signal coeff_103_03 : std_logic_vector(1 downto 0);
signal mac_103_03 : std_logic_vector(13 downto 0);

signal coeff_104_00 : std_logic_vector(1 downto 0);
signal mac_104_00 : std_logic_vector(13 downto 0);

signal coeff_104_01 : std_logic_vector(1 downto 0);
signal mac_104_01 : std_logic_vector(13 downto 0);

signal coeff_104_02 : std_logic_vector(1 downto 0);
signal mac_104_02 : std_logic_vector(13 downto 0);

signal coeff_104_03 : std_logic_vector(1 downto 0);
signal mac_104_03 : std_logic_vector(13 downto 0);

signal coeff_105_00 : std_logic_vector(1 downto 0);
signal mac_105_00 : std_logic_vector(13 downto 0);

signal coeff_105_01 : std_logic_vector(1 downto 0);
signal mac_105_01 : std_logic_vector(13 downto 0);

signal coeff_105_02 : std_logic_vector(1 downto 0);
signal mac_105_02 : std_logic_vector(13 downto 0);

signal coeff_105_03 : std_logic_vector(1 downto 0);
signal mac_105_03 : std_logic_vector(13 downto 0);

signal coeff_106_00 : std_logic_vector(1 downto 0);
signal mac_106_00 : std_logic_vector(13 downto 0);

signal coeff_106_01 : std_logic_vector(1 downto 0);
signal mac_106_01 : std_logic_vector(13 downto 0);

signal coeff_106_02 : std_logic_vector(1 downto 0);
signal mac_106_02 : std_logic_vector(13 downto 0);

signal coeff_106_03 : std_logic_vector(1 downto 0);
signal mac_106_03 : std_logic_vector(13 downto 0);

signal coeff_107_00 : std_logic_vector(1 downto 0);
signal mac_107_00 : std_logic_vector(13 downto 0);

signal coeff_107_01 : std_logic_vector(1 downto 0);
signal mac_107_01 : std_logic_vector(13 downto 0);

signal coeff_107_02 : std_logic_vector(1 downto 0);
signal mac_107_02 : std_logic_vector(13 downto 0);

signal coeff_107_03 : std_logic_vector(1 downto 0);
signal mac_107_03 : std_logic_vector(13 downto 0);

signal coeff_108_00 : std_logic_vector(1 downto 0);
signal mac_108_00 : std_logic_vector(13 downto 0);

signal coeff_108_01 : std_logic_vector(1 downto 0);
signal mac_108_01 : std_logic_vector(13 downto 0);

signal coeff_108_02 : std_logic_vector(1 downto 0);
signal mac_108_02 : std_logic_vector(13 downto 0);

signal coeff_108_03 : std_logic_vector(1 downto 0);
signal mac_108_03 : std_logic_vector(13 downto 0);

signal coeff_109_00 : std_logic_vector(1 downto 0);
signal mac_109_00 : std_logic_vector(13 downto 0);

signal coeff_109_01 : std_logic_vector(1 downto 0);
signal mac_109_01 : std_logic_vector(13 downto 0);

signal coeff_109_02 : std_logic_vector(1 downto 0);
signal mac_109_02 : std_logic_vector(13 downto 0);

signal coeff_109_03 : std_logic_vector(1 downto 0);
signal mac_109_03 : std_logic_vector(13 downto 0);

signal coeff_110_00 : std_logic_vector(1 downto 0);
signal mac_110_00 : std_logic_vector(13 downto 0);

signal coeff_110_01 : std_logic_vector(1 downto 0);
signal mac_110_01 : std_logic_vector(13 downto 0);

signal coeff_110_02 : std_logic_vector(1 downto 0);
signal mac_110_02 : std_logic_vector(13 downto 0);

signal coeff_110_03 : std_logic_vector(1 downto 0);
signal mac_110_03 : std_logic_vector(13 downto 0);

signal coeff_111_00 : std_logic_vector(1 downto 0);
signal mac_111_00 : std_logic_vector(13 downto 0);

signal coeff_111_01 : std_logic_vector(1 downto 0);
signal mac_111_01 : std_logic_vector(13 downto 0);

signal coeff_111_02 : std_logic_vector(1 downto 0);
signal mac_111_02 : std_logic_vector(13 downto 0);

signal coeff_111_03 : std_logic_vector(1 downto 0);
signal mac_111_03 : std_logic_vector(13 downto 0);

signal coeff_112_00 : std_logic_vector(1 downto 0);
signal mac_112_00 : std_logic_vector(13 downto 0);

signal coeff_112_01 : std_logic_vector(1 downto 0);
signal mac_112_01 : std_logic_vector(13 downto 0);

signal coeff_112_02 : std_logic_vector(1 downto 0);
signal mac_112_02 : std_logic_vector(13 downto 0);

signal coeff_112_03 : std_logic_vector(1 downto 0);
signal mac_112_03 : std_logic_vector(13 downto 0);

signal coeff_113_00 : std_logic_vector(1 downto 0);
signal mac_113_00 : std_logic_vector(13 downto 0);

signal coeff_113_01 : std_logic_vector(1 downto 0);
signal mac_113_01 : std_logic_vector(13 downto 0);

signal coeff_113_02 : std_logic_vector(1 downto 0);
signal mac_113_02 : std_logic_vector(13 downto 0);

signal coeff_113_03 : std_logic_vector(1 downto 0);
signal mac_113_03 : std_logic_vector(13 downto 0);

signal coeff_114_00 : std_logic_vector(1 downto 0);
signal mac_114_00 : std_logic_vector(13 downto 0);

signal coeff_114_01 : std_logic_vector(1 downto 0);
signal mac_114_01 : std_logic_vector(13 downto 0);

signal coeff_114_02 : std_logic_vector(1 downto 0);
signal mac_114_02 : std_logic_vector(13 downto 0);

signal coeff_114_03 : std_logic_vector(1 downto 0);
signal mac_114_03 : std_logic_vector(13 downto 0);

signal coeff_115_00 : std_logic_vector(1 downto 0);
signal mac_115_00 : std_logic_vector(13 downto 0);

signal coeff_115_01 : std_logic_vector(1 downto 0);
signal mac_115_01 : std_logic_vector(13 downto 0);

signal coeff_115_02 : std_logic_vector(1 downto 0);
signal mac_115_02 : std_logic_vector(13 downto 0);

signal coeff_115_03 : std_logic_vector(1 downto 0);
signal mac_115_03 : std_logic_vector(13 downto 0);

signal coeff_116_00 : std_logic_vector(1 downto 0);
signal mac_116_00 : std_logic_vector(13 downto 0);

signal coeff_116_01 : std_logic_vector(1 downto 0);
signal mac_116_01 : std_logic_vector(13 downto 0);

signal coeff_116_02 : std_logic_vector(1 downto 0);
signal mac_116_02 : std_logic_vector(13 downto 0);

signal coeff_116_03 : std_logic_vector(1 downto 0);
signal mac_116_03 : std_logic_vector(13 downto 0);

signal coeff_117_00 : std_logic_vector(1 downto 0);
signal mac_117_00 : std_logic_vector(13 downto 0);

signal coeff_117_01 : std_logic_vector(1 downto 0);
signal mac_117_01 : std_logic_vector(13 downto 0);

signal coeff_117_02 : std_logic_vector(1 downto 0);
signal mac_117_02 : std_logic_vector(13 downto 0);

signal coeff_117_03 : std_logic_vector(1 downto 0);
signal mac_117_03 : std_logic_vector(13 downto 0);

signal coeff_118_00 : std_logic_vector(1 downto 0);
signal mac_118_00 : std_logic_vector(13 downto 0);

signal coeff_118_01 : std_logic_vector(1 downto 0);
signal mac_118_01 : std_logic_vector(13 downto 0);

signal coeff_118_02 : std_logic_vector(1 downto 0);
signal mac_118_02 : std_logic_vector(13 downto 0);

signal coeff_118_03 : std_logic_vector(1 downto 0);
signal mac_118_03 : std_logic_vector(13 downto 0);

signal coeff_119_00 : std_logic_vector(1 downto 0);
signal mac_119_00 : std_logic_vector(13 downto 0);

signal coeff_119_01 : std_logic_vector(1 downto 0);
signal mac_119_01 : std_logic_vector(13 downto 0);

signal coeff_119_02 : std_logic_vector(1 downto 0);
signal mac_119_02 : std_logic_vector(13 downto 0);

signal coeff_119_03 : std_logic_vector(1 downto 0);
signal mac_119_03 : std_logic_vector(13 downto 0);

signal coeff_120_00 : std_logic_vector(1 downto 0);
signal mac_120_00 : std_logic_vector(13 downto 0);

signal coeff_120_01 : std_logic_vector(1 downto 0);
signal mac_120_01 : std_logic_vector(13 downto 0);

signal coeff_120_02 : std_logic_vector(1 downto 0);
signal mac_120_02 : std_logic_vector(13 downto 0);

signal coeff_120_03 : std_logic_vector(1 downto 0);
signal mac_120_03 : std_logic_vector(13 downto 0);

signal coeff_121_00 : std_logic_vector(1 downto 0);
signal mac_121_00 : std_logic_vector(13 downto 0);

signal coeff_121_01 : std_logic_vector(1 downto 0);
signal mac_121_01 : std_logic_vector(13 downto 0);

signal coeff_121_02 : std_logic_vector(1 downto 0);
signal mac_121_02 : std_logic_vector(13 downto 0);

signal coeff_121_03 : std_logic_vector(1 downto 0);
signal mac_121_03 : std_logic_vector(13 downto 0);

signal coeff_122_00 : std_logic_vector(1 downto 0);
signal mac_122_00 : std_logic_vector(13 downto 0);

signal coeff_122_01 : std_logic_vector(1 downto 0);
signal mac_122_01 : std_logic_vector(13 downto 0);

signal coeff_122_02 : std_logic_vector(1 downto 0);
signal mac_122_02 : std_logic_vector(13 downto 0);

signal coeff_122_03 : std_logic_vector(1 downto 0);
signal mac_122_03 : std_logic_vector(13 downto 0);

signal coeff_123_00 : std_logic_vector(1 downto 0);
signal mac_123_00 : std_logic_vector(13 downto 0);

signal coeff_123_01 : std_logic_vector(1 downto 0);
signal mac_123_01 : std_logic_vector(13 downto 0);

signal coeff_123_02 : std_logic_vector(1 downto 0);
signal mac_123_02 : std_logic_vector(13 downto 0);

signal coeff_123_03 : std_logic_vector(1 downto 0);
signal mac_123_03 : std_logic_vector(13 downto 0);

signal coeff_124_00 : std_logic_vector(1 downto 0);
signal mac_124_00 : std_logic_vector(13 downto 0);

signal coeff_124_01 : std_logic_vector(1 downto 0);
signal mac_124_01 : std_logic_vector(13 downto 0);

signal coeff_124_02 : std_logic_vector(1 downto 0);
signal mac_124_02 : std_logic_vector(13 downto 0);

signal coeff_124_03 : std_logic_vector(1 downto 0);
signal mac_124_03 : std_logic_vector(13 downto 0);

signal coeff_125_00 : std_logic_vector(1 downto 0);
signal mac_125_00 : std_logic_vector(13 downto 0);

signal coeff_125_01 : std_logic_vector(1 downto 0);
signal mac_125_01 : std_logic_vector(13 downto 0);

signal coeff_125_02 : std_logic_vector(1 downto 0);
signal mac_125_02 : std_logic_vector(13 downto 0);

signal coeff_125_03 : std_logic_vector(1 downto 0);
signal mac_125_03 : std_logic_vector(13 downto 0);

signal coeff_126_00 : std_logic_vector(1 downto 0);
signal mac_126_00 : std_logic_vector(13 downto 0);

signal coeff_126_01 : std_logic_vector(1 downto 0);
signal mac_126_01 : std_logic_vector(13 downto 0);

signal coeff_126_02 : std_logic_vector(1 downto 0);
signal mac_126_02 : std_logic_vector(13 downto 0);

signal coeff_126_03 : std_logic_vector(1 downto 0);
signal mac_126_03 : std_logic_vector(13 downto 0);

signal coeff_127_00 : std_logic_vector(1 downto 0);
signal mac_127_00 : std_logic_vector(13 downto 0);

signal coeff_127_01 : std_logic_vector(1 downto 0);
signal mac_127_01 : std_logic_vector(13 downto 0);

signal coeff_127_02 : std_logic_vector(1 downto 0);
signal mac_127_02 : std_logic_vector(13 downto 0);

signal coeff_127_03 : std_logic_vector(1 downto 0);
signal mac_127_03 : std_logic_vector(13 downto 0);


signal MacAll_000, MacAll_001, MacAll_002, MacAll_003, MacAll_004, MacAll_005, MacAll_006, MacAll_007, MacAll_008, MacAll_009: std_logic_vector(13 downto 0);
signal MacAll_010, MacAll_011, MacAll_012, MacAll_013, MacAll_014, MacAll_015, MacAll_016, MacAll_017, MacAll_018, MacAll_019: std_logic_vector(13 downto 0);
signal MacAll_020, MacAll_021, MacAll_022, MacAll_023, MacAll_024, MacAll_025, MacAll_026, MacAll_027, MacAll_028, MacAll_029: std_logic_vector(13 downto 0);
signal MacAll_030, MacAll_031, MacAll_032, MacAll_033, MacAll_034, MacAll_035, MacAll_036, MacAll_037, MacAll_038, MacAll_039: std_logic_vector(13 downto 0);
signal MacAll_040, MacAll_041, MacAll_042, MacAll_043, MacAll_044, MacAll_045, MacAll_046, MacAll_047, MacAll_048, MacAll_049: std_logic_vector(13 downto 0);
signal MacAll_050, MacAll_051, MacAll_052, MacAll_053, MacAll_054, MacAll_055, MacAll_056, MacAll_057, MacAll_058, MacAll_059: std_logic_vector(13 downto 0);
signal MacAll_060, MacAll_061, MacAll_062, MacAll_063, MacAll_064, MacAll_065, MacAll_066, MacAll_067, MacAll_068, MacAll_069: std_logic_vector(13 downto 0);
signal MacAll_070, MacAll_071, MacAll_072, MacAll_073, MacAll_074, MacAll_075, MacAll_076, MacAll_077, MacAll_078, MacAll_079: std_logic_vector(13 downto 0);
signal MacAll_080, MacAll_081, MacAll_082, MacAll_083, MacAll_084, MacAll_085, MacAll_086, MacAll_087, MacAll_088, MacAll_089: std_logic_vector(13 downto 0);
signal MacAll_090, MacAll_091, MacAll_092, MacAll_093, MacAll_094, MacAll_095, MacAll_096, MacAll_097, MacAll_098, MacAll_099: std_logic_vector(13 downto 0);
signal MacAll_100, MacAll_101, MacAll_102, MacAll_103, MacAll_104, MacAll_105, MacAll_106, MacAll_107, MacAll_108, MacAll_109: std_logic_vector(13 downto 0);
signal MacAll_110, MacAll_111, MacAll_112, MacAll_113, MacAll_114, MacAll_115, MacAll_116, MacAll_117, MacAll_118, MacAll_119: std_logic_vector(13 downto 0);
signal MacAll_120, MacAll_121, MacAll_122, MacAll_123, MacAll_124, MacAll_125, MacAll_126, MacAll_127: std_logic_vector(13 downto 0);

signal MacShift_000, MacShift_001, MacShift_002, MacShift_003, MacShift_004, MacShift_005, MacShift_006, MacShift_007, MacShift_008, MacShift_009: std_logic_vector(17 downto 0);
signal MacShift_010, MacShift_011, MacShift_012, MacShift_013, MacShift_014, MacShift_015, MacShift_016, MacShift_017, MacShift_018, MacShift_019: std_logic_vector(17 downto 0);
signal MacShift_020, MacShift_021, MacShift_022, MacShift_023, MacShift_024, MacShift_025, MacShift_026, MacShift_027, MacShift_028, MacShift_029: std_logic_vector(17 downto 0);
signal MacShift_030, MacShift_031, MacShift_032, MacShift_033, MacShift_034, MacShift_035, MacShift_036, MacShift_037, MacShift_038, MacShift_039: std_logic_vector(17 downto 0);
signal MacShift_040, MacShift_041, MacShift_042, MacShift_043, MacShift_044, MacShift_045, MacShift_046, MacShift_047, MacShift_048, MacShift_049: std_logic_vector(17 downto 0);
signal MacShift_050, MacShift_051, MacShift_052, MacShift_053, MacShift_054, MacShift_055, MacShift_056, MacShift_057, MacShift_058, MacShift_059: std_logic_vector(17 downto 0);
signal MacShift_060, MacShift_061, MacShift_062, MacShift_063, MacShift_064, MacShift_065, MacShift_066, MacShift_067, MacShift_068, MacShift_069: std_logic_vector(17 downto 0);
signal MacShift_070, MacShift_071, MacShift_072, MacShift_073, MacShift_074, MacShift_075, MacShift_076, MacShift_077, MacShift_078, MacShift_079: std_logic_vector(17 downto 0);
signal MacShift_080, MacShift_081, MacShift_082, MacShift_083, MacShift_084, MacShift_085, MacShift_086, MacShift_087, MacShift_088, MacShift_089: std_logic_vector(17 downto 0);
signal MacShift_090, MacShift_091, MacShift_092, MacShift_093, MacShift_094, MacShift_095, MacShift_096, MacShift_097, MacShift_098, MacShift_099: std_logic_vector(17 downto 0);
signal MacShift_100, MacShift_101, MacShift_102, MacShift_103, MacShift_104, MacShift_105, MacShift_106, MacShift_107, MacShift_108, MacShift_109: std_logic_vector(17 downto 0);
signal MacShift_110, MacShift_111, MacShift_112, MacShift_113, MacShift_114, MacShift_115, MacShift_116, MacShift_117, MacShift_118, MacShift_119: std_logic_vector(17 downto 0);
signal MacShift_120, MacShift_121, MacShift_122, MacShift_123, MacShift_124, MacShift_125, MacShift_126, MacShift_127: std_logic_vector(17 downto 0);

signal BIAS_000, BIAS_001, BIAS_002, BIAS_003, BIAS_004, BIAS_005, BIAS_006, BIAS_007, BIAS_008, BIAS_009: std_logic_vector(17 downto 0);
signal BIAS_010, BIAS_011, BIAS_012, BIAS_013, BIAS_014, BIAS_015, BIAS_016, BIAS_017, BIAS_018, BIAS_019: std_logic_vector(17 downto 0);
signal BIAS_020, BIAS_021, BIAS_022, BIAS_023, BIAS_024, BIAS_025, BIAS_026, BIAS_027, BIAS_028, BIAS_029: std_logic_vector(17 downto 0);
signal BIAS_030, BIAS_031, BIAS_032, BIAS_033, BIAS_034, BIAS_035, BIAS_036, BIAS_037, BIAS_038, BIAS_039: std_logic_vector(17 downto 0);
signal BIAS_040, BIAS_041, BIAS_042, BIAS_043, BIAS_044, BIAS_045, BIAS_046, BIAS_047, BIAS_048, BIAS_049: std_logic_vector(17 downto 0);
signal BIAS_050, BIAS_051, BIAS_052, BIAS_053, BIAS_054, BIAS_055, BIAS_056, BIAS_057, BIAS_058, BIAS_059: std_logic_vector(17 downto 0);
signal BIAS_060, BIAS_061, BIAS_062, BIAS_063, BIAS_064, BIAS_065, BIAS_066, BIAS_067, BIAS_068, BIAS_069: std_logic_vector(17 downto 0);
signal BIAS_070, BIAS_071, BIAS_072, BIAS_073, BIAS_074, BIAS_075, BIAS_076, BIAS_077, BIAS_078, BIAS_079: std_logic_vector(17 downto 0);
signal BIAS_080, BIAS_081, BIAS_082, BIAS_083, BIAS_084, BIAS_085, BIAS_086, BIAS_087, BIAS_088, BIAS_089: std_logic_vector(17 downto 0);
signal BIAS_090, BIAS_091, BIAS_092, BIAS_093, BIAS_094, BIAS_095, BIAS_096, BIAS_097, BIAS_098, BIAS_099: std_logic_vector(17 downto 0);
signal BIAS_100, BIAS_101, BIAS_102, BIAS_103, BIAS_104, BIAS_105, BIAS_106, BIAS_107, BIAS_108, BIAS_109: std_logic_vector(17 downto 0);
signal BIAS_110, BIAS_111, BIAS_112, BIAS_113, BIAS_114, BIAS_115, BIAS_116, BIAS_117, BIAS_118, BIAS_119: std_logic_vector(17 downto 0);
signal BIAS_120, BIAS_121, BIAS_122, BIAS_123, BIAS_124, BIAS_125, BIAS_126, BIAS_127: std_logic_vector(17 downto 0);

signal plusBias_000, plusBias_001, plusBias_002, plusBias_003, plusBias_004, plusBias_005, plusBias_006, plusBias_007, plusBias_008, plusBias_009: std_logic_vector(17 downto 0);
signal plusBias_010, plusBias_011, plusBias_012, plusBias_013, plusBias_014, plusBias_015, plusBias_016, plusBias_017, plusBias_018, plusBias_019: std_logic_vector(17 downto 0);
signal plusBias_020, plusBias_021, plusBias_022, plusBias_023, plusBias_024, plusBias_025, plusBias_026, plusBias_027, plusBias_028, plusBias_029: std_logic_vector(17 downto 0);
signal plusBias_030, plusBias_031, plusBias_032, plusBias_033, plusBias_034, plusBias_035, plusBias_036, plusBias_037, plusBias_038, plusBias_039: std_logic_vector(17 downto 0);
signal plusBias_040, plusBias_041, plusBias_042, plusBias_043, plusBias_044, plusBias_045, plusBias_046, plusBias_047, plusBias_048, plusBias_049: std_logic_vector(17 downto 0);
signal plusBias_050, plusBias_051, plusBias_052, plusBias_053, plusBias_054, plusBias_055, plusBias_056, plusBias_057, plusBias_058, plusBias_059: std_logic_vector(17 downto 0);
signal plusBias_060, plusBias_061, plusBias_062, plusBias_063, plusBias_064, plusBias_065, plusBias_066, plusBias_067, plusBias_068, plusBias_069: std_logic_vector(17 downto 0);
signal plusBias_070, plusBias_071, plusBias_072, plusBias_073, plusBias_074, plusBias_075, plusBias_076, plusBias_077, plusBias_078, plusBias_079: std_logic_vector(17 downto 0);
signal plusBias_080, plusBias_081, plusBias_082, plusBias_083, plusBias_084, plusBias_085, plusBias_086, plusBias_087, plusBias_088, plusBias_089: std_logic_vector(17 downto 0);
signal plusBias_090, plusBias_091, plusBias_092, plusBias_093, plusBias_094, plusBias_095, plusBias_096, plusBias_097, plusBias_098, plusBias_099: std_logic_vector(17 downto 0);
signal plusBias_100, plusBias_101, plusBias_102, plusBias_103, plusBias_104, plusBias_105, plusBias_106, plusBias_107, plusBias_108, plusBias_109: std_logic_vector(17 downto 0);
signal plusBias_110, plusBias_111, plusBias_112, plusBias_113, plusBias_114, plusBias_115, plusBias_116, plusBias_117, plusBias_118, plusBias_119: std_logic_vector(17 downto 0);
signal plusBias_120, plusBias_121, plusBias_122, plusBias_123, plusBias_124, plusBias_125, plusBias_126, plusBias_127: std_logic_vector(17 downto 0);

signal truncate_000, truncate_001, truncate_002, truncate_003, truncate_004, truncate_005, truncate_006, truncate_007, truncate_008, truncate_009: std_logic_vector(9 downto 0);
signal truncate_010, truncate_011, truncate_012, truncate_013, truncate_014, truncate_015, truncate_016, truncate_017, truncate_018, truncate_019: std_logic_vector(9 downto 0);
signal truncate_020, truncate_021, truncate_022, truncate_023, truncate_024, truncate_025, truncate_026, truncate_027, truncate_028, truncate_029: std_logic_vector(9 downto 0);
signal truncate_030, truncate_031, truncate_032, truncate_033, truncate_034, truncate_035, truncate_036, truncate_037, truncate_038, truncate_039: std_logic_vector(9 downto 0);
signal truncate_040, truncate_041, truncate_042, truncate_043, truncate_044, truncate_045, truncate_046, truncate_047, truncate_048, truncate_049: std_logic_vector(9 downto 0);
signal truncate_050, truncate_051, truncate_052, truncate_053, truncate_054, truncate_055, truncate_056, truncate_057, truncate_058, truncate_059: std_logic_vector(9 downto 0);
signal truncate_060, truncate_061, truncate_062, truncate_063, truncate_064, truncate_065, truncate_066, truncate_067, truncate_068, truncate_069: std_logic_vector(9 downto 0);
signal truncate_070, truncate_071, truncate_072, truncate_073, truncate_074, truncate_075, truncate_076, truncate_077, truncate_078, truncate_079: std_logic_vector(9 downto 0);
signal truncate_080, truncate_081, truncate_082, truncate_083, truncate_084, truncate_085, truncate_086, truncate_087, truncate_088, truncate_089: std_logic_vector(9 downto 0);
signal truncate_090, truncate_091, truncate_092, truncate_093, truncate_094, truncate_095, truncate_096, truncate_097, truncate_098, truncate_099: std_logic_vector(9 downto 0);
signal truncate_100, truncate_101, truncate_102, truncate_103, truncate_104, truncate_105, truncate_106, truncate_107, truncate_108, truncate_109: std_logic_vector(9 downto 0);
signal truncate_110, truncate_111, truncate_112, truncate_113, truncate_114, truncate_115, truncate_116, truncate_117, truncate_118, truncate_119: std_logic_vector(9 downto 0);
signal truncate_120, truncate_121, truncate_122, truncate_123, truncate_124, truncate_125, truncate_126, truncate_127: std_logic_vector(9 downto 0);

signal Relu_000, Relu_001, Relu_002, Relu_003, Relu_004, Relu_005, Relu_006, Relu_007, Relu_008, Relu_009: std_logic_vector(4 downto 0);
signal Relu_010, Relu_011, Relu_012, Relu_013, Relu_014, Relu_015, Relu_016, Relu_017, Relu_018, Relu_019: std_logic_vector(4 downto 0);
signal Relu_020, Relu_021, Relu_022, Relu_023, Relu_024, Relu_025, Relu_026, Relu_027, Relu_028, Relu_029: std_logic_vector(4 downto 0);
signal Relu_030, Relu_031, Relu_032, Relu_033, Relu_034, Relu_035, Relu_036, Relu_037, Relu_038, Relu_039: std_logic_vector(4 downto 0);
signal Relu_040, Relu_041, Relu_042, Relu_043, Relu_044, Relu_045, Relu_046, Relu_047, Relu_048, Relu_049: std_logic_vector(4 downto 0);
signal Relu_050, Relu_051, Relu_052, Relu_053, Relu_054, Relu_055, Relu_056, Relu_057, Relu_058, Relu_059: std_logic_vector(4 downto 0);
signal Relu_060, Relu_061, Relu_062, Relu_063, Relu_064, Relu_065, Relu_066, Relu_067, Relu_068, Relu_069: std_logic_vector(4 downto 0);
signal Relu_070, Relu_071, Relu_072, Relu_073, Relu_074, Relu_075, Relu_076, Relu_077, Relu_078, Relu_079: std_logic_vector(4 downto 0);
signal Relu_080, Relu_081, Relu_082, Relu_083, Relu_084, Relu_085, Relu_086, Relu_087, Relu_088, Relu_089: std_logic_vector(4 downto 0);
signal Relu_090, Relu_091, Relu_092, Relu_093, Relu_094, Relu_095, Relu_096, Relu_097, Relu_098, Relu_099: std_logic_vector(4 downto 0);
signal Relu_100, Relu_101, Relu_102, Relu_103, Relu_104, Relu_105, Relu_106, Relu_107, Relu_108, Relu_109: std_logic_vector(4 downto 0);
signal Relu_110, Relu_111, Relu_112, Relu_113, Relu_114, Relu_115, Relu_116, Relu_117, Relu_118, Relu_119: std_logic_vector(4 downto 0);
signal Relu_120, Relu_121, Relu_122, Relu_123, Relu_124, Relu_125, Relu_126, Relu_127: std_logic_vector(4 downto 0);

signal ReluAll: std_logic_vector(639 downto 0);

signal CO9is0 : std_logic;

signal inch_00, inch_01, inch_02, inch_03 : std_logic_vector(4 downto 0);
signal in3ChSel_dly : std_logic_vector(3 downto 0);
signal CnnStart_dly : std_logic;
signal CO9_p0,CO9_p9,CO9_p18,CO9_p27,CO9_p36,CO9_p45,CO9_p54,CO9_p63,CO9_p72,CO9_p81,CO9_p90,CO9_p99,CO9_p108,CO9_p117,CO9_p126,CO9_p135 : std_logic_vector(7 downto 0);

signal pool_00, pool_01, pool_02, pool_03: std_logic_vector(4 downto 0);
signal Poolout: std_logic_vector(639 downto 0);

begin

inch_00 <= 
LAY4IN( 319 downto 315 ) when in3ChSel_dly = "0000" else 
LAY4IN( 314 downto 310 ) when in3ChSel_dly = "0001" else 
LAY4IN( 309 downto 305 ) when in3ChSel_dly = "0010" else 
LAY4IN( 304 downto 300 ) when in3ChSel_dly = "0011" else 
LAY4IN( 299 downto 295 ) when in3ChSel_dly = "0100" else 
LAY4IN( 294 downto 290 ) when in3ChSel_dly = "0101" else 
LAY4IN( 289 downto 285 ) when in3ChSel_dly = "0110" else 
LAY4IN( 284 downto 280 ) when in3ChSel_dly = "0111" else 
LAY4IN( 279 downto 275 ) when in3ChSel_dly = "1000" else 
LAY4IN( 274 downto 270 ) when in3ChSel_dly = "1001" else 
LAY4IN( 269 downto 265 ) when in3ChSel_dly = "1010" else 
LAY4IN( 264 downto 260 ) when in3ChSel_dly = "1011" else 
LAY4IN( 259 downto 255 ) when in3ChSel_dly = "1100" else 
LAY4IN( 254 downto 250 ) when in3ChSel_dly = "1101" else 
LAY4IN( 249 downto 245 ) when in3ChSel_dly = "1110" else 
LAY4IN( 244 downto 240 ); -- when in3ChSel_dly = "1111" else 

inch_01 <= 
LAY4IN( 239 downto 235 ) when in3ChSel_dly = "0000" else 
LAY4IN( 234 downto 230 ) when in3ChSel_dly = "0001" else 
LAY4IN( 229 downto 225 ) when in3ChSel_dly = "0010" else 
LAY4IN( 224 downto 220 ) when in3ChSel_dly = "0011" else 
LAY4IN( 219 downto 215 ) when in3ChSel_dly = "0100" else 
LAY4IN( 214 downto 210 ) when in3ChSel_dly = "0101" else 
LAY4IN( 209 downto 205 ) when in3ChSel_dly = "0110" else 
LAY4IN( 204 downto 200 ) when in3ChSel_dly = "0111" else 
LAY4IN( 199 downto 195 ) when in3ChSel_dly = "1000" else 
LAY4IN( 194 downto 190 ) when in3ChSel_dly = "1001" else 
LAY4IN( 189 downto 185 ) when in3ChSel_dly = "1010" else 
LAY4IN( 184 downto 180 ) when in3ChSel_dly = "1011" else 
LAY4IN( 179 downto 175 ) when in3ChSel_dly = "1100" else 
LAY4IN( 174 downto 170 ) when in3ChSel_dly = "1101" else 
LAY4IN( 169 downto 165 ) when in3ChSel_dly = "1110" else 
LAY4IN( 164 downto 160 ); -- when in3ChSel_dly = "1111" else 

inch_02 <= 
LAY4IN( 159 downto 155 ) when in3ChSel_dly = "0000" else 
LAY4IN( 154 downto 150 ) when in3ChSel_dly = "0001" else 
LAY4IN( 149 downto 145 ) when in3ChSel_dly = "0010" else 
LAY4IN( 144 downto 140 ) when in3ChSel_dly = "0011" else 
LAY4IN( 139 downto 135 ) when in3ChSel_dly = "0100" else 
LAY4IN( 134 downto 130 ) when in3ChSel_dly = "0101" else 
LAY4IN( 129 downto 125 ) when in3ChSel_dly = "0110" else 
LAY4IN( 124 downto 120 ) when in3ChSel_dly = "0111" else 
LAY4IN( 119 downto 115 ) when in3ChSel_dly = "1000" else 
LAY4IN( 114 downto 110 ) when in3ChSel_dly = "1001" else 
LAY4IN( 109 downto 105 ) when in3ChSel_dly = "1010" else 
LAY4IN( 104 downto 100 ) when in3ChSel_dly = "1011" else 
LAY4IN( 99 downto 95 ) when in3ChSel_dly = "1100" else 
LAY4IN( 94 downto 90 ) when in3ChSel_dly = "1101" else 
LAY4IN( 89 downto 85 ) when in3ChSel_dly = "1110" else 
LAY4IN( 84 downto 80 ); -- when in3ChSel_dly = "1111" else 

inch_03 <= 
LAY4IN( 79 downto 75 ) when in3ChSel_dly = "0000" else 
LAY4IN( 74 downto 70 ) when in3ChSel_dly = "0001" else 
LAY4IN( 69 downto 65 ) when in3ChSel_dly = "0010" else 
LAY4IN( 64 downto 60 ) when in3ChSel_dly = "0011" else 
LAY4IN( 59 downto 55 ) when in3ChSel_dly = "0100" else 
LAY4IN( 54 downto 50 ) when in3ChSel_dly = "0101" else 
LAY4IN( 49 downto 45 ) when in3ChSel_dly = "0110" else 
LAY4IN( 44 downto 40 ) when in3ChSel_dly = "0111" else 
LAY4IN( 39 downto 35 ) when in3ChSel_dly = "1000" else 
LAY4IN( 34 downto 30 ) when in3ChSel_dly = "1001" else 
LAY4IN( 29 downto 25 ) when in3ChSel_dly = "1010" else 
LAY4IN( 24 downto 20 ) when in3ChSel_dly = "1011" else 
LAY4IN( 19 downto 15 ) when in3ChSel_dly = "1100" else 
LAY4IN( 14 downto 10 ) when in3ChSel_dly = "1101" else 
LAY4IN( 9 downto 5 ) when in3ChSel_dly = "1110" else 
LAY4IN( 4 downto 0 ); -- when in3ChSel_dly = "1111" else 


  coeff_000_00 <= W4Data_00( 255 downto 254 );
  coeff_000_01 <= W4Data_01( 255 downto 254 );
  coeff_000_02 <= W4Data_02( 255 downto 254 );
  coeff_000_03 <= W4Data_03( 255 downto 254 );

  coeff_001_00 <= W4Data_00( 253 downto 252 );
  coeff_001_01 <= W4Data_01( 253 downto 252 );
  coeff_001_02 <= W4Data_02( 253 downto 252 );
  coeff_001_03 <= W4Data_03( 253 downto 252 );

  coeff_002_00 <= W4Data_00( 251 downto 250 );
  coeff_002_01 <= W4Data_01( 251 downto 250 );
  coeff_002_02 <= W4Data_02( 251 downto 250 );
  coeff_002_03 <= W4Data_03( 251 downto 250 );

  coeff_003_00 <= W4Data_00( 249 downto 248 );
  coeff_003_01 <= W4Data_01( 249 downto 248 );
  coeff_003_02 <= W4Data_02( 249 downto 248 );
  coeff_003_03 <= W4Data_03( 249 downto 248 );

  coeff_004_00 <= W4Data_00( 247 downto 246 );
  coeff_004_01 <= W4Data_01( 247 downto 246 );
  coeff_004_02 <= W4Data_02( 247 downto 246 );
  coeff_004_03 <= W4Data_03( 247 downto 246 );

  coeff_005_00 <= W4Data_00( 245 downto 244 );
  coeff_005_01 <= W4Data_01( 245 downto 244 );
  coeff_005_02 <= W4Data_02( 245 downto 244 );
  coeff_005_03 <= W4Data_03( 245 downto 244 );

  coeff_006_00 <= W4Data_00( 243 downto 242 );
  coeff_006_01 <= W4Data_01( 243 downto 242 );
  coeff_006_02 <= W4Data_02( 243 downto 242 );
  coeff_006_03 <= W4Data_03( 243 downto 242 );

  coeff_007_00 <= W4Data_00( 241 downto 240 );
  coeff_007_01 <= W4Data_01( 241 downto 240 );
  coeff_007_02 <= W4Data_02( 241 downto 240 );
  coeff_007_03 <= W4Data_03( 241 downto 240 );

  coeff_008_00 <= W4Data_00( 239 downto 238 );
  coeff_008_01 <= W4Data_01( 239 downto 238 );
  coeff_008_02 <= W4Data_02( 239 downto 238 );
  coeff_008_03 <= W4Data_03( 239 downto 238 );

  coeff_009_00 <= W4Data_00( 237 downto 236 );
  coeff_009_01 <= W4Data_01( 237 downto 236 );
  coeff_009_02 <= W4Data_02( 237 downto 236 );
  coeff_009_03 <= W4Data_03( 237 downto 236 );

  coeff_010_00 <= W4Data_00( 235 downto 234 );
  coeff_010_01 <= W4Data_01( 235 downto 234 );
  coeff_010_02 <= W4Data_02( 235 downto 234 );
  coeff_010_03 <= W4Data_03( 235 downto 234 );

  coeff_011_00 <= W4Data_00( 233 downto 232 );
  coeff_011_01 <= W4Data_01( 233 downto 232 );
  coeff_011_02 <= W4Data_02( 233 downto 232 );
  coeff_011_03 <= W4Data_03( 233 downto 232 );

  coeff_012_00 <= W4Data_00( 231 downto 230 );
  coeff_012_01 <= W4Data_01( 231 downto 230 );
  coeff_012_02 <= W4Data_02( 231 downto 230 );
  coeff_012_03 <= W4Data_03( 231 downto 230 );

  coeff_013_00 <= W4Data_00( 229 downto 228 );
  coeff_013_01 <= W4Data_01( 229 downto 228 );
  coeff_013_02 <= W4Data_02( 229 downto 228 );
  coeff_013_03 <= W4Data_03( 229 downto 228 );

  coeff_014_00 <= W4Data_00( 227 downto 226 );
  coeff_014_01 <= W4Data_01( 227 downto 226 );
  coeff_014_02 <= W4Data_02( 227 downto 226 );
  coeff_014_03 <= W4Data_03( 227 downto 226 );

  coeff_015_00 <= W4Data_00( 225 downto 224 );
  coeff_015_01 <= W4Data_01( 225 downto 224 );
  coeff_015_02 <= W4Data_02( 225 downto 224 );
  coeff_015_03 <= W4Data_03( 225 downto 224 );

  coeff_016_00 <= W4Data_00( 223 downto 222 );
  coeff_016_01 <= W4Data_01( 223 downto 222 );
  coeff_016_02 <= W4Data_02( 223 downto 222 );
  coeff_016_03 <= W4Data_03( 223 downto 222 );

  coeff_017_00 <= W4Data_00( 221 downto 220 );
  coeff_017_01 <= W4Data_01( 221 downto 220 );
  coeff_017_02 <= W4Data_02( 221 downto 220 );
  coeff_017_03 <= W4Data_03( 221 downto 220 );

  coeff_018_00 <= W4Data_00( 219 downto 218 );
  coeff_018_01 <= W4Data_01( 219 downto 218 );
  coeff_018_02 <= W4Data_02( 219 downto 218 );
  coeff_018_03 <= W4Data_03( 219 downto 218 );

  coeff_019_00 <= W4Data_00( 217 downto 216 );
  coeff_019_01 <= W4Data_01( 217 downto 216 );
  coeff_019_02 <= W4Data_02( 217 downto 216 );
  coeff_019_03 <= W4Data_03( 217 downto 216 );

  coeff_020_00 <= W4Data_00( 215 downto 214 );
  coeff_020_01 <= W4Data_01( 215 downto 214 );
  coeff_020_02 <= W4Data_02( 215 downto 214 );
  coeff_020_03 <= W4Data_03( 215 downto 214 );

  coeff_021_00 <= W4Data_00( 213 downto 212 );
  coeff_021_01 <= W4Data_01( 213 downto 212 );
  coeff_021_02 <= W4Data_02( 213 downto 212 );
  coeff_021_03 <= W4Data_03( 213 downto 212 );

  coeff_022_00 <= W4Data_00( 211 downto 210 );
  coeff_022_01 <= W4Data_01( 211 downto 210 );
  coeff_022_02 <= W4Data_02( 211 downto 210 );
  coeff_022_03 <= W4Data_03( 211 downto 210 );

  coeff_023_00 <= W4Data_00( 209 downto 208 );
  coeff_023_01 <= W4Data_01( 209 downto 208 );
  coeff_023_02 <= W4Data_02( 209 downto 208 );
  coeff_023_03 <= W4Data_03( 209 downto 208 );

  coeff_024_00 <= W4Data_00( 207 downto 206 );
  coeff_024_01 <= W4Data_01( 207 downto 206 );
  coeff_024_02 <= W4Data_02( 207 downto 206 );
  coeff_024_03 <= W4Data_03( 207 downto 206 );

  coeff_025_00 <= W4Data_00( 205 downto 204 );
  coeff_025_01 <= W4Data_01( 205 downto 204 );
  coeff_025_02 <= W4Data_02( 205 downto 204 );
  coeff_025_03 <= W4Data_03( 205 downto 204 );

  coeff_026_00 <= W4Data_00( 203 downto 202 );
  coeff_026_01 <= W4Data_01( 203 downto 202 );
  coeff_026_02 <= W4Data_02( 203 downto 202 );
  coeff_026_03 <= W4Data_03( 203 downto 202 );

  coeff_027_00 <= W4Data_00( 201 downto 200 );
  coeff_027_01 <= W4Data_01( 201 downto 200 );
  coeff_027_02 <= W4Data_02( 201 downto 200 );
  coeff_027_03 <= W4Data_03( 201 downto 200 );

  coeff_028_00 <= W4Data_00( 199 downto 198 );
  coeff_028_01 <= W4Data_01( 199 downto 198 );
  coeff_028_02 <= W4Data_02( 199 downto 198 );
  coeff_028_03 <= W4Data_03( 199 downto 198 );

  coeff_029_00 <= W4Data_00( 197 downto 196 );
  coeff_029_01 <= W4Data_01( 197 downto 196 );
  coeff_029_02 <= W4Data_02( 197 downto 196 );
  coeff_029_03 <= W4Data_03( 197 downto 196 );

  coeff_030_00 <= W4Data_00( 195 downto 194 );
  coeff_030_01 <= W4Data_01( 195 downto 194 );
  coeff_030_02 <= W4Data_02( 195 downto 194 );
  coeff_030_03 <= W4Data_03( 195 downto 194 );

  coeff_031_00 <= W4Data_00( 193 downto 192 );
  coeff_031_01 <= W4Data_01( 193 downto 192 );
  coeff_031_02 <= W4Data_02( 193 downto 192 );
  coeff_031_03 <= W4Data_03( 193 downto 192 );

  coeff_032_00 <= W4Data_00( 191 downto 190 );
  coeff_032_01 <= W4Data_01( 191 downto 190 );
  coeff_032_02 <= W4Data_02( 191 downto 190 );
  coeff_032_03 <= W4Data_03( 191 downto 190 );

  coeff_033_00 <= W4Data_00( 189 downto 188 );
  coeff_033_01 <= W4Data_01( 189 downto 188 );
  coeff_033_02 <= W4Data_02( 189 downto 188 );
  coeff_033_03 <= W4Data_03( 189 downto 188 );

  coeff_034_00 <= W4Data_00( 187 downto 186 );
  coeff_034_01 <= W4Data_01( 187 downto 186 );
  coeff_034_02 <= W4Data_02( 187 downto 186 );
  coeff_034_03 <= W4Data_03( 187 downto 186 );

  coeff_035_00 <= W4Data_00( 185 downto 184 );
  coeff_035_01 <= W4Data_01( 185 downto 184 );
  coeff_035_02 <= W4Data_02( 185 downto 184 );
  coeff_035_03 <= W4Data_03( 185 downto 184 );

  coeff_036_00 <= W4Data_00( 183 downto 182 );
  coeff_036_01 <= W4Data_01( 183 downto 182 );
  coeff_036_02 <= W4Data_02( 183 downto 182 );
  coeff_036_03 <= W4Data_03( 183 downto 182 );

  coeff_037_00 <= W4Data_00( 181 downto 180 );
  coeff_037_01 <= W4Data_01( 181 downto 180 );
  coeff_037_02 <= W4Data_02( 181 downto 180 );
  coeff_037_03 <= W4Data_03( 181 downto 180 );

  coeff_038_00 <= W4Data_00( 179 downto 178 );
  coeff_038_01 <= W4Data_01( 179 downto 178 );
  coeff_038_02 <= W4Data_02( 179 downto 178 );
  coeff_038_03 <= W4Data_03( 179 downto 178 );

  coeff_039_00 <= W4Data_00( 177 downto 176 );
  coeff_039_01 <= W4Data_01( 177 downto 176 );
  coeff_039_02 <= W4Data_02( 177 downto 176 );
  coeff_039_03 <= W4Data_03( 177 downto 176 );

  coeff_040_00 <= W4Data_00( 175 downto 174 );
  coeff_040_01 <= W4Data_01( 175 downto 174 );
  coeff_040_02 <= W4Data_02( 175 downto 174 );
  coeff_040_03 <= W4Data_03( 175 downto 174 );

  coeff_041_00 <= W4Data_00( 173 downto 172 );
  coeff_041_01 <= W4Data_01( 173 downto 172 );
  coeff_041_02 <= W4Data_02( 173 downto 172 );
  coeff_041_03 <= W4Data_03( 173 downto 172 );

  coeff_042_00 <= W4Data_00( 171 downto 170 );
  coeff_042_01 <= W4Data_01( 171 downto 170 );
  coeff_042_02 <= W4Data_02( 171 downto 170 );
  coeff_042_03 <= W4Data_03( 171 downto 170 );

  coeff_043_00 <= W4Data_00( 169 downto 168 );
  coeff_043_01 <= W4Data_01( 169 downto 168 );
  coeff_043_02 <= W4Data_02( 169 downto 168 );
  coeff_043_03 <= W4Data_03( 169 downto 168 );

  coeff_044_00 <= W4Data_00( 167 downto 166 );
  coeff_044_01 <= W4Data_01( 167 downto 166 );
  coeff_044_02 <= W4Data_02( 167 downto 166 );
  coeff_044_03 <= W4Data_03( 167 downto 166 );

  coeff_045_00 <= W4Data_00( 165 downto 164 );
  coeff_045_01 <= W4Data_01( 165 downto 164 );
  coeff_045_02 <= W4Data_02( 165 downto 164 );
  coeff_045_03 <= W4Data_03( 165 downto 164 );

  coeff_046_00 <= W4Data_00( 163 downto 162 );
  coeff_046_01 <= W4Data_01( 163 downto 162 );
  coeff_046_02 <= W4Data_02( 163 downto 162 );
  coeff_046_03 <= W4Data_03( 163 downto 162 );

  coeff_047_00 <= W4Data_00( 161 downto 160 );
  coeff_047_01 <= W4Data_01( 161 downto 160 );
  coeff_047_02 <= W4Data_02( 161 downto 160 );
  coeff_047_03 <= W4Data_03( 161 downto 160 );

  coeff_048_00 <= W4Data_00( 159 downto 158 );
  coeff_048_01 <= W4Data_01( 159 downto 158 );
  coeff_048_02 <= W4Data_02( 159 downto 158 );
  coeff_048_03 <= W4Data_03( 159 downto 158 );

  coeff_049_00 <= W4Data_00( 157 downto 156 );
  coeff_049_01 <= W4Data_01( 157 downto 156 );
  coeff_049_02 <= W4Data_02( 157 downto 156 );
  coeff_049_03 <= W4Data_03( 157 downto 156 );

  coeff_050_00 <= W4Data_00( 155 downto 154 );
  coeff_050_01 <= W4Data_01( 155 downto 154 );
  coeff_050_02 <= W4Data_02( 155 downto 154 );
  coeff_050_03 <= W4Data_03( 155 downto 154 );

  coeff_051_00 <= W4Data_00( 153 downto 152 );
  coeff_051_01 <= W4Data_01( 153 downto 152 );
  coeff_051_02 <= W4Data_02( 153 downto 152 );
  coeff_051_03 <= W4Data_03( 153 downto 152 );

  coeff_052_00 <= W4Data_00( 151 downto 150 );
  coeff_052_01 <= W4Data_01( 151 downto 150 );
  coeff_052_02 <= W4Data_02( 151 downto 150 );
  coeff_052_03 <= W4Data_03( 151 downto 150 );

  coeff_053_00 <= W4Data_00( 149 downto 148 );
  coeff_053_01 <= W4Data_01( 149 downto 148 );
  coeff_053_02 <= W4Data_02( 149 downto 148 );
  coeff_053_03 <= W4Data_03( 149 downto 148 );

  coeff_054_00 <= W4Data_00( 147 downto 146 );
  coeff_054_01 <= W4Data_01( 147 downto 146 );
  coeff_054_02 <= W4Data_02( 147 downto 146 );
  coeff_054_03 <= W4Data_03( 147 downto 146 );

  coeff_055_00 <= W4Data_00( 145 downto 144 );
  coeff_055_01 <= W4Data_01( 145 downto 144 );
  coeff_055_02 <= W4Data_02( 145 downto 144 );
  coeff_055_03 <= W4Data_03( 145 downto 144 );

  coeff_056_00 <= W4Data_00( 143 downto 142 );
  coeff_056_01 <= W4Data_01( 143 downto 142 );
  coeff_056_02 <= W4Data_02( 143 downto 142 );
  coeff_056_03 <= W4Data_03( 143 downto 142 );

  coeff_057_00 <= W4Data_00( 141 downto 140 );
  coeff_057_01 <= W4Data_01( 141 downto 140 );
  coeff_057_02 <= W4Data_02( 141 downto 140 );
  coeff_057_03 <= W4Data_03( 141 downto 140 );

  coeff_058_00 <= W4Data_00( 139 downto 138 );
  coeff_058_01 <= W4Data_01( 139 downto 138 );
  coeff_058_02 <= W4Data_02( 139 downto 138 );
  coeff_058_03 <= W4Data_03( 139 downto 138 );

  coeff_059_00 <= W4Data_00( 137 downto 136 );
  coeff_059_01 <= W4Data_01( 137 downto 136 );
  coeff_059_02 <= W4Data_02( 137 downto 136 );
  coeff_059_03 <= W4Data_03( 137 downto 136 );

  coeff_060_00 <= W4Data_00( 135 downto 134 );
  coeff_060_01 <= W4Data_01( 135 downto 134 );
  coeff_060_02 <= W4Data_02( 135 downto 134 );
  coeff_060_03 <= W4Data_03( 135 downto 134 );

  coeff_061_00 <= W4Data_00( 133 downto 132 );
  coeff_061_01 <= W4Data_01( 133 downto 132 );
  coeff_061_02 <= W4Data_02( 133 downto 132 );
  coeff_061_03 <= W4Data_03( 133 downto 132 );

  coeff_062_00 <= W4Data_00( 131 downto 130 );
  coeff_062_01 <= W4Data_01( 131 downto 130 );
  coeff_062_02 <= W4Data_02( 131 downto 130 );
  coeff_062_03 <= W4Data_03( 131 downto 130 );

  coeff_063_00 <= W4Data_00( 129 downto 128 );
  coeff_063_01 <= W4Data_01( 129 downto 128 );
  coeff_063_02 <= W4Data_02( 129 downto 128 );
  coeff_063_03 <= W4Data_03( 129 downto 128 );

  coeff_064_00 <= W4Data_00( 127 downto 126 );
  coeff_064_01 <= W4Data_01( 127 downto 126 );
  coeff_064_02 <= W4Data_02( 127 downto 126 );
  coeff_064_03 <= W4Data_03( 127 downto 126 );

  coeff_065_00 <= W4Data_00( 125 downto 124 );
  coeff_065_01 <= W4Data_01( 125 downto 124 );
  coeff_065_02 <= W4Data_02( 125 downto 124 );
  coeff_065_03 <= W4Data_03( 125 downto 124 );

  coeff_066_00 <= W4Data_00( 123 downto 122 );
  coeff_066_01 <= W4Data_01( 123 downto 122 );
  coeff_066_02 <= W4Data_02( 123 downto 122 );
  coeff_066_03 <= W4Data_03( 123 downto 122 );

  coeff_067_00 <= W4Data_00( 121 downto 120 );
  coeff_067_01 <= W4Data_01( 121 downto 120 );
  coeff_067_02 <= W4Data_02( 121 downto 120 );
  coeff_067_03 <= W4Data_03( 121 downto 120 );

  coeff_068_00 <= W4Data_00( 119 downto 118 );
  coeff_068_01 <= W4Data_01( 119 downto 118 );
  coeff_068_02 <= W4Data_02( 119 downto 118 );
  coeff_068_03 <= W4Data_03( 119 downto 118 );

  coeff_069_00 <= W4Data_00( 117 downto 116 );
  coeff_069_01 <= W4Data_01( 117 downto 116 );
  coeff_069_02 <= W4Data_02( 117 downto 116 );
  coeff_069_03 <= W4Data_03( 117 downto 116 );

  coeff_070_00 <= W4Data_00( 115 downto 114 );
  coeff_070_01 <= W4Data_01( 115 downto 114 );
  coeff_070_02 <= W4Data_02( 115 downto 114 );
  coeff_070_03 <= W4Data_03( 115 downto 114 );

  coeff_071_00 <= W4Data_00( 113 downto 112 );
  coeff_071_01 <= W4Data_01( 113 downto 112 );
  coeff_071_02 <= W4Data_02( 113 downto 112 );
  coeff_071_03 <= W4Data_03( 113 downto 112 );

  coeff_072_00 <= W4Data_00( 111 downto 110 );
  coeff_072_01 <= W4Data_01( 111 downto 110 );
  coeff_072_02 <= W4Data_02( 111 downto 110 );
  coeff_072_03 <= W4Data_03( 111 downto 110 );

  coeff_073_00 <= W4Data_00( 109 downto 108 );
  coeff_073_01 <= W4Data_01( 109 downto 108 );
  coeff_073_02 <= W4Data_02( 109 downto 108 );
  coeff_073_03 <= W4Data_03( 109 downto 108 );

  coeff_074_00 <= W4Data_00( 107 downto 106 );
  coeff_074_01 <= W4Data_01( 107 downto 106 );
  coeff_074_02 <= W4Data_02( 107 downto 106 );
  coeff_074_03 <= W4Data_03( 107 downto 106 );

  coeff_075_00 <= W4Data_00( 105 downto 104 );
  coeff_075_01 <= W4Data_01( 105 downto 104 );
  coeff_075_02 <= W4Data_02( 105 downto 104 );
  coeff_075_03 <= W4Data_03( 105 downto 104 );

  coeff_076_00 <= W4Data_00( 103 downto 102 );
  coeff_076_01 <= W4Data_01( 103 downto 102 );
  coeff_076_02 <= W4Data_02( 103 downto 102 );
  coeff_076_03 <= W4Data_03( 103 downto 102 );

  coeff_077_00 <= W4Data_00( 101 downto 100 );
  coeff_077_01 <= W4Data_01( 101 downto 100 );
  coeff_077_02 <= W4Data_02( 101 downto 100 );
  coeff_077_03 <= W4Data_03( 101 downto 100 );

  coeff_078_00 <= W4Data_00( 99 downto 98 );
  coeff_078_01 <= W4Data_01( 99 downto 98 );
  coeff_078_02 <= W4Data_02( 99 downto 98 );
  coeff_078_03 <= W4Data_03( 99 downto 98 );

  coeff_079_00 <= W4Data_00( 97 downto 96 );
  coeff_079_01 <= W4Data_01( 97 downto 96 );
  coeff_079_02 <= W4Data_02( 97 downto 96 );
  coeff_079_03 <= W4Data_03( 97 downto 96 );

  coeff_080_00 <= W4Data_00( 95 downto 94 );
  coeff_080_01 <= W4Data_01( 95 downto 94 );
  coeff_080_02 <= W4Data_02( 95 downto 94 );
  coeff_080_03 <= W4Data_03( 95 downto 94 );

  coeff_081_00 <= W4Data_00( 93 downto 92 );
  coeff_081_01 <= W4Data_01( 93 downto 92 );
  coeff_081_02 <= W4Data_02( 93 downto 92 );
  coeff_081_03 <= W4Data_03( 93 downto 92 );

  coeff_082_00 <= W4Data_00( 91 downto 90 );
  coeff_082_01 <= W4Data_01( 91 downto 90 );
  coeff_082_02 <= W4Data_02( 91 downto 90 );
  coeff_082_03 <= W4Data_03( 91 downto 90 );

  coeff_083_00 <= W4Data_00( 89 downto 88 );
  coeff_083_01 <= W4Data_01( 89 downto 88 );
  coeff_083_02 <= W4Data_02( 89 downto 88 );
  coeff_083_03 <= W4Data_03( 89 downto 88 );

  coeff_084_00 <= W4Data_00( 87 downto 86 );
  coeff_084_01 <= W4Data_01( 87 downto 86 );
  coeff_084_02 <= W4Data_02( 87 downto 86 );
  coeff_084_03 <= W4Data_03( 87 downto 86 );

  coeff_085_00 <= W4Data_00( 85 downto 84 );
  coeff_085_01 <= W4Data_01( 85 downto 84 );
  coeff_085_02 <= W4Data_02( 85 downto 84 );
  coeff_085_03 <= W4Data_03( 85 downto 84 );

  coeff_086_00 <= W4Data_00( 83 downto 82 );
  coeff_086_01 <= W4Data_01( 83 downto 82 );
  coeff_086_02 <= W4Data_02( 83 downto 82 );
  coeff_086_03 <= W4Data_03( 83 downto 82 );

  coeff_087_00 <= W4Data_00( 81 downto 80 );
  coeff_087_01 <= W4Data_01( 81 downto 80 );
  coeff_087_02 <= W4Data_02( 81 downto 80 );
  coeff_087_03 <= W4Data_03( 81 downto 80 );

  coeff_088_00 <= W4Data_00( 79 downto 78 );
  coeff_088_01 <= W4Data_01( 79 downto 78 );
  coeff_088_02 <= W4Data_02( 79 downto 78 );
  coeff_088_03 <= W4Data_03( 79 downto 78 );

  coeff_089_00 <= W4Data_00( 77 downto 76 );
  coeff_089_01 <= W4Data_01( 77 downto 76 );
  coeff_089_02 <= W4Data_02( 77 downto 76 );
  coeff_089_03 <= W4Data_03( 77 downto 76 );

  coeff_090_00 <= W4Data_00( 75 downto 74 );
  coeff_090_01 <= W4Data_01( 75 downto 74 );
  coeff_090_02 <= W4Data_02( 75 downto 74 );
  coeff_090_03 <= W4Data_03( 75 downto 74 );

  coeff_091_00 <= W4Data_00( 73 downto 72 );
  coeff_091_01 <= W4Data_01( 73 downto 72 );
  coeff_091_02 <= W4Data_02( 73 downto 72 );
  coeff_091_03 <= W4Data_03( 73 downto 72 );

  coeff_092_00 <= W4Data_00( 71 downto 70 );
  coeff_092_01 <= W4Data_01( 71 downto 70 );
  coeff_092_02 <= W4Data_02( 71 downto 70 );
  coeff_092_03 <= W4Data_03( 71 downto 70 );

  coeff_093_00 <= W4Data_00( 69 downto 68 );
  coeff_093_01 <= W4Data_01( 69 downto 68 );
  coeff_093_02 <= W4Data_02( 69 downto 68 );
  coeff_093_03 <= W4Data_03( 69 downto 68 );

  coeff_094_00 <= W4Data_00( 67 downto 66 );
  coeff_094_01 <= W4Data_01( 67 downto 66 );
  coeff_094_02 <= W4Data_02( 67 downto 66 );
  coeff_094_03 <= W4Data_03( 67 downto 66 );

  coeff_095_00 <= W4Data_00( 65 downto 64 );
  coeff_095_01 <= W4Data_01( 65 downto 64 );
  coeff_095_02 <= W4Data_02( 65 downto 64 );
  coeff_095_03 <= W4Data_03( 65 downto 64 );

  coeff_096_00 <= W4Data_00( 63 downto 62 );
  coeff_096_01 <= W4Data_01( 63 downto 62 );
  coeff_096_02 <= W4Data_02( 63 downto 62 );
  coeff_096_03 <= W4Data_03( 63 downto 62 );

  coeff_097_00 <= W4Data_00( 61 downto 60 );
  coeff_097_01 <= W4Data_01( 61 downto 60 );
  coeff_097_02 <= W4Data_02( 61 downto 60 );
  coeff_097_03 <= W4Data_03( 61 downto 60 );

  coeff_098_00 <= W4Data_00( 59 downto 58 );
  coeff_098_01 <= W4Data_01( 59 downto 58 );
  coeff_098_02 <= W4Data_02( 59 downto 58 );
  coeff_098_03 <= W4Data_03( 59 downto 58 );

  coeff_099_00 <= W4Data_00( 57 downto 56 );
  coeff_099_01 <= W4Data_01( 57 downto 56 );
  coeff_099_02 <= W4Data_02( 57 downto 56 );
  coeff_099_03 <= W4Data_03( 57 downto 56 );

  coeff_100_00 <= W4Data_00( 55 downto 54 );
  coeff_100_01 <= W4Data_01( 55 downto 54 );
  coeff_100_02 <= W4Data_02( 55 downto 54 );
  coeff_100_03 <= W4Data_03( 55 downto 54 );

  coeff_101_00 <= W4Data_00( 53 downto 52 );
  coeff_101_01 <= W4Data_01( 53 downto 52 );
  coeff_101_02 <= W4Data_02( 53 downto 52 );
  coeff_101_03 <= W4Data_03( 53 downto 52 );

  coeff_102_00 <= W4Data_00( 51 downto 50 );
  coeff_102_01 <= W4Data_01( 51 downto 50 );
  coeff_102_02 <= W4Data_02( 51 downto 50 );
  coeff_102_03 <= W4Data_03( 51 downto 50 );

  coeff_103_00 <= W4Data_00( 49 downto 48 );
  coeff_103_01 <= W4Data_01( 49 downto 48 );
  coeff_103_02 <= W4Data_02( 49 downto 48 );
  coeff_103_03 <= W4Data_03( 49 downto 48 );

  coeff_104_00 <= W4Data_00( 47 downto 46 );
  coeff_104_01 <= W4Data_01( 47 downto 46 );
  coeff_104_02 <= W4Data_02( 47 downto 46 );
  coeff_104_03 <= W4Data_03( 47 downto 46 );

  coeff_105_00 <= W4Data_00( 45 downto 44 );
  coeff_105_01 <= W4Data_01( 45 downto 44 );
  coeff_105_02 <= W4Data_02( 45 downto 44 );
  coeff_105_03 <= W4Data_03( 45 downto 44 );

  coeff_106_00 <= W4Data_00( 43 downto 42 );
  coeff_106_01 <= W4Data_01( 43 downto 42 );
  coeff_106_02 <= W4Data_02( 43 downto 42 );
  coeff_106_03 <= W4Data_03( 43 downto 42 );

  coeff_107_00 <= W4Data_00( 41 downto 40 );
  coeff_107_01 <= W4Data_01( 41 downto 40 );
  coeff_107_02 <= W4Data_02( 41 downto 40 );
  coeff_107_03 <= W4Data_03( 41 downto 40 );

  coeff_108_00 <= W4Data_00( 39 downto 38 );
  coeff_108_01 <= W4Data_01( 39 downto 38 );
  coeff_108_02 <= W4Data_02( 39 downto 38 );
  coeff_108_03 <= W4Data_03( 39 downto 38 );

  coeff_109_00 <= W4Data_00( 37 downto 36 );
  coeff_109_01 <= W4Data_01( 37 downto 36 );
  coeff_109_02 <= W4Data_02( 37 downto 36 );
  coeff_109_03 <= W4Data_03( 37 downto 36 );

  coeff_110_00 <= W4Data_00( 35 downto 34 );
  coeff_110_01 <= W4Data_01( 35 downto 34 );
  coeff_110_02 <= W4Data_02( 35 downto 34 );
  coeff_110_03 <= W4Data_03( 35 downto 34 );

  coeff_111_00 <= W4Data_00( 33 downto 32 );
  coeff_111_01 <= W4Data_01( 33 downto 32 );
  coeff_111_02 <= W4Data_02( 33 downto 32 );
  coeff_111_03 <= W4Data_03( 33 downto 32 );

  coeff_112_00 <= W4Data_00( 31 downto 30 );
  coeff_112_01 <= W4Data_01( 31 downto 30 );
  coeff_112_02 <= W4Data_02( 31 downto 30 );
  coeff_112_03 <= W4Data_03( 31 downto 30 );

  coeff_113_00 <= W4Data_00( 29 downto 28 );
  coeff_113_01 <= W4Data_01( 29 downto 28 );
  coeff_113_02 <= W4Data_02( 29 downto 28 );
  coeff_113_03 <= W4Data_03( 29 downto 28 );

  coeff_114_00 <= W4Data_00( 27 downto 26 );
  coeff_114_01 <= W4Data_01( 27 downto 26 );
  coeff_114_02 <= W4Data_02( 27 downto 26 );
  coeff_114_03 <= W4Data_03( 27 downto 26 );

  coeff_115_00 <= W4Data_00( 25 downto 24 );
  coeff_115_01 <= W4Data_01( 25 downto 24 );
  coeff_115_02 <= W4Data_02( 25 downto 24 );
  coeff_115_03 <= W4Data_03( 25 downto 24 );

  coeff_116_00 <= W4Data_00( 23 downto 22 );
  coeff_116_01 <= W4Data_01( 23 downto 22 );
  coeff_116_02 <= W4Data_02( 23 downto 22 );
  coeff_116_03 <= W4Data_03( 23 downto 22 );

  coeff_117_00 <= W4Data_00( 21 downto 20 );
  coeff_117_01 <= W4Data_01( 21 downto 20 );
  coeff_117_02 <= W4Data_02( 21 downto 20 );
  coeff_117_03 <= W4Data_03( 21 downto 20 );

  coeff_118_00 <= W4Data_00( 19 downto 18 );
  coeff_118_01 <= W4Data_01( 19 downto 18 );
  coeff_118_02 <= W4Data_02( 19 downto 18 );
  coeff_118_03 <= W4Data_03( 19 downto 18 );

  coeff_119_00 <= W4Data_00( 17 downto 16 );
  coeff_119_01 <= W4Data_01( 17 downto 16 );
  coeff_119_02 <= W4Data_02( 17 downto 16 );
  coeff_119_03 <= W4Data_03( 17 downto 16 );

  coeff_120_00 <= W4Data_00( 15 downto 14 );
  coeff_120_01 <= W4Data_01( 15 downto 14 );
  coeff_120_02 <= W4Data_02( 15 downto 14 );
  coeff_120_03 <= W4Data_03( 15 downto 14 );

  coeff_121_00 <= W4Data_00( 13 downto 12 );
  coeff_121_01 <= W4Data_01( 13 downto 12 );
  coeff_121_02 <= W4Data_02( 13 downto 12 );
  coeff_121_03 <= W4Data_03( 13 downto 12 );

  coeff_122_00 <= W4Data_00( 11 downto 10 );
  coeff_122_01 <= W4Data_01( 11 downto 10 );
  coeff_122_02 <= W4Data_02( 11 downto 10 );
  coeff_122_03 <= W4Data_03( 11 downto 10 );

  coeff_123_00 <= W4Data_00( 09 downto 08 );
  coeff_123_01 <= W4Data_01( 09 downto 08 );
  coeff_123_02 <= W4Data_02( 09 downto 08 );
  coeff_123_03 <= W4Data_03( 09 downto 08 );

  coeff_124_00 <= W4Data_00( 07 downto 06 );
  coeff_124_01 <= W4Data_01( 07 downto 06 );
  coeff_124_02 <= W4Data_02( 07 downto 06 );
  coeff_124_03 <= W4Data_03( 07 downto 06 );

  coeff_125_00 <= W4Data_00( 05 downto 04 );
  coeff_125_01 <= W4Data_01( 05 downto 04 );
  coeff_125_02 <= W4Data_02( 05 downto 04 );
  coeff_125_03 <= W4Data_03( 05 downto 04 );

  coeff_126_00 <= W4Data_00( 03 downto 02 );
  coeff_126_01 <= W4Data_01( 03 downto 02 );
  coeff_126_02 <= W4Data_02( 03 downto 02 );
  coeff_126_03 <= W4Data_03( 03 downto 02 );

  coeff_127_00 <= W4Data_00( 01 downto 00 );
  coeff_127_01 <= W4Data_01( 01 downto 00 );
  coeff_127_02 <= W4Data_02( 01 downto 00 );
  coeff_127_03 <= W4Data_03( 01 downto 00 );


---------------------------------------------------------------------------------
  CO9_p0 <= "0000" & CO9;
  CO9_p9 <= ("0000" & CO9) + 9;
  CO9_p18 <= ("0000" & CO9) + 18;
  CO9_p27 <= ("0000" & CO9) + 27;
  CO9_p36 <= ("0000" & CO9) + 36;
  CO9_p45 <= ("0000" & CO9) + 45;
  CO9_p54 <= ("0000" & CO9) + 54;
  CO9_p63 <= ("0000" & CO9) + 63;
  CO9_p72 <= ("0000" & CO9) + 72;
  CO9_p81 <= ("0000" & CO9) + 81;
  CO9_p90 <= ("0000" & CO9) + 90;
  CO9_p99 <= ("0000" & CO9) + 99;
  CO9_p108 <= ("0000" & CO9) + 108;
  CO9_p117 <= ("0000" & CO9) + 117;
  CO9_p126 <= ("0000" & CO9) + 126;
  CO9_p135 <= ("0000" & CO9) + 135;

  W4Addr <= 
    CO9_p0 when in3ChSel = "0000" else
    CO9_p9 when in3ChSel = "0001" else
    CO9_p18 when in3ChSel = "0010" else
    CO9_p27 when in3ChSel = "0011" else
    CO9_p36 when in3ChSel = "0100" else
    CO9_p45 when in3ChSel = "0101" else
    CO9_p54 when in3ChSel = "0110" else
    CO9_p63 when in3ChSel = "0111" else
    CO9_p72 when in3ChSel = "1000" else
    CO9_p81 when in3ChSel = "1001" else
    CO9_p90 when in3ChSel = "1010" else
    CO9_p99 when in3ChSel = "1011" else
    CO9_p108 when in3ChSel = "1100" else
    CO9_p117 when in3ChSel = "1101" else
    CO9_p126 when in3ChSel = "1110" else
    CO9_p135; -- when in3ChSel = "1111" else

--  CO9is0 <= '1' when (CO9 = "0000" and in3ChSel_dly = "1111") or CnnStart_dly = '1' else '0';
  CO9is0 <= '1' when CO9 = "0001" and in3ChSel = "0000" else '0';

--------------------------------------------------------------
process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                CnnStart_dly <= '0';
        elsif( CLK'event and CLK = '1' ) then
                CnnStart_dly <= CnnStart;
        end if;
end process;

---------------------------------------------------------------------------------
ROM_W4_00_inst : ROM_W4_00 port map (
  CLK => CLK,
  ADDR => W4Addr,
  DATA => W4Data_00 );

ROM_W4_01_inst : ROM_W4_01 port map (
  CLK => CLK,
  ADDR => W4Addr,
  DATA => W4Data_01 );

ROM_W4_02_inst : ROM_W4_02 port map (
  CLK => CLK,
  ADDR => W4Addr,
  DATA => W4Data_02 );

ROM_W4_03_inst : ROM_W4_03 port map (
  CLK => CLK,
  ADDR => W4Addr,
  DATA => W4Data_03 );

-------------------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    in3ChSel_dly <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    in3ChSel_dly <= in3ChSel;
  end if;
end process;

---------------------------------------------------------------------------------
dotpro_000_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_000_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_000_00 );

dotpro_000_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_000_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_000_01 );

dotpro_000_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_000_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_000_02 );

dotpro_000_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_000_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_000_03 );

dotpro_001_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_001_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_001_00 );

dotpro_001_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_001_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_001_01 );

dotpro_001_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_001_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_001_02 );

dotpro_001_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_001_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_001_03 );

dotpro_002_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_002_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_002_00 );

dotpro_002_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_002_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_002_01 );

dotpro_002_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_002_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_002_02 );

dotpro_002_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_002_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_002_03 );

dotpro_003_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_003_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_003_00 );

dotpro_003_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_003_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_003_01 );

dotpro_003_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_003_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_003_02 );

dotpro_003_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_003_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_003_03 );

dotpro_004_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_004_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_004_00 );

dotpro_004_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_004_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_004_01 );

dotpro_004_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_004_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_004_02 );

dotpro_004_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_004_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_004_03 );

dotpro_005_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_005_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_005_00 );

dotpro_005_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_005_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_005_01 );

dotpro_005_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_005_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_005_02 );

dotpro_005_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_005_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_005_03 );

dotpro_006_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_006_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_006_00 );

dotpro_006_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_006_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_006_01 );

dotpro_006_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_006_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_006_02 );

dotpro_006_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_006_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_006_03 );

dotpro_007_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_007_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_007_00 );

dotpro_007_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_007_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_007_01 );

dotpro_007_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_007_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_007_02 );

dotpro_007_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_007_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_007_03 );

dotpro_008_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_008_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_008_00 );

dotpro_008_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_008_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_008_01 );

dotpro_008_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_008_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_008_02 );

dotpro_008_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_008_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_008_03 );

dotpro_009_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_009_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_009_00 );

dotpro_009_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_009_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_009_01 );

dotpro_009_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_009_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_009_02 );

dotpro_009_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_009_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_009_03 );

dotpro_010_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_010_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_010_00 );

dotpro_010_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_010_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_010_01 );

dotpro_010_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_010_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_010_02 );

dotpro_010_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_010_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_010_03 );

dotpro_011_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_011_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_011_00 );

dotpro_011_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_011_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_011_01 );

dotpro_011_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_011_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_011_02 );

dotpro_011_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_011_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_011_03 );

dotpro_012_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_012_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_012_00 );

dotpro_012_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_012_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_012_01 );

dotpro_012_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_012_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_012_02 );

dotpro_012_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_012_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_012_03 );

dotpro_013_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_013_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_013_00 );

dotpro_013_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_013_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_013_01 );

dotpro_013_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_013_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_013_02 );

dotpro_013_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_013_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_013_03 );

dotpro_014_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_014_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_014_00 );

dotpro_014_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_014_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_014_01 );

dotpro_014_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_014_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_014_02 );

dotpro_014_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_014_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_014_03 );

dotpro_015_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_015_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_015_00 );

dotpro_015_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_015_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_015_01 );

dotpro_015_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_015_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_015_02 );

dotpro_015_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_015_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_015_03 );

dotpro_016_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_016_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_016_00 );

dotpro_016_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_016_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_016_01 );

dotpro_016_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_016_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_016_02 );

dotpro_016_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_016_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_016_03 );

dotpro_017_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_017_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_017_00 );

dotpro_017_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_017_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_017_01 );

dotpro_017_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_017_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_017_02 );

dotpro_017_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_017_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_017_03 );

dotpro_018_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_018_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_018_00 );

dotpro_018_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_018_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_018_01 );

dotpro_018_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_018_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_018_02 );

dotpro_018_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_018_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_018_03 );

dotpro_019_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_019_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_019_00 );

dotpro_019_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_019_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_019_01 );

dotpro_019_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_019_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_019_02 );

dotpro_019_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_019_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_019_03 );

dotpro_020_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_020_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_020_00 );

dotpro_020_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_020_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_020_01 );

dotpro_020_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_020_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_020_02 );

dotpro_020_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_020_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_020_03 );

dotpro_021_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_021_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_021_00 );

dotpro_021_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_021_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_021_01 );

dotpro_021_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_021_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_021_02 );

dotpro_021_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_021_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_021_03 );

dotpro_022_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_022_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_022_00 );

dotpro_022_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_022_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_022_01 );

dotpro_022_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_022_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_022_02 );

dotpro_022_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_022_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_022_03 );

dotpro_023_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_023_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_023_00 );

dotpro_023_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_023_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_023_01 );

dotpro_023_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_023_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_023_02 );

dotpro_023_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_023_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_023_03 );

dotpro_024_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_024_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_024_00 );

dotpro_024_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_024_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_024_01 );

dotpro_024_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_024_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_024_02 );

dotpro_024_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_024_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_024_03 );

dotpro_025_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_025_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_025_00 );

dotpro_025_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_025_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_025_01 );

dotpro_025_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_025_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_025_02 );

dotpro_025_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_025_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_025_03 );

dotpro_026_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_026_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_026_00 );

dotpro_026_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_026_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_026_01 );

dotpro_026_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_026_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_026_02 );

dotpro_026_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_026_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_026_03 );

dotpro_027_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_027_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_027_00 );

dotpro_027_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_027_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_027_01 );

dotpro_027_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_027_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_027_02 );

dotpro_027_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_027_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_027_03 );

dotpro_028_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_028_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_028_00 );

dotpro_028_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_028_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_028_01 );

dotpro_028_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_028_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_028_02 );

dotpro_028_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_028_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_028_03 );

dotpro_029_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_029_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_029_00 );

dotpro_029_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_029_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_029_01 );

dotpro_029_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_029_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_029_02 );

dotpro_029_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_029_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_029_03 );

dotpro_030_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_030_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_030_00 );

dotpro_030_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_030_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_030_01 );

dotpro_030_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_030_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_030_02 );

dotpro_030_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_030_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_030_03 );

dotpro_031_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_031_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_031_00 );

dotpro_031_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_031_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_031_01 );

dotpro_031_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_031_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_031_02 );

dotpro_031_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_031_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_031_03 );

dotpro_032_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_032_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_032_00 );

dotpro_032_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_032_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_032_01 );

dotpro_032_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_032_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_032_02 );

dotpro_032_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_032_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_032_03 );

dotpro_033_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_033_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_033_00 );

dotpro_033_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_033_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_033_01 );

dotpro_033_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_033_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_033_02 );

dotpro_033_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_033_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_033_03 );

dotpro_034_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_034_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_034_00 );

dotpro_034_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_034_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_034_01 );

dotpro_034_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_034_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_034_02 );

dotpro_034_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_034_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_034_03 );

dotpro_035_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_035_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_035_00 );

dotpro_035_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_035_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_035_01 );

dotpro_035_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_035_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_035_02 );

dotpro_035_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_035_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_035_03 );

dotpro_036_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_036_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_036_00 );

dotpro_036_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_036_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_036_01 );

dotpro_036_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_036_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_036_02 );

dotpro_036_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_036_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_036_03 );

dotpro_037_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_037_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_037_00 );

dotpro_037_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_037_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_037_01 );

dotpro_037_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_037_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_037_02 );

dotpro_037_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_037_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_037_03 );

dotpro_038_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_038_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_038_00 );

dotpro_038_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_038_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_038_01 );

dotpro_038_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_038_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_038_02 );

dotpro_038_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_038_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_038_03 );

dotpro_039_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_039_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_039_00 );

dotpro_039_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_039_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_039_01 );

dotpro_039_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_039_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_039_02 );

dotpro_039_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_039_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_039_03 );

dotpro_040_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_040_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_040_00 );

dotpro_040_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_040_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_040_01 );

dotpro_040_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_040_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_040_02 );

dotpro_040_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_040_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_040_03 );

dotpro_041_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_041_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_041_00 );

dotpro_041_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_041_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_041_01 );

dotpro_041_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_041_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_041_02 );

dotpro_041_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_041_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_041_03 );

dotpro_042_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_042_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_042_00 );

dotpro_042_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_042_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_042_01 );

dotpro_042_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_042_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_042_02 );

dotpro_042_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_042_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_042_03 );

dotpro_043_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_043_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_043_00 );

dotpro_043_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_043_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_043_01 );

dotpro_043_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_043_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_043_02 );

dotpro_043_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_043_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_043_03 );

dotpro_044_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_044_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_044_00 );

dotpro_044_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_044_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_044_01 );

dotpro_044_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_044_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_044_02 );

dotpro_044_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_044_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_044_03 );

dotpro_045_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_045_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_045_00 );

dotpro_045_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_045_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_045_01 );

dotpro_045_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_045_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_045_02 );

dotpro_045_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_045_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_045_03 );

dotpro_046_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_046_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_046_00 );

dotpro_046_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_046_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_046_01 );

dotpro_046_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_046_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_046_02 );

dotpro_046_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_046_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_046_03 );

dotpro_047_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_047_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_047_00 );

dotpro_047_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_047_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_047_01 );

dotpro_047_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_047_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_047_02 );

dotpro_047_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_047_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_047_03 );

dotpro_048_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_048_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_048_00 );

dotpro_048_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_048_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_048_01 );

dotpro_048_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_048_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_048_02 );

dotpro_048_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_048_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_048_03 );

dotpro_049_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_049_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_049_00 );

dotpro_049_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_049_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_049_01 );

dotpro_049_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_049_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_049_02 );

dotpro_049_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_049_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_049_03 );

dotpro_050_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_050_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_050_00 );

dotpro_050_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_050_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_050_01 );

dotpro_050_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_050_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_050_02 );

dotpro_050_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_050_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_050_03 );

dotpro_051_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_051_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_051_00 );

dotpro_051_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_051_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_051_01 );

dotpro_051_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_051_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_051_02 );

dotpro_051_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_051_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_051_03 );

dotpro_052_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_052_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_052_00 );

dotpro_052_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_052_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_052_01 );

dotpro_052_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_052_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_052_02 );

dotpro_052_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_052_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_052_03 );

dotpro_053_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_053_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_053_00 );

dotpro_053_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_053_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_053_01 );

dotpro_053_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_053_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_053_02 );

dotpro_053_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_053_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_053_03 );

dotpro_054_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_054_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_054_00 );

dotpro_054_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_054_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_054_01 );

dotpro_054_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_054_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_054_02 );

dotpro_054_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_054_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_054_03 );

dotpro_055_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_055_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_055_00 );

dotpro_055_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_055_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_055_01 );

dotpro_055_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_055_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_055_02 );

dotpro_055_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_055_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_055_03 );

dotpro_056_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_056_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_056_00 );

dotpro_056_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_056_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_056_01 );

dotpro_056_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_056_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_056_02 );

dotpro_056_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_056_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_056_03 );

dotpro_057_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_057_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_057_00 );

dotpro_057_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_057_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_057_01 );

dotpro_057_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_057_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_057_02 );

dotpro_057_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_057_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_057_03 );

dotpro_058_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_058_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_058_00 );

dotpro_058_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_058_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_058_01 );

dotpro_058_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_058_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_058_02 );

dotpro_058_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_058_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_058_03 );

dotpro_059_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_059_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_059_00 );

dotpro_059_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_059_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_059_01 );

dotpro_059_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_059_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_059_02 );

dotpro_059_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_059_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_059_03 );

dotpro_060_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_060_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_060_00 );

dotpro_060_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_060_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_060_01 );

dotpro_060_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_060_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_060_02 );

dotpro_060_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_060_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_060_03 );

dotpro_061_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_061_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_061_00 );

dotpro_061_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_061_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_061_01 );

dotpro_061_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_061_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_061_02 );

dotpro_061_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_061_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_061_03 );

dotpro_062_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_062_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_062_00 );

dotpro_062_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_062_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_062_01 );

dotpro_062_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_062_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_062_02 );

dotpro_062_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_062_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_062_03 );

dotpro_063_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_063_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_063_00 );

dotpro_063_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_063_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_063_01 );

dotpro_063_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_063_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_063_02 );

dotpro_063_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_063_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_063_03 );

dotpro_064_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_064_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_064_00 );

dotpro_064_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_064_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_064_01 );

dotpro_064_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_064_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_064_02 );

dotpro_064_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_064_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_064_03 );

dotpro_065_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_065_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_065_00 );

dotpro_065_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_065_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_065_01 );

dotpro_065_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_065_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_065_02 );

dotpro_065_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_065_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_065_03 );

dotpro_066_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_066_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_066_00 );

dotpro_066_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_066_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_066_01 );

dotpro_066_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_066_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_066_02 );

dotpro_066_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_066_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_066_03 );

dotpro_067_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_067_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_067_00 );

dotpro_067_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_067_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_067_01 );

dotpro_067_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_067_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_067_02 );

dotpro_067_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_067_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_067_03 );

dotpro_068_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_068_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_068_00 );

dotpro_068_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_068_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_068_01 );

dotpro_068_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_068_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_068_02 );

dotpro_068_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_068_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_068_03 );

dotpro_069_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_069_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_069_00 );

dotpro_069_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_069_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_069_01 );

dotpro_069_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_069_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_069_02 );

dotpro_069_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_069_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_069_03 );

dotpro_070_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_070_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_070_00 );

dotpro_070_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_070_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_070_01 );

dotpro_070_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_070_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_070_02 );

dotpro_070_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_070_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_070_03 );

dotpro_071_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_071_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_071_00 );

dotpro_071_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_071_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_071_01 );

dotpro_071_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_071_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_071_02 );

dotpro_071_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_071_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_071_03 );

dotpro_072_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_072_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_072_00 );

dotpro_072_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_072_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_072_01 );

dotpro_072_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_072_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_072_02 );

dotpro_072_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_072_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_072_03 );

dotpro_073_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_073_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_073_00 );

dotpro_073_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_073_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_073_01 );

dotpro_073_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_073_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_073_02 );

dotpro_073_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_073_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_073_03 );

dotpro_074_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_074_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_074_00 );

dotpro_074_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_074_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_074_01 );

dotpro_074_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_074_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_074_02 );

dotpro_074_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_074_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_074_03 );

dotpro_075_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_075_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_075_00 );

dotpro_075_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_075_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_075_01 );

dotpro_075_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_075_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_075_02 );

dotpro_075_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_075_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_075_03 );

dotpro_076_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_076_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_076_00 );

dotpro_076_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_076_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_076_01 );

dotpro_076_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_076_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_076_02 );

dotpro_076_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_076_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_076_03 );

dotpro_077_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_077_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_077_00 );

dotpro_077_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_077_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_077_01 );

dotpro_077_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_077_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_077_02 );

dotpro_077_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_077_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_077_03 );

dotpro_078_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_078_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_078_00 );

dotpro_078_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_078_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_078_01 );

dotpro_078_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_078_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_078_02 );

dotpro_078_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_078_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_078_03 );

dotpro_079_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_079_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_079_00 );

dotpro_079_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_079_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_079_01 );

dotpro_079_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_079_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_079_02 );

dotpro_079_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_079_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_079_03 );

dotpro_080_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_080_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_080_00 );

dotpro_080_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_080_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_080_01 );

dotpro_080_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_080_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_080_02 );

dotpro_080_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_080_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_080_03 );

dotpro_081_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_081_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_081_00 );

dotpro_081_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_081_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_081_01 );

dotpro_081_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_081_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_081_02 );

dotpro_081_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_081_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_081_03 );

dotpro_082_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_082_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_082_00 );

dotpro_082_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_082_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_082_01 );

dotpro_082_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_082_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_082_02 );

dotpro_082_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_082_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_082_03 );

dotpro_083_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_083_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_083_00 );

dotpro_083_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_083_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_083_01 );

dotpro_083_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_083_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_083_02 );

dotpro_083_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_083_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_083_03 );

dotpro_084_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_084_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_084_00 );

dotpro_084_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_084_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_084_01 );

dotpro_084_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_084_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_084_02 );

dotpro_084_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_084_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_084_03 );

dotpro_085_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_085_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_085_00 );

dotpro_085_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_085_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_085_01 );

dotpro_085_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_085_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_085_02 );

dotpro_085_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_085_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_085_03 );

dotpro_086_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_086_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_086_00 );

dotpro_086_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_086_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_086_01 );

dotpro_086_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_086_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_086_02 );

dotpro_086_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_086_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_086_03 );

dotpro_087_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_087_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_087_00 );

dotpro_087_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_087_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_087_01 );

dotpro_087_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_087_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_087_02 );

dotpro_087_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_087_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_087_03 );

dotpro_088_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_088_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_088_00 );

dotpro_088_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_088_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_088_01 );

dotpro_088_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_088_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_088_02 );

dotpro_088_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_088_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_088_03 );

dotpro_089_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_089_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_089_00 );

dotpro_089_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_089_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_089_01 );

dotpro_089_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_089_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_089_02 );

dotpro_089_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_089_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_089_03 );

dotpro_090_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_090_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_090_00 );

dotpro_090_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_090_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_090_01 );

dotpro_090_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_090_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_090_02 );

dotpro_090_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_090_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_090_03 );

dotpro_091_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_091_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_091_00 );

dotpro_091_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_091_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_091_01 );

dotpro_091_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_091_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_091_02 );

dotpro_091_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_091_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_091_03 );

dotpro_092_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_092_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_092_00 );

dotpro_092_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_092_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_092_01 );

dotpro_092_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_092_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_092_02 );

dotpro_092_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_092_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_092_03 );

dotpro_093_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_093_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_093_00 );

dotpro_093_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_093_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_093_01 );

dotpro_093_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_093_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_093_02 );

dotpro_093_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_093_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_093_03 );

dotpro_094_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_094_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_094_00 );

dotpro_094_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_094_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_094_01 );

dotpro_094_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_094_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_094_02 );

dotpro_094_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_094_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_094_03 );

dotpro_095_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_095_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_095_00 );

dotpro_095_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_095_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_095_01 );

dotpro_095_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_095_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_095_02 );

dotpro_095_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_095_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_095_03 );

dotpro_096_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_096_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_096_00 );

dotpro_096_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_096_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_096_01 );

dotpro_096_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_096_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_096_02 );

dotpro_096_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_096_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_096_03 );

dotpro_097_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_097_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_097_00 );

dotpro_097_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_097_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_097_01 );

dotpro_097_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_097_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_097_02 );

dotpro_097_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_097_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_097_03 );

dotpro_098_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_098_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_098_00 );

dotpro_098_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_098_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_098_01 );

dotpro_098_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_098_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_098_02 );

dotpro_098_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_098_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_098_03 );

dotpro_099_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_099_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_099_00 );

dotpro_099_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_099_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_099_01 );

dotpro_099_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_099_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_099_02 );

dotpro_099_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_099_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_099_03 );

dotpro_100_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_100_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_100_00 );

dotpro_100_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_100_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_100_01 );

dotpro_100_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_100_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_100_02 );

dotpro_100_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_100_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_100_03 );

dotpro_101_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_101_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_101_00 );

dotpro_101_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_101_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_101_01 );

dotpro_101_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_101_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_101_02 );

dotpro_101_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_101_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_101_03 );

dotpro_102_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_102_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_102_00 );

dotpro_102_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_102_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_102_01 );

dotpro_102_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_102_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_102_02 );

dotpro_102_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_102_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_102_03 );

dotpro_103_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_103_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_103_00 );

dotpro_103_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_103_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_103_01 );

dotpro_103_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_103_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_103_02 );

dotpro_103_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_103_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_103_03 );

dotpro_104_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_104_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_104_00 );

dotpro_104_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_104_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_104_01 );

dotpro_104_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_104_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_104_02 );

dotpro_104_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_104_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_104_03 );

dotpro_105_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_105_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_105_00 );

dotpro_105_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_105_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_105_01 );

dotpro_105_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_105_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_105_02 );

dotpro_105_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_105_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_105_03 );

dotpro_106_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_106_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_106_00 );

dotpro_106_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_106_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_106_01 );

dotpro_106_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_106_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_106_02 );

dotpro_106_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_106_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_106_03 );

dotpro_107_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_107_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_107_00 );

dotpro_107_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_107_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_107_01 );

dotpro_107_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_107_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_107_02 );

dotpro_107_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_107_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_107_03 );

dotpro_108_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_108_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_108_00 );

dotpro_108_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_108_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_108_01 );

dotpro_108_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_108_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_108_02 );

dotpro_108_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_108_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_108_03 );

dotpro_109_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_109_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_109_00 );

dotpro_109_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_109_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_109_01 );

dotpro_109_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_109_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_109_02 );

dotpro_109_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_109_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_109_03 );

dotpro_110_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_110_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_110_00 );

dotpro_110_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_110_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_110_01 );

dotpro_110_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_110_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_110_02 );

dotpro_110_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_110_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_110_03 );

dotpro_111_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_111_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_111_00 );

dotpro_111_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_111_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_111_01 );

dotpro_111_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_111_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_111_02 );

dotpro_111_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_111_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_111_03 );

dotpro_112_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_112_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_112_00 );

dotpro_112_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_112_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_112_01 );

dotpro_112_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_112_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_112_02 );

dotpro_112_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_112_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_112_03 );

dotpro_113_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_113_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_113_00 );

dotpro_113_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_113_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_113_01 );

dotpro_113_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_113_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_113_02 );

dotpro_113_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_113_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_113_03 );

dotpro_114_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_114_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_114_00 );

dotpro_114_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_114_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_114_01 );

dotpro_114_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_114_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_114_02 );

dotpro_114_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_114_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_114_03 );

dotpro_115_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_115_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_115_00 );

dotpro_115_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_115_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_115_01 );

dotpro_115_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_115_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_115_02 );

dotpro_115_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_115_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_115_03 );

dotpro_116_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_116_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_116_00 );

dotpro_116_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_116_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_116_01 );

dotpro_116_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_116_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_116_02 );

dotpro_116_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_116_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_116_03 );

dotpro_117_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_117_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_117_00 );

dotpro_117_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_117_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_117_01 );

dotpro_117_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_117_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_117_02 );

dotpro_117_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_117_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_117_03 );

dotpro_118_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_118_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_118_00 );

dotpro_118_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_118_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_118_01 );

dotpro_118_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_118_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_118_02 );

dotpro_118_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_118_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_118_03 );

dotpro_119_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_119_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_119_00 );

dotpro_119_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_119_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_119_01 );

dotpro_119_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_119_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_119_02 );

dotpro_119_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_119_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_119_03 );

dotpro_120_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_120_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_120_00 );

dotpro_120_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_120_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_120_01 );

dotpro_120_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_120_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_120_02 );

dotpro_120_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_120_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_120_03 );

dotpro_121_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_121_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_121_00 );

dotpro_121_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_121_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_121_01 );

dotpro_121_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_121_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_121_02 );

dotpro_121_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_121_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_121_03 );

dotpro_122_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_122_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_122_00 );

dotpro_122_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_122_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_122_01 );

dotpro_122_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_122_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_122_02 );

dotpro_122_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_122_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_122_03 );

dotpro_123_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_123_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_123_00 );

dotpro_123_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_123_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_123_01 );

dotpro_123_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_123_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_123_02 );

dotpro_123_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_123_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_123_03 );

dotpro_124_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_124_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_124_00 );

dotpro_124_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_124_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_124_01 );

dotpro_124_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_124_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_124_02 );

dotpro_124_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_124_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_124_03 );

dotpro_125_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_125_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_125_00 );

dotpro_125_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_125_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_125_01 );

dotpro_125_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_125_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_125_02 );

dotpro_125_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_125_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_125_03 );

dotpro_126_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_126_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_126_00 );

dotpro_126_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_126_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_126_01 );

dotpro_126_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_126_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_126_02 );

dotpro_126_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_126_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_126_03 );

dotpro_127_00 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_00,
  MatRomData => coeff_127_00,
  CO9is0 => CO9is0,
  MATMULOUT => mac_127_00 );

dotpro_127_01 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_01,
  MatRomData => coeff_127_01,
  CO9is0 => CO9is0,
  MATMULOUT => mac_127_01 );

dotpro_127_02 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_02,
  MatRomData => coeff_127_02,
  CO9is0 => CO9is0,
  MATMULOUT => mac_127_02 );

dotpro_127_03 : dotpro_w4 port map (
  CLK => CLK,  
  RST_N => RST_N, 
  PixRomData => inch_03,
  MatRomData => coeff_127_03,
  CO9is0 => CO9is0,
  MATMULOUT => mac_127_03 );


---------------------------------------------------------------------------------
  MacAll_000 <= mac_000_00 + mac_000_01 + mac_000_02 + mac_000_03;
  MacShift_000 <= MacAll_000 & "0000"; 

  MacAll_001 <= mac_001_00 + mac_001_01 + mac_001_02 + mac_001_03;
  MacShift_001 <= MacAll_001 & "0000"; 

  MacAll_002 <= mac_002_00 + mac_002_01 + mac_002_02 + mac_002_03;
  MacShift_002 <= MacAll_002 & "0000"; 

  MacAll_003 <= mac_003_00 + mac_003_01 + mac_003_02 + mac_003_03;
  MacShift_003 <= MacAll_003 & "0000"; 

  MacAll_004 <= mac_004_00 + mac_004_01 + mac_004_02 + mac_004_03;
  MacShift_004 <= MacAll_004 & "0000"; 

  MacAll_005 <= mac_005_00 + mac_005_01 + mac_005_02 + mac_005_03;
  MacShift_005 <= MacAll_005 & "0000"; 

  MacAll_006 <= mac_006_00 + mac_006_01 + mac_006_02 + mac_006_03;
  MacShift_006 <= MacAll_006 & "0000"; 

  MacAll_007 <= mac_007_00 + mac_007_01 + mac_007_02 + mac_007_03;
  MacShift_007 <= MacAll_007 & "0000"; 

  MacAll_008 <= mac_008_00 + mac_008_01 + mac_008_02 + mac_008_03;
  MacShift_008 <= MacAll_008 & "0000"; 

  MacAll_009 <= mac_009_00 + mac_009_01 + mac_009_02 + mac_009_03;
  MacShift_009 <= MacAll_009 & "0000"; 

  MacAll_010 <= mac_010_00 + mac_010_01 + mac_010_02 + mac_010_03;
  MacShift_010 <= MacAll_010 & "0000"; 

  MacAll_011 <= mac_011_00 + mac_011_01 + mac_011_02 + mac_011_03;
  MacShift_011 <= MacAll_011 & "0000"; 

  MacAll_012 <= mac_012_00 + mac_012_01 + mac_012_02 + mac_012_03;
  MacShift_012 <= MacAll_012 & "0000"; 

  MacAll_013 <= mac_013_00 + mac_013_01 + mac_013_02 + mac_013_03;
  MacShift_013 <= MacAll_013 & "0000"; 

  MacAll_014 <= mac_014_00 + mac_014_01 + mac_014_02 + mac_014_03;
  MacShift_014 <= MacAll_014 & "0000"; 

  MacAll_015 <= mac_015_00 + mac_015_01 + mac_015_02 + mac_015_03;
  MacShift_015 <= MacAll_015 & "0000"; 

  MacAll_016 <= mac_016_00 + mac_016_01 + mac_016_02 + mac_016_03;
  MacShift_016 <= MacAll_016 & "0000"; 

  MacAll_017 <= mac_017_00 + mac_017_01 + mac_017_02 + mac_017_03;
  MacShift_017 <= MacAll_017 & "0000"; 

  MacAll_018 <= mac_018_00 + mac_018_01 + mac_018_02 + mac_018_03;
  MacShift_018 <= MacAll_018 & "0000"; 

  MacAll_019 <= mac_019_00 + mac_019_01 + mac_019_02 + mac_019_03;
  MacShift_019 <= MacAll_019 & "0000"; 

  MacAll_020 <= mac_020_00 + mac_020_01 + mac_020_02 + mac_020_03;
  MacShift_020 <= MacAll_020 & "0000"; 

  MacAll_021 <= mac_021_00 + mac_021_01 + mac_021_02 + mac_021_03;
  MacShift_021 <= MacAll_021 & "0000"; 

  MacAll_022 <= mac_022_00 + mac_022_01 + mac_022_02 + mac_022_03;
  MacShift_022 <= MacAll_022 & "0000"; 

  MacAll_023 <= mac_023_00 + mac_023_01 + mac_023_02 + mac_023_03;
  MacShift_023 <= MacAll_023 & "0000"; 

  MacAll_024 <= mac_024_00 + mac_024_01 + mac_024_02 + mac_024_03;
  MacShift_024 <= MacAll_024 & "0000"; 

  MacAll_025 <= mac_025_00 + mac_025_01 + mac_025_02 + mac_025_03;
  MacShift_025 <= MacAll_025 & "0000"; 

  MacAll_026 <= mac_026_00 + mac_026_01 + mac_026_02 + mac_026_03;
  MacShift_026 <= MacAll_026 & "0000"; 

  MacAll_027 <= mac_027_00 + mac_027_01 + mac_027_02 + mac_027_03;
  MacShift_027 <= MacAll_027 & "0000"; 

  MacAll_028 <= mac_028_00 + mac_028_01 + mac_028_02 + mac_028_03;
  MacShift_028 <= MacAll_028 & "0000"; 

  MacAll_029 <= mac_029_00 + mac_029_01 + mac_029_02 + mac_029_03;
  MacShift_029 <= MacAll_029 & "0000"; 

  MacAll_030 <= mac_030_00 + mac_030_01 + mac_030_02 + mac_030_03;
  MacShift_030 <= MacAll_030 & "0000"; 

  MacAll_031 <= mac_031_00 + mac_031_01 + mac_031_02 + mac_031_03;
  MacShift_031 <= MacAll_031 & "0000"; 

  MacAll_032 <= mac_032_00 + mac_032_01 + mac_032_02 + mac_032_03;
  MacShift_032 <= MacAll_032 & "0000"; 

  MacAll_033 <= mac_033_00 + mac_033_01 + mac_033_02 + mac_033_03;
  MacShift_033 <= MacAll_033 & "0000"; 

  MacAll_034 <= mac_034_00 + mac_034_01 + mac_034_02 + mac_034_03;
  MacShift_034 <= MacAll_034 & "0000"; 

  MacAll_035 <= mac_035_00 + mac_035_01 + mac_035_02 + mac_035_03;
  MacShift_035 <= MacAll_035 & "0000"; 

  MacAll_036 <= mac_036_00 + mac_036_01 + mac_036_02 + mac_036_03;
  MacShift_036 <= MacAll_036 & "0000"; 

  MacAll_037 <= mac_037_00 + mac_037_01 + mac_037_02 + mac_037_03;
  MacShift_037 <= MacAll_037 & "0000"; 

  MacAll_038 <= mac_038_00 + mac_038_01 + mac_038_02 + mac_038_03;
  MacShift_038 <= MacAll_038 & "0000"; 

  MacAll_039 <= mac_039_00 + mac_039_01 + mac_039_02 + mac_039_03;
  MacShift_039 <= MacAll_039 & "0000"; 

  MacAll_040 <= mac_040_00 + mac_040_01 + mac_040_02 + mac_040_03;
  MacShift_040 <= MacAll_040 & "0000"; 

  MacAll_041 <= mac_041_00 + mac_041_01 + mac_041_02 + mac_041_03;
  MacShift_041 <= MacAll_041 & "0000"; 

  MacAll_042 <= mac_042_00 + mac_042_01 + mac_042_02 + mac_042_03;
  MacShift_042 <= MacAll_042 & "0000"; 

  MacAll_043 <= mac_043_00 + mac_043_01 + mac_043_02 + mac_043_03;
  MacShift_043 <= MacAll_043 & "0000"; 

  MacAll_044 <= mac_044_00 + mac_044_01 + mac_044_02 + mac_044_03;
  MacShift_044 <= MacAll_044 & "0000"; 

  MacAll_045 <= mac_045_00 + mac_045_01 + mac_045_02 + mac_045_03;
  MacShift_045 <= MacAll_045 & "0000"; 

  MacAll_046 <= mac_046_00 + mac_046_01 + mac_046_02 + mac_046_03;
  MacShift_046 <= MacAll_046 & "0000"; 

  MacAll_047 <= mac_047_00 + mac_047_01 + mac_047_02 + mac_047_03;
  MacShift_047 <= MacAll_047 & "0000"; 

  MacAll_048 <= mac_048_00 + mac_048_01 + mac_048_02 + mac_048_03;
  MacShift_048 <= MacAll_048 & "0000"; 

  MacAll_049 <= mac_049_00 + mac_049_01 + mac_049_02 + mac_049_03;
  MacShift_049 <= MacAll_049 & "0000"; 

  MacAll_050 <= mac_050_00 + mac_050_01 + mac_050_02 + mac_050_03;
  MacShift_050 <= MacAll_050 & "0000"; 

  MacAll_051 <= mac_051_00 + mac_051_01 + mac_051_02 + mac_051_03;
  MacShift_051 <= MacAll_051 & "0000"; 

  MacAll_052 <= mac_052_00 + mac_052_01 + mac_052_02 + mac_052_03;
  MacShift_052 <= MacAll_052 & "0000"; 

  MacAll_053 <= mac_053_00 + mac_053_01 + mac_053_02 + mac_053_03;
  MacShift_053 <= MacAll_053 & "0000"; 

  MacAll_054 <= mac_054_00 + mac_054_01 + mac_054_02 + mac_054_03;
  MacShift_054 <= MacAll_054 & "0000"; 

  MacAll_055 <= mac_055_00 + mac_055_01 + mac_055_02 + mac_055_03;
  MacShift_055 <= MacAll_055 & "0000"; 

  MacAll_056 <= mac_056_00 + mac_056_01 + mac_056_02 + mac_056_03;
  MacShift_056 <= MacAll_056 & "0000"; 

  MacAll_057 <= mac_057_00 + mac_057_01 + mac_057_02 + mac_057_03;
  MacShift_057 <= MacAll_057 & "0000"; 

  MacAll_058 <= mac_058_00 + mac_058_01 + mac_058_02 + mac_058_03;
  MacShift_058 <= MacAll_058 & "0000"; 

  MacAll_059 <= mac_059_00 + mac_059_01 + mac_059_02 + mac_059_03;
  MacShift_059 <= MacAll_059 & "0000"; 

  MacAll_060 <= mac_060_00 + mac_060_01 + mac_060_02 + mac_060_03;
  MacShift_060 <= MacAll_060 & "0000"; 

  MacAll_061 <= mac_061_00 + mac_061_01 + mac_061_02 + mac_061_03;
  MacShift_061 <= MacAll_061 & "0000"; 

  MacAll_062 <= mac_062_00 + mac_062_01 + mac_062_02 + mac_062_03;
  MacShift_062 <= MacAll_062 & "0000"; 

  MacAll_063 <= mac_063_00 + mac_063_01 + mac_063_02 + mac_063_03;
  MacShift_063 <= MacAll_063 & "0000"; 

  MacAll_064 <= mac_064_00 + mac_064_01 + mac_064_02 + mac_064_03;
  MacShift_064 <= MacAll_064 & "0000"; 

  MacAll_065 <= mac_065_00 + mac_065_01 + mac_065_02 + mac_065_03;
  MacShift_065 <= MacAll_065 & "0000"; 

  MacAll_066 <= mac_066_00 + mac_066_01 + mac_066_02 + mac_066_03;
  MacShift_066 <= MacAll_066 & "0000"; 

  MacAll_067 <= mac_067_00 + mac_067_01 + mac_067_02 + mac_067_03;
  MacShift_067 <= MacAll_067 & "0000"; 

  MacAll_068 <= mac_068_00 + mac_068_01 + mac_068_02 + mac_068_03;
  MacShift_068 <= MacAll_068 & "0000"; 

  MacAll_069 <= mac_069_00 + mac_069_01 + mac_069_02 + mac_069_03;
  MacShift_069 <= MacAll_069 & "0000"; 

  MacAll_070 <= mac_070_00 + mac_070_01 + mac_070_02 + mac_070_03;
  MacShift_070 <= MacAll_070 & "0000"; 

  MacAll_071 <= mac_071_00 + mac_071_01 + mac_071_02 + mac_071_03;
  MacShift_071 <= MacAll_071 & "0000"; 

  MacAll_072 <= mac_072_00 + mac_072_01 + mac_072_02 + mac_072_03;
  MacShift_072 <= MacAll_072 & "0000"; 

  MacAll_073 <= mac_073_00 + mac_073_01 + mac_073_02 + mac_073_03;
  MacShift_073 <= MacAll_073 & "0000"; 

  MacAll_074 <= mac_074_00 + mac_074_01 + mac_074_02 + mac_074_03;
  MacShift_074 <= MacAll_074 & "0000"; 

  MacAll_075 <= mac_075_00 + mac_075_01 + mac_075_02 + mac_075_03;
  MacShift_075 <= MacAll_075 & "0000"; 

  MacAll_076 <= mac_076_00 + mac_076_01 + mac_076_02 + mac_076_03;
  MacShift_076 <= MacAll_076 & "0000"; 

  MacAll_077 <= mac_077_00 + mac_077_01 + mac_077_02 + mac_077_03;
  MacShift_077 <= MacAll_077 & "0000"; 

  MacAll_078 <= mac_078_00 + mac_078_01 + mac_078_02 + mac_078_03;
  MacShift_078 <= MacAll_078 & "0000"; 

  MacAll_079 <= mac_079_00 + mac_079_01 + mac_079_02 + mac_079_03;
  MacShift_079 <= MacAll_079 & "0000"; 

  MacAll_080 <= mac_080_00 + mac_080_01 + mac_080_02 + mac_080_03;
  MacShift_080 <= MacAll_080 & "0000"; 

  MacAll_081 <= mac_081_00 + mac_081_01 + mac_081_02 + mac_081_03;
  MacShift_081 <= MacAll_081 & "0000"; 

  MacAll_082 <= mac_082_00 + mac_082_01 + mac_082_02 + mac_082_03;
  MacShift_082 <= MacAll_082 & "0000"; 

  MacAll_083 <= mac_083_00 + mac_083_01 + mac_083_02 + mac_083_03;
  MacShift_083 <= MacAll_083 & "0000"; 

  MacAll_084 <= mac_084_00 + mac_084_01 + mac_084_02 + mac_084_03;
  MacShift_084 <= MacAll_084 & "0000"; 

  MacAll_085 <= mac_085_00 + mac_085_01 + mac_085_02 + mac_085_03;
  MacShift_085 <= MacAll_085 & "0000"; 

  MacAll_086 <= mac_086_00 + mac_086_01 + mac_086_02 + mac_086_03;
  MacShift_086 <= MacAll_086 & "0000"; 

  MacAll_087 <= mac_087_00 + mac_087_01 + mac_087_02 + mac_087_03;
  MacShift_087 <= MacAll_087 & "0000"; 

  MacAll_088 <= mac_088_00 + mac_088_01 + mac_088_02 + mac_088_03;
  MacShift_088 <= MacAll_088 & "0000"; 

  MacAll_089 <= mac_089_00 + mac_089_01 + mac_089_02 + mac_089_03;
  MacShift_089 <= MacAll_089 & "0000"; 

  MacAll_090 <= mac_090_00 + mac_090_01 + mac_090_02 + mac_090_03;
  MacShift_090 <= MacAll_090 & "0000"; 

  MacAll_091 <= mac_091_00 + mac_091_01 + mac_091_02 + mac_091_03;
  MacShift_091 <= MacAll_091 & "0000"; 

  MacAll_092 <= mac_092_00 + mac_092_01 + mac_092_02 + mac_092_03;
  MacShift_092 <= MacAll_092 & "0000"; 

  MacAll_093 <= mac_093_00 + mac_093_01 + mac_093_02 + mac_093_03;
  MacShift_093 <= MacAll_093 & "0000"; 

  MacAll_094 <= mac_094_00 + mac_094_01 + mac_094_02 + mac_094_03;
  MacShift_094 <= MacAll_094 & "0000"; 

  MacAll_095 <= mac_095_00 + mac_095_01 + mac_095_02 + mac_095_03;
  MacShift_095 <= MacAll_095 & "0000"; 

  MacAll_096 <= mac_096_00 + mac_096_01 + mac_096_02 + mac_096_03;
  MacShift_096 <= MacAll_096 & "0000"; 

  MacAll_097 <= mac_097_00 + mac_097_01 + mac_097_02 + mac_097_03;
  MacShift_097 <= MacAll_097 & "0000"; 

  MacAll_098 <= mac_098_00 + mac_098_01 + mac_098_02 + mac_098_03;
  MacShift_098 <= MacAll_098 & "0000"; 

  MacAll_099 <= mac_099_00 + mac_099_01 + mac_099_02 + mac_099_03;
  MacShift_099 <= MacAll_099 & "0000"; 

  MacAll_100 <= mac_100_00 + mac_100_01 + mac_100_02 + mac_100_03;
  MacShift_100 <= MacAll_100 & "0000"; 

  MacAll_101 <= mac_101_00 + mac_101_01 + mac_101_02 + mac_101_03;
  MacShift_101 <= MacAll_101 & "0000"; 

  MacAll_102 <= mac_102_00 + mac_102_01 + mac_102_02 + mac_102_03;
  MacShift_102 <= MacAll_102 & "0000"; 

  MacAll_103 <= mac_103_00 + mac_103_01 + mac_103_02 + mac_103_03;
  MacShift_103 <= MacAll_103 & "0000"; 

  MacAll_104 <= mac_104_00 + mac_104_01 + mac_104_02 + mac_104_03;
  MacShift_104 <= MacAll_104 & "0000"; 

  MacAll_105 <= mac_105_00 + mac_105_01 + mac_105_02 + mac_105_03;
  MacShift_105 <= MacAll_105 & "0000"; 

  MacAll_106 <= mac_106_00 + mac_106_01 + mac_106_02 + mac_106_03;
  MacShift_106 <= MacAll_106 & "0000"; 

  MacAll_107 <= mac_107_00 + mac_107_01 + mac_107_02 + mac_107_03;
  MacShift_107 <= MacAll_107 & "0000"; 

  MacAll_108 <= mac_108_00 + mac_108_01 + mac_108_02 + mac_108_03;
  MacShift_108 <= MacAll_108 & "0000"; 

  MacAll_109 <= mac_109_00 + mac_109_01 + mac_109_02 + mac_109_03;
  MacShift_109 <= MacAll_109 & "0000"; 

  MacAll_110 <= mac_110_00 + mac_110_01 + mac_110_02 + mac_110_03;
  MacShift_110 <= MacAll_110 & "0000"; 

  MacAll_111 <= mac_111_00 + mac_111_01 + mac_111_02 + mac_111_03;
  MacShift_111 <= MacAll_111 & "0000"; 

  MacAll_112 <= mac_112_00 + mac_112_01 + mac_112_02 + mac_112_03;
  MacShift_112 <= MacAll_112 & "0000"; 

  MacAll_113 <= mac_113_00 + mac_113_01 + mac_113_02 + mac_113_03;
  MacShift_113 <= MacAll_113 & "0000"; 

  MacAll_114 <= mac_114_00 + mac_114_01 + mac_114_02 + mac_114_03;
  MacShift_114 <= MacAll_114 & "0000"; 

  MacAll_115 <= mac_115_00 + mac_115_01 + mac_115_02 + mac_115_03;
  MacShift_115 <= MacAll_115 & "0000"; 

  MacAll_116 <= mac_116_00 + mac_116_01 + mac_116_02 + mac_116_03;
  MacShift_116 <= MacAll_116 & "0000"; 

  MacAll_117 <= mac_117_00 + mac_117_01 + mac_117_02 + mac_117_03;
  MacShift_117 <= MacAll_117 & "0000"; 

  MacAll_118 <= mac_118_00 + mac_118_01 + mac_118_02 + mac_118_03;
  MacShift_118 <= MacAll_118 & "0000"; 

  MacAll_119 <= mac_119_00 + mac_119_01 + mac_119_02 + mac_119_03;
  MacShift_119 <= MacAll_119 & "0000"; 

  MacAll_120 <= mac_120_00 + mac_120_01 + mac_120_02 + mac_120_03;
  MacShift_120 <= MacAll_120 & "0000"; 

  MacAll_121 <= mac_121_00 + mac_121_01 + mac_121_02 + mac_121_03;
  MacShift_121 <= MacAll_121 & "0000"; 

  MacAll_122 <= mac_122_00 + mac_122_01 + mac_122_02 + mac_122_03;
  MacShift_122 <= MacAll_122 & "0000"; 

  MacAll_123 <= mac_123_00 + mac_123_01 + mac_123_02 + mac_123_03;
  MacShift_123 <= MacAll_123 & "0000"; 

  MacAll_124 <= mac_124_00 + mac_124_01 + mac_124_02 + mac_124_03;
  MacShift_124 <= MacAll_124 & "0000"; 

  MacAll_125 <= mac_125_00 + mac_125_01 + mac_125_02 + mac_125_03;
  MacShift_125 <= MacAll_125 & "0000"; 

  MacAll_126 <= mac_126_00 + mac_126_01 + mac_126_02 + mac_126_03;
  MacShift_126 <= MacAll_126 & "0000"; 

  MacAll_127 <= mac_127_00 + mac_127_01 + mac_127_02 + mac_127_03;
  MacShift_127 <= MacAll_127 & "0000"; 

  BIAS_000 <= "111111111001111001";
  BIAS_001 <= "111111101110011110";
  BIAS_002 <= "111111111110010011";
  BIAS_003 <= "111111111110101001";
  BIAS_004 <= "111111110001110000";
  BIAS_005 <= "111111111110010000";
  BIAS_006 <= "111111111011100000";
  BIAS_007 <= "000000000100110100";
  BIAS_008 <= "111111111000000101";
  BIAS_009 <= "000000000101010010";
  BIAS_010 <= "111111111101111010";
  BIAS_011 <= "111111111111011001";
  BIAS_012 <= "000000001010001010";
  BIAS_013 <= "111111111001111010";
  BIAS_014 <= "000000000100000101";
  BIAS_015 <= "000000010110000101";
  BIAS_016 <= "111111110101101010";
  BIAS_017 <= "000000000000000101";
  BIAS_018 <= "111111111101010101";
  BIAS_019 <= "000000000101100101";
  BIAS_020 <= "111111110010001000";
  BIAS_021 <= "111111111101000011";
  BIAS_022 <= "111111111111001010";
  BIAS_023 <= "111111111011110010";
  BIAS_024 <= "111111110100011100";
  BIAS_025 <= "000000000110000111";
  BIAS_026 <= "000000001111000100";
  BIAS_027 <= "111111111010010110";
  BIAS_028 <= "111111111011101111";
  BIAS_029 <= "111111111000110111";
  BIAS_030 <= "000000000110101010";
  BIAS_031 <= "111111110111000100";
  BIAS_032 <= "111111111111100010";
  BIAS_033 <= "000000000111100111";
  BIAS_034 <= "111111111101100101";
  BIAS_035 <= "111111110111111110";
  BIAS_036 <= "000000001110110000";
  BIAS_037 <= "111111111010010011";
  BIAS_038 <= "111111111110011000";
  BIAS_039 <= "000000000101011000";
  BIAS_040 <= "111111111000000101";
  BIAS_041 <= "000000001010010101";
  BIAS_042 <= "000000000101100001";
  BIAS_043 <= "111111110100111001";
  BIAS_044 <= "111111111011111001";
  BIAS_045 <= "111111110101100111";
  BIAS_046 <= "111111101111000110";
  BIAS_047 <= "111111110000001000";
  BIAS_048 <= "111111110110111100";
  BIAS_049 <= "000000001010011111";
  BIAS_050 <= "000000000000110001";
  BIAS_051 <= "000000000001110111";
  BIAS_052 <= "000000000001101111";
  BIAS_053 <= "111111111001100111";
  BIAS_054 <= "111111111000000110";
  BIAS_055 <= "000000000101110101";
  BIAS_056 <= "111111110101000001";
  BIAS_057 <= "111111111011110111";
  BIAS_058 <= "000000000100110100";
  BIAS_059 <= "000000000000010000";
  BIAS_060 <= "111111110111101010";
  BIAS_061 <= "111111111100011110";
  BIAS_062 <= "111111111111001010";
  BIAS_063 <= "111111111001011111";
  BIAS_064 <= "111111110001011000";
  BIAS_065 <= "111111111010100110";
  BIAS_066 <= "111111111011101010";
  BIAS_067 <= "000000000110010100";
  BIAS_068 <= "111111101111111111";
  BIAS_069 <= "111111110110011001";
  BIAS_070 <= "000000000101011110";
  BIAS_071 <= "000000000010101100";
  BIAS_072 <= "111111111100010110";
  BIAS_073 <= "111111111010000101";
  BIAS_074 <= "111111111100010110";
  BIAS_075 <= "111111111000111111";
  BIAS_076 <= "000000000010011001";
  BIAS_077 <= "000000000001011001";
  BIAS_078 <= "111111110001110110";
  BIAS_079 <= "000000000110000110";
  BIAS_080 <= "111111111110000111";
  BIAS_081 <= "000000010000110010";
  BIAS_082 <= "111111111101111100";
  BIAS_083 <= "000000001110110001";
  BIAS_084 <= "000000000110001011";
  BIAS_085 <= "111111110100011011";
  BIAS_086 <= "000000000000110100";
  BIAS_087 <= "111111111011010100";
  BIAS_088 <= "111111111110000001";
  BIAS_089 <= "111111111001101011";
  BIAS_090 <= "111111110011111001";
  BIAS_091 <= "111111111011111101";
  BIAS_092 <= "111111101110000101";
  BIAS_093 <= "111111111000010111";
  BIAS_094 <= "111111110100011011";
  BIAS_095 <= "000000000001000101";
  BIAS_096 <= "000000000000011110";
  BIAS_097 <= "111111110010101010";
  BIAS_098 <= "111111110110100110";
  BIAS_099 <= "111111110101110001";
  BIAS_100 <= "111111110001000110";
  BIAS_101 <= "111111111011000101";
  BIAS_102 <= "111111111100000100";
  BIAS_103 <= "111111110001110101";
  BIAS_104 <= "111111110010101011";
  BIAS_105 <= "000000000010101100";
  BIAS_106 <= "111111111001110000";
  BIAS_107 <= "000000000010000011";
  BIAS_108 <= "000000000100101101";
  BIAS_109 <= "111111111111011010";
  BIAS_110 <= "000000011001111101";
  BIAS_111 <= "111111110011001101";
  BIAS_112 <= "000000001001011100";
  BIAS_113 <= "111111101100101100";
  BIAS_114 <= "111111111001001100";
  BIAS_115 <= "000000000011110101";
  BIAS_116 <= "111111111110110101";
  BIAS_117 <= "111111111101110100";
  BIAS_118 <= "111111111111101010";
  BIAS_119 <= "111111111011000000";
  BIAS_120 <= "000000000000001100";
  BIAS_121 <= "111111111011111001";
  BIAS_122 <= "111111111010110000";
  BIAS_123 <= "000000010000001010";
  BIAS_124 <= "000000000111110001";
  BIAS_125 <= "111111110110101101";
  BIAS_126 <= "111111111001100110";
  BIAS_127 <= "111111111001001110";

  plusBias_000 <= MacShift_000 + BIAS_000;
  plusBias_001 <= MacShift_001 + BIAS_001;
  plusBias_002 <= MacShift_002 + BIAS_002;
  plusBias_003 <= MacShift_003 + BIAS_003;
  plusBias_004 <= MacShift_004 + BIAS_004;
  plusBias_005 <= MacShift_005 + BIAS_005;
  plusBias_006 <= MacShift_006 + BIAS_006;
  plusBias_007 <= MacShift_007 + BIAS_007;
  plusBias_008 <= MacShift_008 + BIAS_008;
  plusBias_009 <= MacShift_009 + BIAS_009;
  plusBias_010 <= MacShift_010 + BIAS_010;
  plusBias_011 <= MacShift_011 + BIAS_011;
  plusBias_012 <= MacShift_012 + BIAS_012;
  plusBias_013 <= MacShift_013 + BIAS_013;
  plusBias_014 <= MacShift_014 + BIAS_014;
  plusBias_015 <= MacShift_015 + BIAS_015;
  plusBias_016 <= MacShift_016 + BIAS_016;
  plusBias_017 <= MacShift_017 + BIAS_017;
  plusBias_018 <= MacShift_018 + BIAS_018;
  plusBias_019 <= MacShift_019 + BIAS_019;
  plusBias_020 <= MacShift_020 + BIAS_020;
  plusBias_021 <= MacShift_021 + BIAS_021;
  plusBias_022 <= MacShift_022 + BIAS_022;
  plusBias_023 <= MacShift_023 + BIAS_023;
  plusBias_024 <= MacShift_024 + BIAS_024;
  plusBias_025 <= MacShift_025 + BIAS_025;
  plusBias_026 <= MacShift_026 + BIAS_026;
  plusBias_027 <= MacShift_027 + BIAS_027;
  plusBias_028 <= MacShift_028 + BIAS_028;
  plusBias_029 <= MacShift_029 + BIAS_029;
  plusBias_030 <= MacShift_030 + BIAS_030;
  plusBias_031 <= MacShift_031 + BIAS_031;
  plusBias_032 <= MacShift_032 + BIAS_032;
  plusBias_033 <= MacShift_033 + BIAS_033;
  plusBias_034 <= MacShift_034 + BIAS_034;
  plusBias_035 <= MacShift_035 + BIAS_035;
  plusBias_036 <= MacShift_036 + BIAS_036;
  plusBias_037 <= MacShift_037 + BIAS_037;
  plusBias_038 <= MacShift_038 + BIAS_038;
  plusBias_039 <= MacShift_039 + BIAS_039;
  plusBias_040 <= MacShift_040 + BIAS_040;
  plusBias_041 <= MacShift_041 + BIAS_041;
  plusBias_042 <= MacShift_042 + BIAS_042;
  plusBias_043 <= MacShift_043 + BIAS_043;
  plusBias_044 <= MacShift_044 + BIAS_044;
  plusBias_045 <= MacShift_045 + BIAS_045;
  plusBias_046 <= MacShift_046 + BIAS_046;
  plusBias_047 <= MacShift_047 + BIAS_047;
  plusBias_048 <= MacShift_048 + BIAS_048;
  plusBias_049 <= MacShift_049 + BIAS_049;
  plusBias_050 <= MacShift_050 + BIAS_050;
  plusBias_051 <= MacShift_051 + BIAS_051;
  plusBias_052 <= MacShift_052 + BIAS_052;
  plusBias_053 <= MacShift_053 + BIAS_053;
  plusBias_054 <= MacShift_054 + BIAS_054;
  plusBias_055 <= MacShift_055 + BIAS_055;
  plusBias_056 <= MacShift_056 + BIAS_056;
  plusBias_057 <= MacShift_057 + BIAS_057;
  plusBias_058 <= MacShift_058 + BIAS_058;
  plusBias_059 <= MacShift_059 + BIAS_059;
  plusBias_060 <= MacShift_060 + BIAS_060;
  plusBias_061 <= MacShift_061 + BIAS_061;
  plusBias_062 <= MacShift_062 + BIAS_062;
  plusBias_063 <= MacShift_063 + BIAS_063;
  plusBias_064 <= MacShift_064 + BIAS_064;
  plusBias_065 <= MacShift_065 + BIAS_065;
  plusBias_066 <= MacShift_066 + BIAS_066;
  plusBias_067 <= MacShift_067 + BIAS_067;
  plusBias_068 <= MacShift_068 + BIAS_068;
  plusBias_069 <= MacShift_069 + BIAS_069;
  plusBias_070 <= MacShift_070 + BIAS_070;
  plusBias_071 <= MacShift_071 + BIAS_071;
  plusBias_072 <= MacShift_072 + BIAS_072;
  plusBias_073 <= MacShift_073 + BIAS_073;
  plusBias_074 <= MacShift_074 + BIAS_074;
  plusBias_075 <= MacShift_075 + BIAS_075;
  plusBias_076 <= MacShift_076 + BIAS_076;
  plusBias_077 <= MacShift_077 + BIAS_077;
  plusBias_078 <= MacShift_078 + BIAS_078;
  plusBias_079 <= MacShift_079 + BIAS_079;
  plusBias_080 <= MacShift_080 + BIAS_080;
  plusBias_081 <= MacShift_081 + BIAS_081;
  plusBias_082 <= MacShift_082 + BIAS_082;
  plusBias_083 <= MacShift_083 + BIAS_083;
  plusBias_084 <= MacShift_084 + BIAS_084;
  plusBias_085 <= MacShift_085 + BIAS_085;
  plusBias_086 <= MacShift_086 + BIAS_086;
  plusBias_087 <= MacShift_087 + BIAS_087;
  plusBias_088 <= MacShift_088 + BIAS_088;
  plusBias_089 <= MacShift_089 + BIAS_089;
  plusBias_090 <= MacShift_090 + BIAS_090;
  plusBias_091 <= MacShift_091 + BIAS_091;
  plusBias_092 <= MacShift_092 + BIAS_092;
  plusBias_093 <= MacShift_093 + BIAS_093;
  plusBias_094 <= MacShift_094 + BIAS_094;
  plusBias_095 <= MacShift_095 + BIAS_095;
  plusBias_096 <= MacShift_096 + BIAS_096;
  plusBias_097 <= MacShift_097 + BIAS_097;
  plusBias_098 <= MacShift_098 + BIAS_098;
  plusBias_099 <= MacShift_099 + BIAS_099;
  plusBias_100 <= MacShift_100 + BIAS_100;
  plusBias_101 <= MacShift_101 + BIAS_101;
  plusBias_102 <= MacShift_102 + BIAS_102;
  plusBias_103 <= MacShift_103 + BIAS_103;
  plusBias_104 <= MacShift_104 + BIAS_104;
  plusBias_105 <= MacShift_105 + BIAS_105;
  plusBias_106 <= MacShift_106 + BIAS_106;
  plusBias_107 <= MacShift_107 + BIAS_107;
  plusBias_108 <= MacShift_108 + BIAS_108;
  plusBias_109 <= MacShift_109 + BIAS_109;
  plusBias_110 <= MacShift_110 + BIAS_110;
  plusBias_111 <= MacShift_111 + BIAS_111;
  plusBias_112 <= MacShift_112 + BIAS_112;
  plusBias_113 <= MacShift_113 + BIAS_113;
  plusBias_114 <= MacShift_114 + BIAS_114;
  plusBias_115 <= MacShift_115 + BIAS_115;
  plusBias_116 <= MacShift_116 + BIAS_116;
  plusBias_117 <= MacShift_117 + BIAS_117;
  plusBias_118 <= MacShift_118 + BIAS_118;
  plusBias_119 <= MacShift_119 + BIAS_119;
  plusBias_120 <= MacShift_120 + BIAS_120;
  plusBias_121 <= MacShift_121 + BIAS_121;
  plusBias_122 <= MacShift_122 + BIAS_122;
  plusBias_123 <= MacShift_123 + BIAS_123;
  plusBias_124 <= MacShift_124 + BIAS_124;
  plusBias_125 <= MacShift_125 + BIAS_125;
  plusBias_126 <= MacShift_126 + BIAS_126;
  plusBias_127 <= MacShift_127 + BIAS_127;

  truncate_000 <= plusBias_000(17 downto 8);
  truncate_001 <= plusBias_001(17 downto 8);
  truncate_002 <= plusBias_002(17 downto 8);
  truncate_003 <= plusBias_003(17 downto 8);
  truncate_004 <= plusBias_004(17 downto 8);
  truncate_005 <= plusBias_005(17 downto 8);
  truncate_006 <= plusBias_006(17 downto 8);
  truncate_007 <= plusBias_007(17 downto 8);
  truncate_008 <= plusBias_008(17 downto 8);
  truncate_009 <= plusBias_009(17 downto 8);
  truncate_010 <= plusBias_010(17 downto 8);
  truncate_011 <= plusBias_011(17 downto 8);
  truncate_012 <= plusBias_012(17 downto 8);
  truncate_013 <= plusBias_013(17 downto 8);
  truncate_014 <= plusBias_014(17 downto 8);
  truncate_015 <= plusBias_015(17 downto 8);
  truncate_016 <= plusBias_016(17 downto 8);
  truncate_017 <= plusBias_017(17 downto 8);
  truncate_018 <= plusBias_018(17 downto 8);
  truncate_019 <= plusBias_019(17 downto 8);
  truncate_020 <= plusBias_020(17 downto 8);
  truncate_021 <= plusBias_021(17 downto 8);
  truncate_022 <= plusBias_022(17 downto 8);
  truncate_023 <= plusBias_023(17 downto 8);
  truncate_024 <= plusBias_024(17 downto 8);
  truncate_025 <= plusBias_025(17 downto 8);
  truncate_026 <= plusBias_026(17 downto 8);
  truncate_027 <= plusBias_027(17 downto 8);
  truncate_028 <= plusBias_028(17 downto 8);
  truncate_029 <= plusBias_029(17 downto 8);
  truncate_030 <= plusBias_030(17 downto 8);
  truncate_031 <= plusBias_031(17 downto 8);
  truncate_032 <= plusBias_032(17 downto 8);
  truncate_033 <= plusBias_033(17 downto 8);
  truncate_034 <= plusBias_034(17 downto 8);
  truncate_035 <= plusBias_035(17 downto 8);
  truncate_036 <= plusBias_036(17 downto 8);
  truncate_037 <= plusBias_037(17 downto 8);
  truncate_038 <= plusBias_038(17 downto 8);
  truncate_039 <= plusBias_039(17 downto 8);
  truncate_040 <= plusBias_040(17 downto 8);
  truncate_041 <= plusBias_041(17 downto 8);
  truncate_042 <= plusBias_042(17 downto 8);
  truncate_043 <= plusBias_043(17 downto 8);
  truncate_044 <= plusBias_044(17 downto 8);
  truncate_045 <= plusBias_045(17 downto 8);
  truncate_046 <= plusBias_046(17 downto 8);
  truncate_047 <= plusBias_047(17 downto 8);
  truncate_048 <= plusBias_048(17 downto 8);
  truncate_049 <= plusBias_049(17 downto 8);
  truncate_050 <= plusBias_050(17 downto 8);
  truncate_051 <= plusBias_051(17 downto 8);
  truncate_052 <= plusBias_052(17 downto 8);
  truncate_053 <= plusBias_053(17 downto 8);
  truncate_054 <= plusBias_054(17 downto 8);
  truncate_055 <= plusBias_055(17 downto 8);
  truncate_056 <= plusBias_056(17 downto 8);
  truncate_057 <= plusBias_057(17 downto 8);
  truncate_058 <= plusBias_058(17 downto 8);
  truncate_059 <= plusBias_059(17 downto 8);
  truncate_060 <= plusBias_060(17 downto 8);
  truncate_061 <= plusBias_061(17 downto 8);
  truncate_062 <= plusBias_062(17 downto 8);
  truncate_063 <= plusBias_063(17 downto 8);
  truncate_064 <= plusBias_064(17 downto 8);
  truncate_065 <= plusBias_065(17 downto 8);
  truncate_066 <= plusBias_066(17 downto 8);
  truncate_067 <= plusBias_067(17 downto 8);
  truncate_068 <= plusBias_068(17 downto 8);
  truncate_069 <= plusBias_069(17 downto 8);
  truncate_070 <= plusBias_070(17 downto 8);
  truncate_071 <= plusBias_071(17 downto 8);
  truncate_072 <= plusBias_072(17 downto 8);
  truncate_073 <= plusBias_073(17 downto 8);
  truncate_074 <= plusBias_074(17 downto 8);
  truncate_075 <= plusBias_075(17 downto 8);
  truncate_076 <= plusBias_076(17 downto 8);
  truncate_077 <= plusBias_077(17 downto 8);
  truncate_078 <= plusBias_078(17 downto 8);
  truncate_079 <= plusBias_079(17 downto 8);
  truncate_080 <= plusBias_080(17 downto 8);
  truncate_081 <= plusBias_081(17 downto 8);
  truncate_082 <= plusBias_082(17 downto 8);
  truncate_083 <= plusBias_083(17 downto 8);
  truncate_084 <= plusBias_084(17 downto 8);
  truncate_085 <= plusBias_085(17 downto 8);
  truncate_086 <= plusBias_086(17 downto 8);
  truncate_087 <= plusBias_087(17 downto 8);
  truncate_088 <= plusBias_088(17 downto 8);
  truncate_089 <= plusBias_089(17 downto 8);
  truncate_090 <= plusBias_090(17 downto 8);
  truncate_091 <= plusBias_091(17 downto 8);
  truncate_092 <= plusBias_092(17 downto 8);
  truncate_093 <= plusBias_093(17 downto 8);
  truncate_094 <= plusBias_094(17 downto 8);
  truncate_095 <= plusBias_095(17 downto 8);
  truncate_096 <= plusBias_096(17 downto 8);
  truncate_097 <= plusBias_097(17 downto 8);
  truncate_098 <= plusBias_098(17 downto 8);
  truncate_099 <= plusBias_099(17 downto 8);
  truncate_100 <= plusBias_100(17 downto 8);
  truncate_101 <= plusBias_101(17 downto 8);
  truncate_102 <= plusBias_102(17 downto 8);
  truncate_103 <= plusBias_103(17 downto 8);
  truncate_104 <= plusBias_104(17 downto 8);
  truncate_105 <= plusBias_105(17 downto 8);
  truncate_106 <= plusBias_106(17 downto 8);
  truncate_107 <= plusBias_107(17 downto 8);
  truncate_108 <= plusBias_108(17 downto 8);
  truncate_109 <= plusBias_109(17 downto 8);
  truncate_110 <= plusBias_110(17 downto 8);
  truncate_111 <= plusBias_111(17 downto 8);
  truncate_112 <= plusBias_112(17 downto 8);
  truncate_113 <= plusBias_113(17 downto 8);
  truncate_114 <= plusBias_114(17 downto 8);
  truncate_115 <= plusBias_115(17 downto 8);
  truncate_116 <= plusBias_116(17 downto 8);
  truncate_117 <= plusBias_117(17 downto 8);
  truncate_118 <= plusBias_118(17 downto 8);
  truncate_119 <= plusBias_119(17 downto 8);
  truncate_120 <= plusBias_120(17 downto 8);
  truncate_121 <= plusBias_121(17 downto 8);
  truncate_122 <= plusBias_122(17 downto 8);
  truncate_123 <= plusBias_123(17 downto 8);
  truncate_124 <= plusBias_124(17 downto 8);
  truncate_125 <= plusBias_125(17 downto 8);
  truncate_126 <= plusBias_126(17 downto 8);
  truncate_127 <= plusBias_127(17 downto 8);

  Relu_000 <= (others => '0') when truncate_000 < 0 else
             (others => '1') when truncate_000 > 31 else
             truncate_000(4 downto 0);

  Relu_001 <= (others => '0') when truncate_001 < 0 else
             (others => '1') when truncate_001 > 31 else
             truncate_001(4 downto 0);

  Relu_002 <= (others => '0') when truncate_002 < 0 else
             (others => '1') when truncate_002 > 31 else
             truncate_002(4 downto 0);

  Relu_003 <= (others => '0') when truncate_003 < 0 else
             (others => '1') when truncate_003 > 31 else
             truncate_003(4 downto 0);

  Relu_004 <= (others => '0') when truncate_004 < 0 else
             (others => '1') when truncate_004 > 31 else
             truncate_004(4 downto 0);

  Relu_005 <= (others => '0') when truncate_005 < 0 else
             (others => '1') when truncate_005 > 31 else
             truncate_005(4 downto 0);

  Relu_006 <= (others => '0') when truncate_006 < 0 else
             (others => '1') when truncate_006 > 31 else
             truncate_006(4 downto 0);

  Relu_007 <= (others => '0') when truncate_007 < 0 else
             (others => '1') when truncate_007 > 31 else
             truncate_007(4 downto 0);

  Relu_008 <= (others => '0') when truncate_008 < 0 else
             (others => '1') when truncate_008 > 31 else
             truncate_008(4 downto 0);

  Relu_009 <= (others => '0') when truncate_009 < 0 else
             (others => '1') when truncate_009 > 31 else
             truncate_009(4 downto 0);

  Relu_010 <= (others => '0') when truncate_010 < 0 else
             (others => '1') when truncate_010 > 31 else
             truncate_010(4 downto 0);

  Relu_011 <= (others => '0') when truncate_011 < 0 else
             (others => '1') when truncate_011 > 31 else
             truncate_011(4 downto 0);

  Relu_012 <= (others => '0') when truncate_012 < 0 else
             (others => '1') when truncate_012 > 31 else
             truncate_012(4 downto 0);

  Relu_013 <= (others => '0') when truncate_013 < 0 else
             (others => '1') when truncate_013 > 31 else
             truncate_013(4 downto 0);

  Relu_014 <= (others => '0') when truncate_014 < 0 else
             (others => '1') when truncate_014 > 31 else
             truncate_014(4 downto 0);

  Relu_015 <= (others => '0') when truncate_015 < 0 else
             (others => '1') when truncate_015 > 31 else
             truncate_015(4 downto 0);

  Relu_016 <= (others => '0') when truncate_016 < 0 else
             (others => '1') when truncate_016 > 31 else
             truncate_016(4 downto 0);

  Relu_017 <= (others => '0') when truncate_017 < 0 else
             (others => '1') when truncate_017 > 31 else
             truncate_017(4 downto 0);

  Relu_018 <= (others => '0') when truncate_018 < 0 else
             (others => '1') when truncate_018 > 31 else
             truncate_018(4 downto 0);

  Relu_019 <= (others => '0') when truncate_019 < 0 else
             (others => '1') when truncate_019 > 31 else
             truncate_019(4 downto 0);

  Relu_020 <= (others => '0') when truncate_020 < 0 else
             (others => '1') when truncate_020 > 31 else
             truncate_020(4 downto 0);

  Relu_021 <= (others => '0') when truncate_021 < 0 else
             (others => '1') when truncate_021 > 31 else
             truncate_021(4 downto 0);

  Relu_022 <= (others => '0') when truncate_022 < 0 else
             (others => '1') when truncate_022 > 31 else
             truncate_022(4 downto 0);

  Relu_023 <= (others => '0') when truncate_023 < 0 else
             (others => '1') when truncate_023 > 31 else
             truncate_023(4 downto 0);

  Relu_024 <= (others => '0') when truncate_024 < 0 else
             (others => '1') when truncate_024 > 31 else
             truncate_024(4 downto 0);

  Relu_025 <= (others => '0') when truncate_025 < 0 else
             (others => '1') when truncate_025 > 31 else
             truncate_025(4 downto 0);

  Relu_026 <= (others => '0') when truncate_026 < 0 else
             (others => '1') when truncate_026 > 31 else
             truncate_026(4 downto 0);

  Relu_027 <= (others => '0') when truncate_027 < 0 else
             (others => '1') when truncate_027 > 31 else
             truncate_027(4 downto 0);

  Relu_028 <= (others => '0') when truncate_028 < 0 else
             (others => '1') when truncate_028 > 31 else
             truncate_028(4 downto 0);

  Relu_029 <= (others => '0') when truncate_029 < 0 else
             (others => '1') when truncate_029 > 31 else
             truncate_029(4 downto 0);

  Relu_030 <= (others => '0') when truncate_030 < 0 else
             (others => '1') when truncate_030 > 31 else
             truncate_030(4 downto 0);

  Relu_031 <= (others => '0') when truncate_031 < 0 else
             (others => '1') when truncate_031 > 31 else
             truncate_031(4 downto 0);

  Relu_032 <= (others => '0') when truncate_032 < 0 else
             (others => '1') when truncate_032 > 31 else
             truncate_032(4 downto 0);

  Relu_033 <= (others => '0') when truncate_033 < 0 else
             (others => '1') when truncate_033 > 31 else
             truncate_033(4 downto 0);

  Relu_034 <= (others => '0') when truncate_034 < 0 else
             (others => '1') when truncate_034 > 31 else
             truncate_034(4 downto 0);

  Relu_035 <= (others => '0') when truncate_035 < 0 else
             (others => '1') when truncate_035 > 31 else
             truncate_035(4 downto 0);

  Relu_036 <= (others => '0') when truncate_036 < 0 else
             (others => '1') when truncate_036 > 31 else
             truncate_036(4 downto 0);

  Relu_037 <= (others => '0') when truncate_037 < 0 else
             (others => '1') when truncate_037 > 31 else
             truncate_037(4 downto 0);

  Relu_038 <= (others => '0') when truncate_038 < 0 else
             (others => '1') when truncate_038 > 31 else
             truncate_038(4 downto 0);

  Relu_039 <= (others => '0') when truncate_039 < 0 else
             (others => '1') when truncate_039 > 31 else
             truncate_039(4 downto 0);

  Relu_040 <= (others => '0') when truncate_040 < 0 else
             (others => '1') when truncate_040 > 31 else
             truncate_040(4 downto 0);

  Relu_041 <= (others => '0') when truncate_041 < 0 else
             (others => '1') when truncate_041 > 31 else
             truncate_041(4 downto 0);

  Relu_042 <= (others => '0') when truncate_042 < 0 else
             (others => '1') when truncate_042 > 31 else
             truncate_042(4 downto 0);

  Relu_043 <= (others => '0') when truncate_043 < 0 else
             (others => '1') when truncate_043 > 31 else
             truncate_043(4 downto 0);

  Relu_044 <= (others => '0') when truncate_044 < 0 else
             (others => '1') when truncate_044 > 31 else
             truncate_044(4 downto 0);

  Relu_045 <= (others => '0') when truncate_045 < 0 else
             (others => '1') when truncate_045 > 31 else
             truncate_045(4 downto 0);

  Relu_046 <= (others => '0') when truncate_046 < 0 else
             (others => '1') when truncate_046 > 31 else
             truncate_046(4 downto 0);

  Relu_047 <= (others => '0') when truncate_047 < 0 else
             (others => '1') when truncate_047 > 31 else
             truncate_047(4 downto 0);

  Relu_048 <= (others => '0') when truncate_048 < 0 else
             (others => '1') when truncate_048 > 31 else
             truncate_048(4 downto 0);

  Relu_049 <= (others => '0') when truncate_049 < 0 else
             (others => '1') when truncate_049 > 31 else
             truncate_049(4 downto 0);

  Relu_050 <= (others => '0') when truncate_050 < 0 else
             (others => '1') when truncate_050 > 31 else
             truncate_050(4 downto 0);

  Relu_051 <= (others => '0') when truncate_051 < 0 else
             (others => '1') when truncate_051 > 31 else
             truncate_051(4 downto 0);

  Relu_052 <= (others => '0') when truncate_052 < 0 else
             (others => '1') when truncate_052 > 31 else
             truncate_052(4 downto 0);

  Relu_053 <= (others => '0') when truncate_053 < 0 else
             (others => '1') when truncate_053 > 31 else
             truncate_053(4 downto 0);

  Relu_054 <= (others => '0') when truncate_054 < 0 else
             (others => '1') when truncate_054 > 31 else
             truncate_054(4 downto 0);

  Relu_055 <= (others => '0') when truncate_055 < 0 else
             (others => '1') when truncate_055 > 31 else
             truncate_055(4 downto 0);

  Relu_056 <= (others => '0') when truncate_056 < 0 else
             (others => '1') when truncate_056 > 31 else
             truncate_056(4 downto 0);

  Relu_057 <= (others => '0') when truncate_057 < 0 else
             (others => '1') when truncate_057 > 31 else
             truncate_057(4 downto 0);

  Relu_058 <= (others => '0') when truncate_058 < 0 else
             (others => '1') when truncate_058 > 31 else
             truncate_058(4 downto 0);

  Relu_059 <= (others => '0') when truncate_059 < 0 else
             (others => '1') when truncate_059 > 31 else
             truncate_059(4 downto 0);

  Relu_060 <= (others => '0') when truncate_060 < 0 else
             (others => '1') when truncate_060 > 31 else
             truncate_060(4 downto 0);

  Relu_061 <= (others => '0') when truncate_061 < 0 else
             (others => '1') when truncate_061 > 31 else
             truncate_061(4 downto 0);

  Relu_062 <= (others => '0') when truncate_062 < 0 else
             (others => '1') when truncate_062 > 31 else
             truncate_062(4 downto 0);

  Relu_063 <= (others => '0') when truncate_063 < 0 else
             (others => '1') when truncate_063 > 31 else
             truncate_063(4 downto 0);

  Relu_064 <= (others => '0') when truncate_064 < 0 else
             (others => '1') when truncate_064 > 31 else
             truncate_064(4 downto 0);

  Relu_065 <= (others => '0') when truncate_065 < 0 else
             (others => '1') when truncate_065 > 31 else
             truncate_065(4 downto 0);

  Relu_066 <= (others => '0') when truncate_066 < 0 else
             (others => '1') when truncate_066 > 31 else
             truncate_066(4 downto 0);

  Relu_067 <= (others => '0') when truncate_067 < 0 else
             (others => '1') when truncate_067 > 31 else
             truncate_067(4 downto 0);

  Relu_068 <= (others => '0') when truncate_068 < 0 else
             (others => '1') when truncate_068 > 31 else
             truncate_068(4 downto 0);

  Relu_069 <= (others => '0') when truncate_069 < 0 else
             (others => '1') when truncate_069 > 31 else
             truncate_069(4 downto 0);

  Relu_070 <= (others => '0') when truncate_070 < 0 else
             (others => '1') when truncate_070 > 31 else
             truncate_070(4 downto 0);

  Relu_071 <= (others => '0') when truncate_071 < 0 else
             (others => '1') when truncate_071 > 31 else
             truncate_071(4 downto 0);

  Relu_072 <= (others => '0') when truncate_072 < 0 else
             (others => '1') when truncate_072 > 31 else
             truncate_072(4 downto 0);

  Relu_073 <= (others => '0') when truncate_073 < 0 else
             (others => '1') when truncate_073 > 31 else
             truncate_073(4 downto 0);

  Relu_074 <= (others => '0') when truncate_074 < 0 else
             (others => '1') when truncate_074 > 31 else
             truncate_074(4 downto 0);

  Relu_075 <= (others => '0') when truncate_075 < 0 else
             (others => '1') when truncate_075 > 31 else
             truncate_075(4 downto 0);

  Relu_076 <= (others => '0') when truncate_076 < 0 else
             (others => '1') when truncate_076 > 31 else
             truncate_076(4 downto 0);

  Relu_077 <= (others => '0') when truncate_077 < 0 else
             (others => '1') when truncate_077 > 31 else
             truncate_077(4 downto 0);

  Relu_078 <= (others => '0') when truncate_078 < 0 else
             (others => '1') when truncate_078 > 31 else
             truncate_078(4 downto 0);

  Relu_079 <= (others => '0') when truncate_079 < 0 else
             (others => '1') when truncate_079 > 31 else
             truncate_079(4 downto 0);

  Relu_080 <= (others => '0') when truncate_080 < 0 else
             (others => '1') when truncate_080 > 31 else
             truncate_080(4 downto 0);

  Relu_081 <= (others => '0') when truncate_081 < 0 else
             (others => '1') when truncate_081 > 31 else
             truncate_081(4 downto 0);

  Relu_082 <= (others => '0') when truncate_082 < 0 else
             (others => '1') when truncate_082 > 31 else
             truncate_082(4 downto 0);

  Relu_083 <= (others => '0') when truncate_083 < 0 else
             (others => '1') when truncate_083 > 31 else
             truncate_083(4 downto 0);

  Relu_084 <= (others => '0') when truncate_084 < 0 else
             (others => '1') when truncate_084 > 31 else
             truncate_084(4 downto 0);

  Relu_085 <= (others => '0') when truncate_085 < 0 else
             (others => '1') when truncate_085 > 31 else
             truncate_085(4 downto 0);

  Relu_086 <= (others => '0') when truncate_086 < 0 else
             (others => '1') when truncate_086 > 31 else
             truncate_086(4 downto 0);

  Relu_087 <= (others => '0') when truncate_087 < 0 else
             (others => '1') when truncate_087 > 31 else
             truncate_087(4 downto 0);

  Relu_088 <= (others => '0') when truncate_088 < 0 else
             (others => '1') when truncate_088 > 31 else
             truncate_088(4 downto 0);

  Relu_089 <= (others => '0') when truncate_089 < 0 else
             (others => '1') when truncate_089 > 31 else
             truncate_089(4 downto 0);

  Relu_090 <= (others => '0') when truncate_090 < 0 else
             (others => '1') when truncate_090 > 31 else
             truncate_090(4 downto 0);

  Relu_091 <= (others => '0') when truncate_091 < 0 else
             (others => '1') when truncate_091 > 31 else
             truncate_091(4 downto 0);

  Relu_092 <= (others => '0') when truncate_092 < 0 else
             (others => '1') when truncate_092 > 31 else
             truncate_092(4 downto 0);

  Relu_093 <= (others => '0') when truncate_093 < 0 else
             (others => '1') when truncate_093 > 31 else
             truncate_093(4 downto 0);

  Relu_094 <= (others => '0') when truncate_094 < 0 else
             (others => '1') when truncate_094 > 31 else
             truncate_094(4 downto 0);

  Relu_095 <= (others => '0') when truncate_095 < 0 else
             (others => '1') when truncate_095 > 31 else
             truncate_095(4 downto 0);

  Relu_096 <= (others => '0') when truncate_096 < 0 else
             (others => '1') when truncate_096 > 31 else
             truncate_096(4 downto 0);

  Relu_097 <= (others => '0') when truncate_097 < 0 else
             (others => '1') when truncate_097 > 31 else
             truncate_097(4 downto 0);

  Relu_098 <= (others => '0') when truncate_098 < 0 else
             (others => '1') when truncate_098 > 31 else
             truncate_098(4 downto 0);

  Relu_099 <= (others => '0') when truncate_099 < 0 else
             (others => '1') when truncate_099 > 31 else
             truncate_099(4 downto 0);

  Relu_100 <= (others => '0') when truncate_100 < 0 else
             (others => '1') when truncate_100 > 31 else
             truncate_100(4 downto 0);

  Relu_101 <= (others => '0') when truncate_101 < 0 else
             (others => '1') when truncate_101 > 31 else
             truncate_101(4 downto 0);

  Relu_102 <= (others => '0') when truncate_102 < 0 else
             (others => '1') when truncate_102 > 31 else
             truncate_102(4 downto 0);

  Relu_103 <= (others => '0') when truncate_103 < 0 else
             (others => '1') when truncate_103 > 31 else
             truncate_103(4 downto 0);

  Relu_104 <= (others => '0') when truncate_104 < 0 else
             (others => '1') when truncate_104 > 31 else
             truncate_104(4 downto 0);

  Relu_105 <= (others => '0') when truncate_105 < 0 else
             (others => '1') when truncate_105 > 31 else
             truncate_105(4 downto 0);

  Relu_106 <= (others => '0') when truncate_106 < 0 else
             (others => '1') when truncate_106 > 31 else
             truncate_106(4 downto 0);

  Relu_107 <= (others => '0') when truncate_107 < 0 else
             (others => '1') when truncate_107 > 31 else
             truncate_107(4 downto 0);

  Relu_108 <= (others => '0') when truncate_108 < 0 else
             (others => '1') when truncate_108 > 31 else
             truncate_108(4 downto 0);

  Relu_109 <= (others => '0') when truncate_109 < 0 else
             (others => '1') when truncate_109 > 31 else
             truncate_109(4 downto 0);

  Relu_110 <= (others => '0') when truncate_110 < 0 else
             (others => '1') when truncate_110 > 31 else
             truncate_110(4 downto 0);

  Relu_111 <= (others => '0') when truncate_111 < 0 else
             (others => '1') when truncate_111 > 31 else
             truncate_111(4 downto 0);

  Relu_112 <= (others => '0') when truncate_112 < 0 else
             (others => '1') when truncate_112 > 31 else
             truncate_112(4 downto 0);

  Relu_113 <= (others => '0') when truncate_113 < 0 else
             (others => '1') when truncate_113 > 31 else
             truncate_113(4 downto 0);

  Relu_114 <= (others => '0') when truncate_114 < 0 else
             (others => '1') when truncate_114 > 31 else
             truncate_114(4 downto 0);

  Relu_115 <= (others => '0') when truncate_115 < 0 else
             (others => '1') when truncate_115 > 31 else
             truncate_115(4 downto 0);

  Relu_116 <= (others => '0') when truncate_116 < 0 else
             (others => '1') when truncate_116 > 31 else
             truncate_116(4 downto 0);

  Relu_117 <= (others => '0') when truncate_117 < 0 else
             (others => '1') when truncate_117 > 31 else
             truncate_117(4 downto 0);

  Relu_118 <= (others => '0') when truncate_118 < 0 else
             (others => '1') when truncate_118 > 31 else
             truncate_118(4 downto 0);

  Relu_119 <= (others => '0') when truncate_119 < 0 else
             (others => '1') when truncate_119 > 31 else
             truncate_119(4 downto 0);

  Relu_120 <= (others => '0') when truncate_120 < 0 else
             (others => '1') when truncate_120 > 31 else
             truncate_120(4 downto 0);

  Relu_121 <= (others => '0') when truncate_121 < 0 else
             (others => '1') when truncate_121 > 31 else
             truncate_121(4 downto 0);

  Relu_122 <= (others => '0') when truncate_122 < 0 else
             (others => '1') when truncate_122 > 31 else
             truncate_122(4 downto 0);

  Relu_123 <= (others => '0') when truncate_123 < 0 else
             (others => '1') when truncate_123 > 31 else
             truncate_123(4 downto 0);

  Relu_124 <= (others => '0') when truncate_124 < 0 else
             (others => '1') when truncate_124 > 31 else
             truncate_124(4 downto 0);

  Relu_125 <= (others => '0') when truncate_125 < 0 else
             (others => '1') when truncate_125 > 31 else
             truncate_125(4 downto 0);

  Relu_126 <= (others => '0') when truncate_126 < 0 else
             (others => '1') when truncate_126 > 31 else
             truncate_126(4 downto 0);

  Relu_127 <= (others => '0') when truncate_127 < 0 else
             (others => '1') when truncate_127 > 31 else
             truncate_127(4 downto 0);


ReluAll <= 
  Relu_000 & Relu_001 & Relu_002 & Relu_003 & Relu_004 & Relu_005 & Relu_006 & Relu_007 & Relu_008 & Relu_009 &
  Relu_010 & Relu_011 & Relu_012 & Relu_013 & Relu_014 & Relu_015 & Relu_016 & Relu_017 & Relu_018 & Relu_019 &
  Relu_020 & Relu_021 & Relu_022 & Relu_023 & Relu_024 & Relu_025 & Relu_026 & Relu_027 & Relu_028 & Relu_029 &
  Relu_030 & Relu_031 & Relu_032 & Relu_033 & Relu_034 & Relu_035 & Relu_036 & Relu_037 & Relu_038 & Relu_039 &
  Relu_040 & Relu_041 & Relu_042 & Relu_043 & Relu_044 & Relu_045 & Relu_046 & Relu_047 & Relu_048 & Relu_049 &
  Relu_050 & Relu_051 & Relu_052 & Relu_053 & Relu_054 & Relu_055 & Relu_056 & Relu_057 & Relu_058 & Relu_059 &
  Relu_060 & Relu_061 & Relu_062 & Relu_063 & Relu_064 & Relu_065 & Relu_066 & Relu_067 & Relu_068 & Relu_069 &
  Relu_070 & Relu_071 & Relu_072 & Relu_073 & Relu_074 & Relu_075 & Relu_076 & Relu_077 & Relu_078 & Relu_079 &
  Relu_080 & Relu_081 & Relu_082 & Relu_083 & Relu_084 & Relu_085 & Relu_086 & Relu_087 & Relu_088 & Relu_089 &
  Relu_090 & Relu_091 & Relu_092 & Relu_093 & Relu_094 & Relu_095 & Relu_096 & Relu_097 & Relu_098 & Relu_099 &
  Relu_100 & Relu_101 & Relu_102 & Relu_103 & Relu_104 & Relu_105 & Relu_106 & Relu_107 & Relu_108 & Relu_109 &
  Relu_110 & Relu_111 & Relu_112 & Relu_113 & Relu_114 & Relu_115 & Relu_116 & Relu_117 & Relu_118 & Relu_119 &
  Relu_120 & Relu_121 & Relu_122 & Relu_123 & Relu_124 & Relu_125 & Relu_126 & Relu_127 ;

---------------------------------------------------------
pooling_lay4_inst : pooling_lay4 port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> ReluAll,
   WR_N	=> P_WR_N,
   WR_ADDR	=> P_WR_ADDR,
   Poolout	=> Poolout);

pool_00 <= Poolout( 639 downto 635 );
pool_01 <= Poolout( 634 downto 630 );
pool_02 <= Poolout( 9 downto 5 );
pool_03 <= Poolout( 4 downto 0 );

  LAY4OUT <= Poolout;

end RTL;

